@@ -1,9 +1,9 @@
-/* $Id: glu.h,v 1.8 1997/10/29 02:03:20 brianp Exp $ */
+/* $Id: glu.h,v 3.2 1998/07/26 01:36:27 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,29 +23,14 @@
 
 /*
  * $Log: glu.h,v $
- * Revision 1.8  1997/10/29 02:03:20  brianp
- * added WINGDIAPI, APIENTRY stuff (David Bucciarelli, v20 3dfx driver)
+ * Revision 3.2  1998/07/26 01:36:27  brianp
+ * changes for Windows compilation per Ted Jump
  *
- * Revision 1.7  1997/08/19 02:35:07  brianp
- * added some Macintosh-only pragmas (Miklos Fazekas)
+ * Revision 3.1  1998/06/23 00:33:08  brianp
+ * added some WIN32 APIENTRY, CALLBACK stuff (Eric Lassauge)
  *
- * Revision 1.6  1997/07/13 22:59:34  brianp
- * added const to viewport parameter of gluPickMatrix()
- *
- * Revision 1.5  1997/05/28 02:31:01  brianp
- * added a comment about typedefs
- *
- * Revision 1.4  1997/02/19 10:13:54  brianp
- * now test for __QUICKDRAW__ like for __BEOS__ (Randy Frank)
- *
- * Revision 1.3  1997/02/03 20:05:33  brianp
- * patches for BeOS
- *
- * Revision 1.2  1997/02/03 19:15:15  brianp
- * conditionally include glu_mangle.h
- *
- * Revision 1.1  1996/09/13 01:26:41  brianp
- * Initial revision
+ * Revision 3.0  1998/02/20 05:06:01  brianp
+ * initial rev
  *
  */
 
@@ -66,6 +51,18 @@ extern "C" {
 
 #include "GL/gl.h"
 
+	/* to facilitate clean DLL building ... */
+#if defined(__WIN32__) || defined(__CYGWIN32__)
+#if defined(_MSC_VER) && defined(BUILD_GLU32) /* tag specify we're building mesa as a DLL */
+#define GLUAPI __declspec(dllexport)
+#elif defined(_MSC_VER) && defined(_DLL) /* tag specifying we're building for DLL runtime support */
+#define GLUAPI __declspec(dllimport)
+#else /* for use with static link lib build of Win32 edition only */
+#define GLUAPI extern
+#endif /* _STATIC_MESA support */
+#else
+#define GLUAPI
+#endif /* WIN32 / CYGWIN32 bracket */
 
 #ifdef macintosh
 	#pragma enumsalwaysint on
@@ -215,36 +212,40 @@ typedef struct GLUnurbsObj GLUnurbsObj;
  *
  */
 
-extern void APIENTRY gluLookAt( GLdouble eyex, GLdouble eyey, GLdouble eyez,
-                                GLdouble centerx, GLdouble centery, GLdouble centerz,
+GLUAPI void APIENTRY gluLookAt( GLdouble eyex, GLdouble eyey, GLdouble eyez,
+                                GLdouble centerx, GLdouble centery,
+                                GLdouble centerz,
                                 GLdouble upx, GLdouble upy, GLdouble upz );
 
 
-extern void APIENTRY gluOrtho2D( GLdouble left, GLdouble right,
+GLUAPI void APIENTRY gluOrtho2D( GLdouble left, GLdouble right,
                                  GLdouble bottom, GLdouble top );
 
 
-extern void APIENTRY gluPerspective( GLdouble fovy, GLdouble aspect,
+GLUAPI void APIENTRY gluPerspective( GLdouble fovy, GLdouble aspect,
                                      GLdouble zNear, GLdouble zFar );
 
 
-extern void APIENTRY gluPickMatrix( GLdouble x, GLdouble y,
+GLUAPI void APIENTRY gluPickMatrix( GLdouble x, GLdouble y,
                                     GLdouble width, GLdouble height,
                                     const GLint viewport[4] );
 
-extern GLint APIENTRY gluProject( GLdouble objx, GLdouble objy, GLdouble objz,
+GLUAPI GLint APIENTRY gluProject( GLdouble objx, GLdouble objy, GLdouble objz,
                                   const GLdouble modelMatrix[16],
                                   const GLdouble projMatrix[16],
                                   const GLint viewport[4],
-                                  GLdouble *winx, GLdouble *winy, GLdouble *winz );
+                                  GLdouble *winx, GLdouble *winy,
+                                  GLdouble *winz );
 
-extern GLint APIENTRY gluUnProject( GLdouble winx, GLdouble winy, GLdouble winz,
+GLUAPI GLint APIENTRY gluUnProject( GLdouble winx, GLdouble winy,
+                                    GLdouble winz,
                                     const GLdouble modelMatrix[16],
                                     const GLdouble projMatrix[16],
                                     const GLint viewport[4],
-                                    GLdouble *objx, GLdouble *objy, GLdouble *objz );
+                                    GLdouble *objx, GLdouble *objy,
+                                    GLdouble *objz );
 
-extern const GLubyte* APIENTRY gluErrorString( GLenum errorCode );
+GLUAPI const GLubyte* APIENTRY gluErrorString( GLenum errorCode );
 
 
 
@@ -254,18 +255,19 @@ extern const GLubyte* APIENTRY gluErrorString( GLenum errorCode );
  *
  */
 
-extern GLint APIENTRY gluScaleImage( GLenum format,
+GLUAPI GLint APIENTRY gluScaleImage( GLenum format,
                                      GLint widthin, GLint heightin,
                                      GLenum typein, const void *datain,
                                      GLint widthout, GLint heightout,
                                      GLenum typeout, void *dataout );
 
-extern GLint APIENTRY gluBuild1DMipmaps( GLenum target, GLint components,
+GLUAPI GLint APIENTRY gluBuild1DMipmaps( GLenum target, GLint components,
                                          GLint width, GLenum format,
                                          GLenum type, const void *data );
 
-extern GLint APIENTRY gluBuild2DMipmaps( GLenum target, GLint components,
-                                         GLint width, GLint height, GLenum format,
+GLUAPI GLint APIENTRY gluBuild2DMipmaps( GLenum target, GLint components,
+                                         GLint width, GLint height,
+                                         GLenum format,
                                          GLenum type, const void *data );
 
 
@@ -276,40 +278,42 @@ extern GLint APIENTRY gluBuild2DMipmaps( GLenum target, GLint components,
  *
  */
 
-extern GLUquadricObj* APIENTRY gluNewQuadric( void );
+GLUAPI GLUquadricObj* APIENTRY gluNewQuadric( void );
 
-extern void APIENTRY gluDeleteQuadric( GLUquadricObj *state );
+GLUAPI void APIENTRY gluDeleteQuadric( GLUquadricObj *state );
 
-extern void APIENTRY gluQuadricDrawStyle( GLUquadricObj *quadObject,
+GLUAPI void APIENTRY gluQuadricDrawStyle( GLUquadricObj *quadObject,
                                           GLenum drawStyle );
 
-extern void APIENTRY gluQuadricOrientation( GLUquadricObj *quadObject,
+GLUAPI void APIENTRY gluQuadricOrientation( GLUquadricObj *quadObject,
                                             GLenum orientation );
 
-extern void APIENTRY gluQuadricNormals( GLUquadricObj *quadObject, GLenum normals );
+GLUAPI void APIENTRY gluQuadricNormals( GLUquadricObj *quadObject,
+                                        GLenum normals );
 
-extern void APIENTRY gluQuadricTexture( GLUquadricObj *quadObject,
+GLUAPI void APIENTRY gluQuadricTexture( GLUquadricObj *quadObject,
                                         GLboolean textureCoords );
 
-extern void APIENTRY gluQuadricCallback( GLUquadricObj *qobj,
-                                         GLenum which, void (*fn)() );
+GLUAPI void APIENTRY gluQuadricCallback( GLUquadricObj *qobj,
+                                         GLenum which, void (CALLBACK *fn)() );
 
-extern void APIENTRY gluCylinder( GLUquadricObj *qobj,
+GLUAPI void APIENTRY gluCylinder( GLUquadricObj *qobj,
                                   GLdouble baseRadius,
                                   GLdouble topRadius,
                                   GLdouble height,
                                   GLint slices, GLint stacks );
 
-extern void APIENTRY gluSphere( GLUquadricObj *qobj,
+GLUAPI void APIENTRY gluSphere( GLUquadricObj *qobj,
                                 GLdouble radius, GLint slices, GLint stacks );
 
-extern void APIENTRY gluDisk( GLUquadricObj *qobj,
+GLUAPI void APIENTRY gluDisk( GLUquadricObj *qobj,
                               GLdouble innerRadius, GLdouble outerRadius,
                               GLint slices, GLint loops );
 
-extern void APIENTRY gluPartialDisk( GLUquadricObj *qobj, GLdouble innerRadius,
-                                     GLdouble outerRadius, GLint slices, GLint loops,
-                                     GLdouble startAngle, GLdouble sweepAngle );
+GLUAPI void APIENTRY gluPartialDisk( GLUquadricObj *qobj, GLdouble innerRadius,
+                                     GLdouble outerRadius, GLint slices,
+                                     GLint loops, GLdouble startAngle,
+                                     GLdouble sweepAngle );
 
 
 
@@ -319,34 +323,35 @@ extern void APIENTRY gluPartialDisk( GLUquadricObj *qobj, GLdouble innerRadius,
  *
  */
 
-extern GLUnurbsObj* APIENTRY gluNewNurbsRenderer( void );
+GLUAPI GLUnurbsObj* APIENTRY gluNewNurbsRenderer( void );
 
-extern void APIENTRY gluDeleteNurbsRenderer( GLUnurbsObj *nobj );
+GLUAPI void APIENTRY gluDeleteNurbsRenderer( GLUnurbsObj *nobj );
 
-extern void APIENTRY gluLoadSamplingMatrices( GLUnurbsObj *nobj,
+GLUAPI void APIENTRY gluLoadSamplingMatrices( GLUnurbsObj *nobj,
                                               const GLfloat modelMatrix[16],
                                               const GLfloat projMatrix[16],
                                               const GLint viewport[4] );
 
-extern void APIENTRY gluNurbsProperty( GLUnurbsObj *nobj, GLenum property,
+GLUAPI void APIENTRY gluNurbsProperty( GLUnurbsObj *nobj, GLenum property,
                                        GLfloat value );
 
-extern void APIENTRY gluGetNurbsProperty( GLUnurbsObj *nobj, GLenum property,
+GLUAPI void APIENTRY gluGetNurbsProperty( GLUnurbsObj *nobj, GLenum property,
                                           GLfloat *value );
 
-extern void APIENTRY gluBeginCurve( GLUnurbsObj *nobj );
+GLUAPI void APIENTRY gluBeginCurve( GLUnurbsObj *nobj );
 
-extern void APIENTRY gluEndCurve( GLUnurbsObj * nobj );
+GLUAPI void APIENTRY gluEndCurve( GLUnurbsObj * nobj );
 
-extern void APIENTRY gluNurbsCurve( GLUnurbsObj *nobj, GLint nknots, GLfloat *knot,
-                                    GLint stride, GLfloat *ctlarray, GLint order,
+GLUAPI void APIENTRY gluNurbsCurve( GLUnurbsObj *nobj, GLint nknots,
+                                    GLfloat *knot, GLint stride,
+                                    GLfloat *ctlarray, GLint order,
                                     GLenum type );
 
-extern void APIENTRY gluBeginSurface( GLUnurbsObj *nobj );
+GLUAPI void APIENTRY gluBeginSurface( GLUnurbsObj *nobj );
 
-extern void APIENTRY gluEndSurface( GLUnurbsObj * nobj );
+GLUAPI void APIENTRY gluEndSurface( GLUnurbsObj * nobj );
 
-extern void APIENTRY gluNurbsSurface( GLUnurbsObj *nobj,
+GLUAPI void APIENTRY gluNurbsSurface( GLUnurbsObj *nobj,
                                       GLint sknot_count, GLfloat *sknot,
                                       GLint tknot_count, GLfloat *tknot,
                                       GLint s_stride, GLint t_stride,
@@ -354,14 +359,15 @@ extern void APIENTRY gluNurbsSurface( GLUnurbsObj *nobj,
                                       GLint sorder, GLint torder,
                                       GLenum type );
 
-extern void APIENTRY gluBeginTrim( GLUnurbsObj *nobj );
+GLUAPI void APIENTRY gluBeginTrim( GLUnurbsObj *nobj );
 
-extern void APIENTRY gluEndTrim( GLUnurbsObj *nobj );
+GLUAPI void APIENTRY gluEndTrim( GLUnurbsObj *nobj );
 
-extern void APIENTRY gluPwlCurve( GLUnurbsObj *nobj, GLint count, GLfloat *array,
-                                  GLint stride, GLenum type );
+GLUAPI void APIENTRY gluPwlCurve( GLUnurbsObj *nobj, GLint count,
+                                  GLfloat *array, GLint stride, GLenum type );
 
-extern void APIENTRY gluNurbsCallback( GLUnurbsObj *nobj, GLenum which, void (*fn)() );
+GLUAPI void APIENTRY gluNurbsCallback( GLUnurbsObj *nobj, GLenum which,
+                                       void (CALLBACK *fn)() );
 
 
 
@@ -371,20 +377,20 @@ extern void APIENTRY gluNurbsCallback( GLUnurbsObj *nobj, GLenum which, void (*f
  *
  */
 
-extern GLUtriangulatorObj* APIENTRY gluNewTess( void );
+GLUAPI GLUtriangulatorObj* APIENTRY gluNewTess( void );
 
-extern void APIENTRY gluTessCallback( GLUtriangulatorObj *tobj, GLenum which,
-                                      void (*fn)() );
+GLUAPI void APIENTRY gluTessCallback( GLUtriangulatorObj *tobj, GLenum which,
+                                      void (CALLBACK *fn)() );
 
-extern void APIENTRY gluDeleteTess( GLUtriangulatorObj *tobj );
+GLUAPI void APIENTRY gluDeleteTess( GLUtriangulatorObj *tobj );
 
-extern void APIENTRY gluBeginPolygon( GLUtriangulatorObj *tobj );
+GLUAPI void APIENTRY gluBeginPolygon( GLUtriangulatorObj *tobj );
 
-extern void APIENTRY gluEndPolygon( GLUtriangulatorObj *tobj );
+GLUAPI void APIENTRY gluEndPolygon( GLUtriangulatorObj *tobj );
 
-extern void APIENTRY gluNextContour( GLUtriangulatorObj *tobj, GLenum type );
+GLUAPI void APIENTRY gluNextContour( GLUtriangulatorObj *tobj, GLenum type );
 
-extern void APIENTRY gluTessVertex( GLUtriangulatorObj *tobj, GLdouble v[3],
+GLUAPI void APIENTRY gluTessVertex( GLUtriangulatorObj *tobj, GLdouble v[3],
                                     void *data );
 
 
@@ -395,7 +401,7 @@ extern void APIENTRY gluTessVertex( GLUtriangulatorObj *tobj, GLdouble v[3],
  *
  */
 
-extern const GLubyte* APIENTRY gluGetString( GLenum name );
+GLUAPI const GLubyte* APIENTRY gluGetString( GLenum name );
 
 
 #if defined(__BEOS__) || defined(__QUICKDRAW__)
