@@ -1,4 +1,4 @@
-/* $Id: nurbsutl.c,v 1.5 1997/07/24 01:28:44 brianp Exp $ */
+/* $Id: nurbsutl.c,v 1.7 1998/07/26 02:07:59 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -23,6 +23,12 @@
 
 /*
  * $Log: nurbsutl.c,v $
+ * Revision 1.7  1998/07/26 02:07:59  brianp
+ * updated for Windows compilation per Ted Jump
+ *
+ * Revision 1.6  1997/10/29 02:02:20  brianp
+ * various MS Windows compiler changes (David Bucciarelli, v20 3dfx driver)
+ *
  * Revision 1.5  1997/07/24 01:28:44  brianp
  * changed precompiled header symbol from PCH to PC_HEADER
  *
@@ -83,8 +89,11 @@ test_knot(GLint nknots, GLfloat *knot, GLint order)
 	return GLU_NO_ERROR;
 }
 
-/* qsort function */
 static int
+/* qsort function */
+#ifdef WIN32
+__cdecl
+#endif
 knot_sort(const void *a, const void *b)
 {
 	GLfloat x,y;
