@@ -1,8 +1,8 @@
-/* $Id: quadric.c,v 1.6 1997/10/29 02:02:20 brianp Exp $ */
+/* $Id: quadric.c,v 1.16 1998/06/01 01:08:36 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
+ * Version:  2.6
  * Copyright (C) 1995-1997  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,36 @@
 
 /*
  * $Log: quadric.c,v $
+ * Revision 1.16  1998/06/01 01:08:36  brianp
+ * small update for Next/OpenStep from Alexander Mai
+ *
+ * Revision 1.15  1998/03/15 18:28:54  brianp
+ * reimplemented gluDisk() point and line mode
+ *
+ * Revision 1.14  1998/03/15 18:14:17  brianp
+ * fixed a compiler cast warning
+ *
+ * Revision 1.13  1998/02/07 14:28:34  brianp
+ * another change to gluQuadricCallback(), this time for StormC compiler
+ *
+ * Revision 1.12  1998/02/05 00:43:19  brianp
+ * Yes, still another change to gluQuadricCallback()!
+ *
+ * Revision 1.11  1998/02/04 00:27:43  brianp
+ * yet another change to gluQuadricCallback()!
+ *
+ * Revision 1.10  1998/02/04 00:23:23  brianp
+ * fixed CALLBACK problem in gluQuadricCallback() (Stephane Rehel)
+ *
+ * Revision 1.9  1998/02/04 00:20:09  brianp
+ * added missing (int) in ErrorFunc cast
+ *
+ * Revision 1.8  1998/01/16 03:37:51  brianp
+ * fixed another assignment warning in gluQuadricCallback()
+ *
+ * Revision 1.7  1998/01/16 03:35:26  brianp
+ * fixed Windows compilation warnings (Theodore Jump)
+ *
  * Revision 1.6  1997/10/29 02:02:20  brianp
  * various MS Windows compiler changes (David Bucciarelli, v20 3dfx driver)
  *
@@ -109,7 +139,7 @@ static void quadric_error( GLUquadricObj *qobj, GLenum error, const char *msg )
    }
    /* Print a message to stdout if MESA_DEBUG variable is defined */
    if (getenv("MESA_DEBUG")) {
-      fprintf(stderr,"GLUError: %s: %s\n", gluErrorString(error), msg );
+      fprintf(stderr,"GLUError: %s: %s\n", (char*) gluErrorString(error), msg);
    }
 }
 
@@ -181,8 +211,21 @@ void APIENTRY gluQuadricOrientation( GLUquadricObj *quadObject,
 void APIENTRY gluQuadricCallback( GLUquadricObj *qobj,
                                   GLenum which, void (CALLBACK *fn)() )
 {
+   /*
+    * UGH, this is a mess!  I thought ANSI was a standard.
+    */
    if (qobj && which==GLU_ERROR) {
-      qobj->ErrorFunc = fn;
+#ifdef __CYGWIN32__
+      qobj->ErrorFunc = (void(*)(int))fn;
+#elif defined(OPENSTEP)
+      qobj->ErrorFunc = (void(*)(GLenum))fn;
+#elif defined(_WIN32)
+      qobj->ErrorFunc = (void(CALLBACK*)(int))fn;
+#elif defined(__STORM__)
+      qobj->ErrorFunc = (void(CALLBACK*)(GLenum))fn;
+#else
+      qobj->ErrorFunc = (void(CALLBACK*)())fn;
+#endif
    }
 }
 
@@ -525,11 +568,14 @@ void APIENTRY gluDisk( GLUquadricObj *qobj,
                        GLdouble innerRadius, GLdouble outerRadius,
                        GLint slices, GLint loops )
 {
+   GLfloat da, dr;
+#if 0
    GLdouble a, da;
    GLfloat r, dr;
    GLfloat x, y;
    GLfloat r1, r2, dtc;
    GLint s, l;
+#endif
 
    /* Normal vectors */
    if (qobj->Normals!=GLU_NONE) {
@@ -544,20 +590,24 @@ void APIENTRY gluDisk( GLUquadricObj *qobj,
    da = 2.0*M_PI / slices;
    dr = (outerRadius-innerRadius) / (GLfloat) loops;
 
-   /* texture of a gluDisk is a cut out of the texture unit square */
-   /* x, y in [-outerRadius, +outerRadius]; s, t in [0, 1] (linear mapping) */
-   dtc = 2.0f * outerRadius;
-
    switch (qobj->DrawStyle) {
       case GLU_FILL:
-	 {
+      {
+         /* texture of a gluDisk is a cut out of the texture unit square
+          * x, y in [-outerRadius, +outerRadius]; s, t in [0, 1]
+          * (linear mapping)
+          */
+         GLfloat dtc = 2.0f * outerRadius;
          GLfloat sa,ca;
-         r1 = innerRadius;
-         for (l=0;l<loops;l++) {
-	    r2 = r1 + dr;
+         GLfloat r1 = innerRadius;
+         GLint l;
+         for (l=0; l<loops; l++) {
+	    GLfloat r2 = r1 + dr;
 	    if (qobj->Orientation==GLU_OUTSIDE) {
+               GLint s;
 	       glBegin( GL_QUAD_STRIP );
 	       for (s=0;s<=slices;s++) {
+                  GLfloat a;
 		  if (s==slices) a = 0.0;
 		  else  a = s * da;
 		  sa = sin(a); ca = cos(a);
@@ -569,8 +619,10 @@ void APIENTRY gluDisk( GLUquadricObj *qobj,
 	       glEnd();
 	    }
 	    else {
+               GLint s;
 	       glBegin( GL_QUAD_STRIP );
 	       for (s=slices;s>=0;s--) {
+                  GLfloat a;
 		  if (s==slices) a = 0.0;
 		  else  a = s * da;
 		  sa = sin(a); ca = cos(a);
@@ -583,57 +635,76 @@ void APIENTRY gluDisk( GLUquadricObj *qobj,
 	    }
 	    r1 = r2;
 	 }
-         }
          break;
+      }
       case GLU_LINE:
-	 /* draw rings */
-	 for (r=innerRadius; r<=outerRadius; r+=dr) {
+      {
+         GLint l, s;
+	 /* draw loops */
+         for (l=0; l<=loops; l++) {
+            GLfloat r = innerRadius + l * dr;
 	    glBegin( GL_LINE_LOOP );
-	    for (a=0.0; a<2.0*M_PI; a+=da) {
+            for (s=0; s<slices; s++) {
+               GLfloat a = s * da;
 	       glVertex2f( r*sin(a), r*cos(a) );
-	    }
+            }
 	    glEnd();
 	 }
 	 /* draw spokes */
-	 for (a=0.0; a<2.0*M_PI; a+=da) {
-	    x = sin(a);
-	    y = cos(a);
+         for (s=0; s<slices; s++) {
+            GLfloat a = s * da;
+	    GLfloat x = sin(a);
+	    GLfloat y = cos(a);
 	    glBegin( GL_LINE_STRIP );
-	    for (r=innerRadius; r<=outerRadius; r+=dr) {
+            for (l=0; l<=loops; l++) {
+               GLfloat r = innerRadius + l * dr;
 	       glVertex2f( r*x, r*y );
 	    }
 	    glEnd();
 	 }
 	 break;
+      }
       case GLU_POINT:
+      {
+         GLint s;
 	 glBegin( GL_POINTS );
-	 for (a=0.0; a<2.0*M_PI; a+=da) {
-	    x = sin(a);
-	    y = cos(a);
-	    for (r=innerRadius; r<=outerRadius; r+=dr) {
+         for (s=0; s<slices; s++) {
+            GLfloat a = s * da;
+	    GLfloat x = sin(a);
+	    GLfloat y = cos(a);
+            GLint l;
+            for (l=0; l<=loops; l++) {
+               GLfloat r = innerRadius * l * dr;
 	       glVertex2f( r*x, r*y );
 	    }
 	 }
 	 glEnd();
 	 break;
+      }
       case GLU_SILHOUETTE:
+      {
 	 if (innerRadius!=0.0) {
+            GLfloat a;
 	    glBegin( GL_LINE_LOOP );
 	    for (a=0.0; a<2.0*M_PI; a+=da) {
-	       x = innerRadius * sin(a);
-	       y = innerRadius * cos(a);
+	       GLfloat x = innerRadius * sin(a);
+	       GLfloat y = innerRadius * cos(a);
 	       glVertex2f( x, y );
 	    }
 	    glEnd();
 	 }
-	 glBegin( GL_LINE_LOOP );
-	 for (a=0; a<2.0*M_PI; a+=da) {
-	    x = outerRadius * sin(a);
-	    y = outerRadius * cos(a);
-	    glVertex2f( x, y );
-	 }
-	 glEnd();
+         {
+            GLfloat a;
+            glBegin( GL_LINE_LOOP );
+            for (a=0; a<2.0*M_PI; a+=da) {
+               GLfloat x = outerRadius * sin(a);
+               GLfloat y = outerRadius * cos(a);
+               glVertex2f( x, y );
+            }
+            glEnd();
+         }
 	 break;
+      }
       default:
          abort();
    }
