@@ -1,9 +1,9 @@
-/* $Id: alpha.h,v 1.1 1996/09/13 01:38:16 brianp Exp $ */
+/* $Id: alpha.h,v 3.1 1998/03/28 03:57:13 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.0
- * Copyright (C) 1995-1996  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,8 +23,11 @@
 
 /*
  * $Log: alpha.h,v $
- * Revision 1.1  1996/09/13 01:38:16  brianp
- * Initial revision
+ * Revision 3.1  1998/03/28 03:57:13  brianp
+ * added CONST macro to fix IRIX compilation problems
+ *
+ * Revision 3.0  1998/01/31 20:44:19  brianp
+ * initial rev
  *
  */
 
@@ -36,11 +39,11 @@
 #include "types.h"
 
 
-extern GLint gl_alpha_test( GLcontext* ctx,
-                            GLuint n, const GLubyte alpha[], GLubyte mask[] );
+extern GLint gl_alpha_test( const GLcontext *ctx, GLuint n,
+                            CONST GLubyte rgba[][4], GLubyte mask[] );
 
 
-extern void gl_AlphaFunc( GLcontext* ctx, GLenum func, GLclampf ref );
+extern void gl_AlphaFunc( GLcontext *ctx, GLenum func, GLclampf ref );
 
 
 #endif
