@@ -1,9 +1,9 @@
-/* $Id: clip.c,v 1.14 1997/07/24 01:24:45 brianp Exp $ */
+/* $Id: clip.c,v 3.5 1998/06/21 02:00:55 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,48 +23,23 @@
 
 /*
  * $Log: clip.c,v $
- * Revision 1.14  1997/07/24 01:24:45  brianp
- * changed precompiled header symbol from PCH to PC_HEADER
+ * Revision 3.5  1998/06/21 02:00:55  brianp
+ * cleaned up clip interpolation function code
  *
- * Revision 1.13  1997/05/28 03:23:48  brianp
- * added precompiled header (PCH) support
+ * Revision 3.4  1998/03/27 03:30:36  brianp
+ * fixed G++ warnings
  *
- * Revision 1.12  1997/04/02 03:10:06  brianp
- * call gl_analyze_modelview_matrix instead of gl_compute_modelview_inverse
+ * Revision 3.3  1998/02/20 04:50:09  brianp
+ * implemented GL_SGIS_multitexture
  *
- * Revision 1.11  1997/02/13 21:16:09  brianp
- * if too many vertices in polygon return VB_SIZE-1, not VB_SIZE
+ * Revision 3.2  1998/02/08 20:16:50  brianp
+ * removed unneeded headers
  *
- * Revision 1.10  1997/02/10 21:16:12  brianp
- * added checks in polygon clippers to prevent array overflows
+ * Revision 3.1  1998/02/03 23:46:40  brianp
+ * added 'space' parameter to clip interpolation functions
  *
- * Revision 1.9  1997/02/04 19:39:39  brianp
- * changed size of vlist2[] arrays to VB_SIZE per Randy Frank
- *
- * Revision 1.8  1996/12/02 20:10:07  brianp
- * changed the macros in gl_viewclip_polygon() to be like gl_viewclip_line()
- *
- * Revision 1.7  1996/10/29 02:55:02  brianp
- * fixed duplicate vertex bug in gl_viewclip_polygon()
- *
- * Revision 1.6  1996/10/07 23:48:33  brianp
- * changed temporaries to GLdouble in gl_viewclip_polygon()
- *
- * Revision 1.5  1996/10/03 01:43:45  brianp
- * changed INSIDE() macro in gl_viewclip_polygon() to work like other macros
- *
- * Revision 1.4  1996/10/03 01:36:33  brianp
- * changed COMPUTE_INTERSECTION macros in gl_viewclip_polygon to avoid
- * potential roundoff errors
- *
- * Revision 1.3  1996/09/27 01:24:23  brianp
- * removed unused variables
- *
- * Revision 1.2  1996/09/15 01:48:58  brianp
- * removed #define NULL 0
- *
- * Revision 1.1  1996/09/13 01:38:16  brianp
- * Initial revision
+ * Revision 3.0  1998/01/31 20:48:09  brianp
+ * initial rev
  *
  */
 
@@ -75,7 +50,6 @@
 #include <string.h>
 #include "clip.h"
 #include "context.h"
-#include "dlist.h"
 #include "macros.h"
 #include "matrix.h"
 #include "types.h"
@@ -90,10 +64,10 @@
 #define LINTERP( T, A, B )   ( (A) + (T) * ( (B) - (A) ) )
 
 
+/* Clipping coordinate spaces */
 #define EYE_SPACE 1
 #define CLIP_SPACE 2
 
-static GLuint Space;
 
 
 
@@ -107,16 +81,16 @@ static GLuint Space;
  *         in - index of array position corresponding to 'inside' vertex
  *         out - index of array position corresponding to 'outside' vertex
  */
-static void interpolate_aux( GLcontext* ctx,
-                             GLuint dst, GLfloat t, GLuint in, GLuint out )
+void gl_clip_interp_all( GLcontext *ctx, GLuint space,
+                         GLuint dst, GLfloat t, GLuint in, GLuint out )
 {
-   struct vertex_buffer* VB = ctx->VB;
+   struct vertex_buffer *VB = ctx->VB;
 
    if (ctx->ClipMask & CLIP_FCOLOR_BIT) {
-      VB->Fcolor[dst][0] = LINTERP( t, VB->Fcolor[in][0], VB->Fcolor[out][0] );
-      VB->Fcolor[dst][1] = LINTERP( t, VB->Fcolor[in][1], VB->Fcolor[out][1] );
-      VB->Fcolor[dst][2] = LINTERP( t, VB->Fcolor[in][2], VB->Fcolor[out][2] );
-      VB->Fcolor[dst][3] = LINTERP( t, VB->Fcolor[in][3], VB->Fcolor[out][3] );
+      VB->Fcolor[dst][0] = (GLint) LINTERP( t, VB->Fcolor[in][0], VB->Fcolor[out][0] );
+      VB->Fcolor[dst][1] = (GLint) LINTERP( t, VB->Fcolor[in][1], VB->Fcolor[out][1] );
+      VB->Fcolor[dst][2] = (GLint) LINTERP( t, VB->Fcolor[in][2], VB->Fcolor[out][2] );
+      VB->Fcolor[dst][3] = (GLint) LINTERP( t, VB->Fcolor[in][3], VB->Fcolor[out][3] );
    }
    else if (ctx->ClipMask & CLIP_FINDEX_BIT) {
       VB->Findex[dst] = (GLuint) (GLint) LINTERP( t, (GLfloat) VB->Findex[in],
@@ -124,10 +98,10 @@ static void interpolate_aux( GLcontext* ctx,
    }
 
    if (ctx->ClipMask & CLIP_BCOLOR_BIT) {
-      VB->Bcolor[dst][0] = LINTERP( t, VB->Bcolor[in][0], VB->Bcolor[out][0] );
-      VB->Bcolor[dst][1] = LINTERP( t, VB->Bcolor[in][1], VB->Bcolor[out][1] );
-      VB->Bcolor[dst][2] = LINTERP( t, VB->Bcolor[in][2], VB->Bcolor[out][2] );
-      VB->Bcolor[dst][3] = LINTERP( t, VB->Bcolor[in][3], VB->Bcolor[out][3] );
+      VB->Bcolor[dst][0] = (GLubyte) LINTERP( t, VB->Bcolor[in][0], VB->Bcolor[out][0] );
+      VB->Bcolor[dst][1] = (GLubyte) LINTERP( t, VB->Bcolor[in][1], VB->Bcolor[out][1] );
+      VB->Bcolor[dst][2] = (GLubyte) LINTERP( t, VB->Bcolor[in][2], VB->Bcolor[out][2] );
+      VB->Bcolor[dst][3] = (GLubyte) LINTERP( t, VB->Bcolor[in][3], VB->Bcolor[out][3] );
    }
    else if (ctx->ClipMask & CLIP_BINDEX_BIT) {
       VB->Bindex[dst] = (GLuint) (GLint) LINTERP( t, (GLfloat) VB->Bindex[in],
@@ -135,17 +109,75 @@ static void interpolate_aux( GLcontext* ctx,
    }
 
    if (ctx->ClipMask & CLIP_TEXTURE_BIT) {
-      /* TODO: is more sophisticated texture coord interpolation needed?? */
-      if (Space==CLIP_SPACE) {
+      GLuint i;
+      if (space==CLIP_SPACE) {
 	 /* also interpolate eye Z component */
 	 VB->Eye[dst][2] = LINTERP( t, VB->Eye[in][2], VB->Eye[out][2] );
       }
-      VB->TexCoord[dst][0] = LINTERP(t,VB->TexCoord[in][0],VB->TexCoord[out][0]);
-      VB->TexCoord[dst][1] = LINTERP(t,VB->TexCoord[in][1],VB->TexCoord[out][1]);
+      for (i=0;i<MAX_TEX_SETS;i++) {
+         VB->MultiTexCoord[i][dst][0] = LINTERP(t,VB->MultiTexCoord[i][in][0],VB->MultiTexCoord[i][out][0]);
+         VB->MultiTexCoord[i][dst][1] = LINTERP(t,VB->MultiTexCoord[i][in][1],VB->MultiTexCoord[i][out][1]);
+         VB->MultiTexCoord[i][dst][2] = LINTERP(t,VB->MultiTexCoord[i][in][2],VB->MultiTexCoord[i][out][2]);
+         VB->MultiTexCoord[i][dst][3] = LINTERP(t,VB->MultiTexCoord[i][in][3],VB->MultiTexCoord[i][out][3]);
+      }
+   }
+
+}
+
+
+/*
+ * Specialized version gl_clip_interp_all:  just do color and texcoords.
+ */
+void gl_clip_interp_color_tex( GLcontext *ctx, GLuint space,
+                               GLuint dst, GLfloat t, GLuint in, GLuint out )
+{
+   struct vertex_buffer *VB = ctx->VB;
+   VB->Fcolor[dst][0] = (GLint) LINTERP( t, VB->Fcolor[in][0], VB->Fcolor[out][0] );
+   VB->Fcolor[dst][1] = (GLint) LINTERP( t, VB->Fcolor[in][1], VB->Fcolor[out][1] );
+   VB->Fcolor[dst][2] = (GLint) LINTERP( t, VB->Fcolor[in][2], VB->Fcolor[out][2] );
+   VB->Fcolor[dst][3] = (GLint) LINTERP( t, VB->Fcolor[in][3], VB->Fcolor[out][3] );
+
+   VB->Eye[dst][2] = LINTERP( t, VB->Eye[in][2], VB->Eye[out][2] );
+   VB->TexCoord[dst][0] = LINTERP(t,VB->TexCoord[in][0],VB->TexCoord[out][0]);
+   VB->TexCoord[dst][1] = LINTERP(t,VB->TexCoord[in][1],VB->TexCoord[out][1]);
+   if (VB->TexCoordSize != 2) {
       VB->TexCoord[dst][2] = LINTERP(t,VB->TexCoord[in][2],VB->TexCoord[out][2]);
       VB->TexCoord[dst][3] = LINTERP(t,VB->TexCoord[in][3],VB->TexCoord[out][3]);
    }
+   (void) space;
+}
 
+
+/*
+ * Specialized version gl_clip_interp_all:  just do texcoords.
+ */
+void gl_clip_interp_tex( GLcontext *ctx, GLuint space,
+                         GLuint dst, GLfloat t, GLuint in, GLuint out )
+{
+   struct vertex_buffer *VB = ctx->VB;
+   VB->Eye[dst][2] = LINTERP( t, VB->Eye[in][2], VB->Eye[out][2] );
+   VB->TexCoord[dst][0] = LINTERP(t,VB->TexCoord[in][0],VB->TexCoord[out][0]);
+   VB->TexCoord[dst][1] = LINTERP(t,VB->TexCoord[in][1],VB->TexCoord[out][1]);
+   if (VB->TexCoordSize != 2) {
+      VB->TexCoord[dst][2] = LINTERP(t,VB->TexCoord[in][2],VB->TexCoord[out][2]);
+      VB->TexCoord[dst][3] = LINTERP(t,VB->TexCoord[in][3],VB->TexCoord[out][3]);
+   }
+   (void) space;
+}
+
+
+/*
+ * Specialized version gl_clip_interp_all:  just do color.
+ */
+void gl_clip_interp_color( GLcontext *ctx, GLuint space,
+                           GLuint dst, GLfloat t, GLuint in, GLuint out )
+{
+   struct vertex_buffer *VB = ctx->VB;
+   VB->Fcolor[dst][0] = (GLint) LINTERP( t, VB->Fcolor[in][0], VB->Fcolor[out][0] );
+   VB->Fcolor[dst][1] = (GLint) LINTERP( t, VB->Fcolor[in][1], VB->Fcolor[out][1] );
+   VB->Fcolor[dst][2] = (GLint) LINTERP( t, VB->Fcolor[in][2], VB->Fcolor[out][2] );
+   VB->Fcolor[dst][3] = (GLint) LINTERP( t, VB->Fcolor[in][3], VB->Fcolor[out][3] );
+   (void) space;
 }
 
 
@@ -238,7 +270,6 @@ GLuint gl_viewclip_line( GLcontext* ctx, GLuint *i, GLuint *j )
    GLfloat t, dx, dy, dz, dw;
    register GLuint ii, jj;
 
-   Space = CLIP_SPACE;
    ii = *i;
    jj = *j;
 
@@ -258,7 +289,7 @@ GLuint gl_viewclip_line( GLcontext* ctx, GLuint *i, GLuint *j )
 	 /* new vertex put in position VB->Free */			\
          COMPUTE_INTERSECTION( VB->Free, jj, ii )             		\
 	 if (ctx->ClipMask)						\
-	    interpolate_aux( ctx, VB->Free, t, jj, ii );		\
+            (*ctx->ClipInterpFunc)( ctx, CLIP_SPACE, VB->Free, t, jj, ii );\
 	 ii = VB->Free;							\
 	 VB->Free++;							\
 	 if (VB->Free==VB_SIZE)  VB->Free = 1;				\
@@ -270,7 +301,7 @@ GLuint gl_viewclip_line( GLcontext* ctx, GLuint *i, GLuint *j )
 	 /* new vertex put in position VB->Free */			\
          COMPUTE_INTERSECTION( VB->Free, ii, jj );            		\
 	 if (ctx->ClipMask)						\
-	    interpolate_aux( ctx, VB->Free, t, ii, jj );		\
+	    (*ctx->ClipInterpFunc)( ctx, CLIP_SPACE, VB->Free, t, ii, jj );\
 	 jj = VB->Free;							\
 	 VB->Free++;							\
 	 if (VB->Free==VB_SIZE)  VB->Free = 1;				\
@@ -418,8 +449,6 @@ GLuint gl_viewclip_polygon( GLcontext* ctx, GLuint n, GLuint vlist[] )
    GLuint n2;
    GLdouble dx, dy, dz, dw, t, neww;
 
-   Space = CLIP_SPACE;
-
 /*
  * We use 6 instances of this code to implement clipping against the
  * 6 sides of the view volume.  Prior to each we define the macros:
@@ -453,7 +482,7 @@ GLuint gl_viewclip_polygon( GLcontext* ctx, GLuint n, GLuint vlist[] )
 	    if (t>0.0) {						\
 	       /* interpolate aux info using the value of t */		\
 	       if (ctx->ClipMask)					\
-		  interpolate_aux( ctx, VB->Free, t, currj, prevj );	\
+		  (*ctx->ClipInterpFunc)( ctx, CLIP_SPACE, VB->Free, t, currj, prevj ); \
 	       VB->Edgeflag[VB->Free] = VB->Edgeflag[prevj];		\
 	       /* output new point */					\
 	       OUTLIST[OUTCOUNT] = VB->Free;				\
@@ -474,7 +503,7 @@ GLuint gl_viewclip_polygon( GLcontext* ctx, GLuint n, GLuint vlist[] )
 	    if (t>0.0) {						\
 	       /* interpolate aux info using the value of t */		\
 	       if (ctx->ClipMask)					\
-		  interpolate_aux( ctx, VB->Free, t, prevj, currj );	\
+		  (*ctx->ClipInterpFunc)( ctx, CLIP_SPACE, VB->Free, t, prevj, currj ); \
 	       VB->Edgeflag[VB->Free] = VB->Edgeflag[prevj];		\
 	       /* output new point */					\
 	       OUTLIST[OUTCOUNT] = VB->Free;				\
@@ -754,8 +783,6 @@ GLuint gl_userclip_line( GLcontext* ctx, GLuint *i, GLuint *j )
 
    GLuint p, ii, jj;
 
-   Space = EYE_SPACE;
-
    ii = *i;
    jj = *j;
 
@@ -795,7 +822,7 @@ GLuint gl_userclip_line( GLcontext* ctx, GLuint *i, GLuint *j )
 
 	       /* Interpolate colors, indexes, and/or texture coords */
 	       if (ctx->ClipMask)
-		  interpolate_aux( ctx, VB->Free, t, jj, ii );
+		  (*ctx->ClipInterpFunc)( ctx, EYE_SPACE, VB->Free, t, jj, ii );
 
 	       ii = VB->Free;
 	       VB->Free++;
@@ -826,7 +853,7 @@ GLuint gl_userclip_line( GLcontext* ctx, GLuint *i, GLuint *j )
 
 	       /* Interpolate colors, indexes, and/or texture coords */
 	       if (ctx->ClipMask)
-		  interpolate_aux( ctx, VB->Free, t, ii, jj );
+		  (*ctx->ClipInterpFunc)( ctx, EYE_SPACE, VB->Free, t, ii, jj );
 
 	       jj = VB->Free;
 	       VB->Free++;
@@ -865,8 +892,6 @@ GLuint gl_userclip_polygon( GLcontext* ctx, GLuint n, GLuint vlist[] )
    GLuint previ, prevj;
    GLuint p;
 
-   Space = EYE_SPACE;
-
    /* initialize input vertex list */
    incount = n;
    inlist = vlist;
@@ -922,7 +947,7 @@ GLuint gl_userclip_polygon( GLcontext* ctx, GLuint n, GLuint vlist[] )
 
 		  /* interpolate color, index, and/or texture coord */
 		  if (ctx->ClipMask) {
-		     interpolate_aux( ctx, VB->Free, t, currj, prevj);
+		     (*ctx->ClipInterpFunc)( ctx, EYE_SPACE, VB->Free, t, currj, prevj);
 		  }
 		  VB->Edgeflag[VB->Free] = VB->Edgeflag[prevj];
 
@@ -962,7 +987,7 @@ GLuint gl_userclip_polygon( GLcontext* ctx, GLuint n, GLuint vlist[] )
 
 		  /* interpolate color, index, and/or texture coord */
 		  if (ctx->ClipMask) {
-		     interpolate_aux( ctx, VB->Free, t, prevj, currj);
+		     (*ctx->ClipInterpFunc)( ctx, EYE_SPACE, VB->Free, t, prevj, currj);
 		  }
 		  VB->Edgeflag[VB->Free] = VB->Edgeflag[prevj];
 
