@@ -1,9 +1,9 @@
-/* $Id: config.h,v 1.6 1997/06/17 02:20:25 brianp Exp $ */
+/* $Id: config.h,v 3.5 1998/09/16 02:25:53 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.3
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,23 +23,23 @@
 
 /*
  * $Log: config.h,v $
- * Revision 1.6  1997/06/17 02:20:25  brianp
- * changed MAX_WIDTH to 1600 and MAX_HEIGHT to 1200
+ * Revision 3.5  1998/09/16 02:25:53  brianp
+ * removed some Amiga-specific stuff
  *
- * Revision 1.5  1997/05/28 03:27:19  brianp
- * set MAX_TEXTURE_LEVELS to 9 if compiling for 3Dfx driver (FX)
+ * Revision 3.4  1998/06/22 03:16:15  brianp
+ * added VB_MAX define and MITS test
  *
- * Revision 1.4  1997/01/08 20:54:02  brianp
- * added DITHER666 option from Michael Pichler
+ * Revision 3.3  1998/06/07 22:18:52  brianp
+ * implemented GL_EXT_multitexture extension
  *
- * Revision 1.3  1996/10/01 03:30:18  brianp
- * changed MAX_DEPTH to 0x00ffffff for 32-bit depth buffer
+ * Revision 3.2  1998/06/02 23:51:04  brianp
+ * added CHAN_BITS and GLchan type (for the future)
  *
- * Revision 1.2  1996/09/15 01:49:26  brianp
- * removed some junk
+ * Revision 3.1  1998/02/20 04:53:37  brianp
+ * implemented GL_SGIS_multitexture
  *
- * Revision 1.1  1996/09/13 01:38:16  brianp
- * Initial revision
+ * Revision 3.0  1998/01/31 20:48:53  brianp
+ * initial rev
  *
  */
 
@@ -55,6 +55,12 @@
 #define CONFIG_H
 
 
+/*
+ *
+ * OpenGL implementation limits
+ *
+ */
+
 
 /* Maximum modelview matrix stack depth: */
 #define MAX_MODELVIEW_STACK_DEPTH 32
@@ -71,9 +77,6 @@
 /* Maximum client attribute stack depth: */
 #define MAX_CLIENT_ATTRIB_STACK_DEPTH 16
 
-/* Maximum number of display lists: */
-#define MAX_DISPLAYLISTS 7000
-
 /* Maximum recursion depth of display list calls: */
 #define MAX_LIST_NESTING 64
 
@@ -119,18 +122,35 @@
 #define MAX_LINE_WIDTH 10.0
 #define LINE_WIDTH_GRANULARITY 1.0
 
+/* Max texture palette size */
+#define MAX_TEXTURE_PALETTE_SIZE 256
+
+/* Number of texture maps and environments - GL_EXT_multitexture */
+#define MAX_TEX_SETS 2
+
+/* Number of texture coordinate sets - GL_EXT_multitexture */
+#define MAX_TEX_COORD_SETS 2
 
 /* Maximum viewport size: */
-#ifdef AMIGA
-#  define MAX_WIDTH 640
-#  define MAX_HEIGHT 400
-#else
-#  define MAX_WIDTH 1600
-#  define MAX_HEIGHT 1200
-#endif
+#define MAX_WIDTH 1600
+#define MAX_HEIGHT 1200
 
 
 
+
+/*
+ *
+ * Mesa-specific parameters
+ *
+ */
+
+
+/*
+ * Bits per color channel (must be 8 at this time!)
+ */
+#define CHAN_BITS 8
+
+
 /*
  * Bits per accumulation buffer color component:  8 or 16
  */
@@ -145,9 +165,11 @@
 #if DEPTH_BITS==16
 #  define MAX_DEPTH 0xffff
 #  define DEPTH_SCALE 65535.0F
-#else
+#elif DEPTH_BITS==32
 #  define MAX_DEPTH 0x00ffffff
 #  define DEPTH_SCALE ((GLfloat) 0x00ffffff)
+#else
+   invalid value!
 #endif
 
 
@@ -158,9 +180,33 @@
 
 
 
-/***
- *** For X11 driver only:
- ***/
+/*
+ * Color channel component order
+ * (changes will almost certainly cause problems at this time)
+ */
+#define RCOMP 0
+#define GCOMP 1
+#define BCOMP 2
+#define ACOMP 3
+
+
+
+/*
+ * Vertex buffer size.  Must be a multiple of 12.
+ */
+#if defined(FX) && !defined(MITS)
+#  define VB_MAX 72   /* better performance */
+#else
+#  define VB_MAX 480
+#endif
+
+
+
+/*
+ *
+ * For X11 driver only:
+ *
+ */
 
 /*
  * When defined, use 6x6x6 dithering instead of 5x9x5.
@@ -169,4 +215,5 @@
 /*#define DITHER666*/
 
 
+
 #endif
