@@ -1,9 +1,9 @@
-/* $Id: dd.h,v 1.13 1997/10/29 02:23:54 brianp Exp $ */
+/* $Id: dd.h,v 3.11 1998/08/23 22:18:18 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.5
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,44 +23,41 @@
 
 /*
  * $Log: dd.h,v $
- * Revision 1.13  1997/10/29 02:23:54  brianp
- * added UseGlobalTexturePalette() dd function (David Bucciarelli v20 3dfx)
+ * Revision 3.11  1998/08/23 22:18:18  brianp
+ * added Driver.Viewport and Driver.DepthRange function pointers
  *
- * Revision 1.12  1997/10/16 02:27:53  brianp
- * more comments
+ * Revision 3.10  1998/06/07 22:19:31  brianp
+ * implemented GL_EXT_multitexture extension
  *
- * Revision 1.11  1997/10/14 00:07:40  brianp
- * added DavidB's fxmesa v19 changes
+ * Revision 3.9  1998/04/22 00:52:42  brianp
+ * added GLcontext parameter to driver ExtensionString()
  *
- * Revision 1.10  1997/09/29 23:26:50  brianp
- * new texture functions
+ * Revision 3.8  1998/04/01 02:59:42  brianp
+ * updated for v0.24 of 3Dfx/Glide driver
  *
- * Revision 1.9  1997/04/29 01:31:07  brianp
- * added RasterSetup() function to device driver
+ * Revision 3.7  1998/03/28 03:57:39  brianp
+ * added CONST macro to fix IRIX compilation problems
  *
- * Revision 1.8  1997/04/20 19:47:27  brianp
- * added RenderVB to device driver
+ * Revision 3.6  1998/03/10 01:28:09  brianp
+ * added a const keyword
  *
- * Revision 1.7  1997/04/20 16:31:08  brianp
- * added NearFar device driver function
+ * Revision 3.5  1998/02/21 01:01:07  brianp
+ * added GL_MAX_TEXTURES to GetParameteri() documentation
  *
- * Revision 1.6  1997/04/12 12:27:31  brianp
- * added QuadFunc and RectFunc
+ * Revision 3.4  1998/02/15 01:32:59  brianp
+ * updated driver bitmap function
  *
- * Revision 1.5  1997/03/21 01:57:07  brianp
- * added RendererString() function
+ * Revision 3.3  1998/02/07 14:43:35  brianp
+ * added GetParameteri() function
  *
- * Revision 1.4  1997/02/10 19:22:47  brianp
- * added device driver Error() function
+ * Revision 3.2  1998/02/01 15:23:52  brianp
+ * added ExtensionString() function to device driver
  *
- * Revision 1.3  1997/01/16 03:34:33  brianp
- * added preliminary texture mapping functions and cleaned up documentation
+ * Revision 3.1  1998/01/31 23:57:23  brianp
+ * removed ClearDepthBuffer and ClearColorAndDepth functions
  *
- * Revision 1.2  1996/11/13 03:51:59  brianp
- * updated comments
- *
- * Revision 1.1  1996/09/13 01:38:16  brianp
- * Initial revision
+ * Revision 3.0  1998/01/31 20:49:46  brianp
+ * initial rev
  *
  */
 
@@ -69,6 +66,13 @@
 #define DD_INCLUDED
 
 
+#include "macros.h"
+
+
+struct gl_pixelstore_attrib;
+
+
+
 /* THIS FILE ONLY INCLUDED BY types.h !!!!! */
 
 
@@ -140,6 +144,13 @@
 
 
 
+/* Used by the GetParameteri device driver function */
+#define DD_MAX_TEXTURE_SIZE          0
+#define DD_MAX_TEXTURES              1
+#define DD_MAX_TEXTURE_COORD_SETS    2
+#define DD_HAVE_HARDWARE_FOG         3
+
+
 
 /*
  * Device Driver function table.
@@ -178,11 +189,13 @@ struct dd_function_table {
     * the color buffer.
     */
 
-   void (*Clear)( GLcontext *ctx,
-                  GLboolean all, GLint x, GLint y, GLint width, GLint height );
-   /*
-    * Clear the current color buffer.  If 'all' is set the clear the whole
-    * buffer, else clear the region defined by (x,y,width,height).
+   GLbitfield (*Clear)( GLcontext *ctx, GLbitfield mask, GLboolean all,
+                        GLint x, GLint y, GLint width, GLint height );
+   /* Clear the color/depth/stencil/accum buffer(s).
+    * 'mask' indicates which buffers need to be cleared.  Return a bitmask
+    *    indicating which buffers weren't cleared by the driver function.
+    * If 'all' is true then the clear the whole buffer, else clear the
+    *    region defined by (x,y,width,height).
     */
 
    void (*Index)( GLcontext *ctx, GLuint index );
@@ -212,97 +225,85 @@ struct dd_function_table {
    /***
     *** Functions for writing pixels to the frame buffer:
     ***/
-   void (*WriteColorSpan)( GLcontext *ctx,
-                           GLuint n, GLint x, GLint y,
-			   const GLubyte red[], const GLubyte green[],
-			   const GLubyte blue[], const GLubyte alpha[],
-			   const GLubyte mask[] );
-   /*
-    * Write a horizontal run of RGBA pixels.
+
+   void (*WriteRGBASpan)( const GLcontext *ctx,
+                          GLuint n, GLint x, GLint y,
+                          CONST GLubyte rgba[][4], const GLubyte mask[] );
+   void (*WriteRGBSpan)( const GLcontext *ctx,
+                         GLuint n, GLint x, GLint y,
+                         CONST GLubyte rgb[][3], const GLubyte mask[] );
+   /* Write a horizontal run of RGB[A] pixels.  The later version is only
+    * used to accelerate GL_RGB, GL_UNSIGNED_BYTE glDrawPixels() calls.
     */
 
-   void (*WriteMonocolorSpan)( GLcontext *ctx,
-                               GLuint n, GLint x, GLint y,
-			       const GLubyte mask[] );
-   /*
-    * Write a horizontal run of mono-RGBA pixels.
+   void (*WriteMonoRGBASpan)( const GLcontext *ctx, GLuint n, GLint x, GLint y,
+                              const GLubyte mask[] );
+   /* Write a horizontal run of mono-RGBA pixels.
     */
 
-   void (*WriteColorPixels)( GLcontext *ctx,
-                             GLuint n, const GLint x[], const GLint y[],
-			     const GLubyte red[], const GLubyte green[],
-			     const GLubyte blue[], const GLubyte alpha[],
-			     const GLubyte mask[] );
-   /*
-    * Write an array of RGBA pixels at random locations.
+   void (*WriteRGBAPixels)( const GLcontext *ctx,
+                            GLuint n, const GLint x[], const GLint y[],
+                            CONST GLubyte rgba[][4], const GLubyte mask[] );
+   /* Write array of RGBA pixels at random locations.
     */
 
-   void (*WriteMonocolorPixels)( GLcontext *ctx,
-                                 GLuint n, const GLint x[], const GLint y[],
-				 const GLubyte mask[] );
-   /*
-    * Write an array of mono-RGBA pixels at random locations.
+   void (*WriteMonoRGBAPixels)( const GLcontext *ctx,
+                                GLuint n, const GLint x[], const GLint y[],
+                                const GLubyte mask[] );
+   /* Write an array of mono-RGBA pixels at random locations.
     */
 
-   void (*WriteIndexSpan)( GLcontext *ctx,
-                           GLuint n, GLint x, GLint y, const GLuint index[],
-                           const GLubyte mask[] );
-   /*
-    * Write a horizontal run of CI pixels.
+   void (*WriteCI32Span)( const GLcontext *ctx, GLuint n, GLint x, GLint y,
+                          const GLuint index[], const GLubyte mask[] );
+   void (*WriteCI8Span)( const GLcontext *ctx, GLuint n, GLint x, GLint y,
+                         const GLubyte index[], const GLubyte mask[] );
+   /* Write a horizontal run of CI pixels.  32 or 8bpp.
     */
 
-   void (*WriteMonoindexSpan)( GLcontext *ctx,
-                               GLuint n, GLint x, GLint y,
-			       const GLubyte mask[] );
-   /*
-    * Write a horizontal run of mono-CI pixels.
+   void (*WriteMonoCISpan)( const GLcontext *ctx, GLuint n, GLint x, GLint y,
+                            const GLubyte mask[] );
+   /* Write a horizontal run of mono-CI pixels.
     */
 
-   void (*WriteIndexPixels)( GLcontext *ctx,
-                             GLuint n, const GLint x[], const GLint y[],
-                             const GLuint index[], const GLubyte mask[] );
+   void (*WriteCI32Pixels)( const GLcontext *ctx,
+                            GLuint n, const GLint x[], const GLint y[],
+                            const GLuint index[], const GLubyte mask[] );
    /*
     * Write a random array of CI pixels.
     */
 
-   void (*WriteMonoindexPixels)( GLcontext *ctx,
-                                 GLuint n, const GLint x[], const GLint y[],
-				 const GLubyte mask[] );
+   void (*WriteMonoCIPixels)( const GLcontext *ctx,
+                              GLuint n, const GLint x[], const GLint y[],
+                              const GLubyte mask[] );
    /*
     * Write a random array of mono-CI pixels.
     */
 
+
    /***
     *** Functions to read pixels from frame buffer:
     ***/
-   void (*ReadIndexSpan)( GLcontext *ctx,
-                          GLuint n, GLint x, GLint y, GLuint index[] );
-   /*
-    * Read a horizontal run of color index pixels.
+
+   void (*ReadCI32Span)( const GLcontext *ctx,
+                         GLuint n, GLint x, GLint y, GLuint index[] );
+   /* Read a horizontal run of color index pixels.
     */
 
-   void (*ReadColorSpan)( GLcontext *ctx,
-                          GLuint n, GLint x, GLint y,
-			  GLubyte red[], GLubyte green[],
-			  GLubyte blue[], GLubyte alpha[] );
-   /*
-    * Read a horizontal run of RGBA pixels.
+   void (*ReadRGBASpan)( const GLcontext *ctx, GLuint n, GLint x, GLint y,
+                         GLubyte rgba[][4] );
+   /* Read a horizontal run of RGBA pixels.
     */
 
-   void (*ReadIndexPixels)( GLcontext *ctx,
-                            GLuint n, const GLint x[], const GLint y[],
-			    GLuint indx[], const GLubyte mask[] );
-   /*
-    * Read a random array of CI pixels.
+   void (*ReadCI32Pixels)( const GLcontext *ctx,
+                           GLuint n, const GLint x[], const GLint y[],
+                           GLuint indx[], const GLubyte mask[] );
+   /* Read a random array of CI pixels.
     */
 
-   void (*ReadColorPixels)( GLcontext *ctx,
-                            GLuint n, const GLint x[], const GLint y[],
-			    GLubyte red[], GLubyte green[],
-			    GLubyte blue[], GLubyte alpha[],
-                            const GLubyte mask[] );
-   /*
-    * Read a random array of RGBA pixels.
+   void (*ReadRGBAPixels)( const GLcontext *ctx,
+                           GLuint n, const GLint x[], const GLint y[],
+                           GLubyte rgba[][4], const GLubyte mask[] );
+   /* Read a random array of RGBA pixels.
     */
 
 
@@ -314,6 +315,10 @@ struct dd_function_table {
     *** fall-back function.                                            ***
     **********************************************************************/
 
+   const char * (*ExtensionString)( GLcontext *ctx );
+   /* Return a space-separated list of extensions for this driver.
+    */
+
    void (*Finish)( GLcontext *ctx );
    /*
     * Called whenever glFinish() is called.
@@ -359,6 +364,20 @@ struct dd_function_table {
     * uses this.
     */
 
+   GLint (*GetParameteri)( const GLcontext *ctx, GLint param );
+   /* Query the device driver to get an integer parameter.
+    * Current parameters:
+    *     DD_MAX_TEXTURE_SIZE         return maximum texture size
+    *
+    *     DD_MAX_TEXTURES             number of texture sets/stages, usually 1
+    *
+    *     DD_HAVE_HARDWARE_FOG        the driver should return 1 (0 otherwise)
+    *                                 when the hardware support per fragment
+    *                                 fog for free (like the Voodoo Graphics)
+    *                                 so the Mesa core will start to ever use
+    *                                 per fragment fog
+    */
+
 
    /***
     *** For supporting hardware Z buffers:
@@ -369,11 +388,6 @@ struct dd_function_table {
     * Called when the depth buffer must be allocated or possibly resized.
     */
 
-   void (*ClearDepthBuffer)( GLcontext *ctx );
-   /*
-    * Clear the depth buffer to depth specified by CC.Depth.Clear value.
-    */
-
    GLuint (*DepthTestSpan)( GLcontext *ctx,
                             GLuint n, GLint x, GLint y, const GLdepth z[],
                             GLubyte mask[] );
@@ -402,57 +416,51 @@ struct dd_function_table {
     ***/
 
    points_func PointsFunc;
-   /*
-    * Called to draw an array of points.
+   /* Called to draw an array of points.
     */
 
    line_func LineFunc;
-   /*
-    * Called to draw a line segment.
+   /*Called to draw a line segment.
     */
 
    triangle_func TriangleFunc;
-   /* 
-    * Called to draw a filled triangle.
+   /* Called to draw a filled triangle.
     */
 
    quad_func QuadFunc;
-   /* 
-    * Called to draw a filled quadrilateral.
+   /* Called to draw a filled quadrilateral.
     */
 
    rect_func RectFunc;
-   /* 
-    * Called to draw a filled, screen-aligned 2-D rectangle.
+   /* Called to draw a filled, screen-aligned 2-D rectangle.
     */
 
    GLboolean (*DrawPixels)( GLcontext *ctx,
                             GLint x, GLint y, GLsizei width, GLsizei height,
-                            GLenum format, GLenum type, GLboolean packed,
+                            GLenum format, GLenum type,
+                            const struct gl_pixelstore_attrib *unpack,
                             const GLvoid *pixels );
-   /*
-    * Called from glDrawPixels().
+   /* Device driver hook for optimized glDrawPixels.  'unpack' describes how
+    * to unpack the source image data.
     */
 
-   GLboolean (*Bitmap)( GLcontext *ctx, GLsizei width, GLsizei height,
-                        GLfloat xorig, GLfloat yorig,
-                        GLfloat xmove, GLfloat ymove,
-                        const struct gl_image *bitmap );
-   /*
-    * Called from glBitmap().
+   GLboolean (*Bitmap)( GLcontext *ctx,
+                        GLint x, GLint y, GLsizei width, GLsizei height,
+                        const struct gl_pixelstore_attrib *unpack,
+                        const GLubyte *bitmap );
+   /* Device driver hook for optimized glBitmap.  'unpack' describes how
+    * to unpack the source image data.
     */
 
    void (*Begin)( GLcontext *ctx, GLenum mode );
    void (*End)( GLcontext *ctx );
-   /*
-    * These are called whenever glBegin() or glEnd() are called.
+   /* These are called whenever glBegin() or glEnd() are called.
     * The device driver may do some sort of window locking/unlocking here.
     */
 
 
    void (*RasterSetup)( GLcontext *ctx, GLuint start, GLuint end );
-   /*
-    * This function, if not NULL, is called whenever new window coordinates
+   /* This function, if not NULL, is called whenever new window coordinates
     * are put in the vertex buffer.  The vertices in question are those n
     * such that start <= n < end.
     * The device driver can convert the window coords to its own specialized
@@ -460,8 +468,7 @@ struct dd_function_table {
     */
 
    GLboolean (*RenderVB)( GLcontext *ctx, GLboolean allDone );
-   /*
-    * This function allows the device driver to rasterize an entire
+   /* This function allows the device driver to rasterize an entire
     * buffer of primitives at once.  See the gl_render_vb() function
     * in vbrender.c for more details.
     * Return GL_TRUE if vertex buffer successfully rendered.
@@ -493,6 +500,16 @@ struct dd_function_table {
     *       the actual image data.
     */
 
+   void (*TexSubImage)( GLcontext *ctx, GLenum target,
+                        struct gl_texture_object *tObj, GLint level,
+                        GLint xoffset, GLint yoffset,
+                        GLsizei width, GLsizei height,
+                        GLint internalFormat,
+                        const struct gl_texture_image *image );
+   /*
+    * Called from glTexSubImage() to define a sub-region of a texture.
+    */
+
    void (*TexParameter)( GLcontext *ctx, GLenum target,
                          struct gl_texture_object *tObj,
                          GLenum pname, const GLfloat *params );
@@ -509,12 +526,13 @@ struct dd_function_table {
                         struct gl_texture_object *tObj );
    /*
     * Called whenever glBindTexture() is called.  This specifies which
-    * texture is to be the current one.
+    * texture is to be the current one.  No dirty flags will be set.
     */
 
    void (*DeleteTexture)( GLcontext *ctx, struct gl_texture_object *tObj );
    /*
-    * Called when a texture object can be deallocated.
+    * Called when a texture object is about to be deallocated.  Driver
+    * should free anything attached to the DriverData pointers.
     */
 
    void (*UpdateTexturePalette)( GLcontext *ctx,
@@ -530,6 +548,21 @@ struct dd_function_table {
     * Called via glEnable/Disable(GL_SHARED_TEXTURE_PALETTE_EXT)
     */
 
+
+   /***
+    *** NEW
+    ***/
+
+   void (*Viewport)( GLcontext *ctx, GLint x, GLint y, GLsizei w, GLsizei h );
+   /*
+    * Called when viewport parameters are changed.
+    */
+
+   void (*DepthRange)( GLcontext *ctx, GLclampd nearval, GLclampd farval );
+   /*
+    * Called when depth range parameters are changed.
+    */
+
 };
 
 
