@@ -1,9 +1,9 @@
-/* $Id: dlist.c,v 1.35 1997/10/29 01:29:09 brianp Exp $ */
+/* $Id: dlist.c,v 3.8 1998/08/21 02:48:14 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.5
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,111 +23,32 @@
 
 /*
  * $Log: dlist.c,v $
- * Revision 1.35  1997/10/29 01:29:09  brianp
- * added GL_EXT_point_parameters extension from Daniel Barrero
+ * Revision 3.8  1998/08/21 02:48:14  brianp
+ * fixed polygon stipple pointer casting typo
  *
- * Revision 1.34  1997/10/02 00:48:29  brianp
- * removed the EXEC() macro stuff, it caused bugs in gl_save_Color*()
+ * Revision 3.7  1998/08/21 02:42:21  brianp
+ * changed gl_save_PolygonStipple() param type to GLuint *
  *
- * Revision 1.33  1997/09/27 00:13:44  brianp
- * added GL_EXT_paletted_texture extension
+ * Revision 3.6  1998/06/07 22:18:52  brianp
+ * implemented GL_EXT_multitexture extension
  *
- * Revision 1.32  1997/09/23 00:57:52  brianp
- * removed list>MAX_DISPLAYLISTS test
+ * Revision 3.5  1998/03/28 03:58:52  brianp
+ * fixed a few IRIX compiler warnings
  *
- * Revision 1.31  1997/09/22 02:33:58  brianp
- * display lists now implemented with hash table
+ * Revision 3.4  1998/03/27 03:39:14  brianp
+ * fixed G++ warnings
  *
- * Revision 1.30  1997/07/24 01:25:01  brianp
- * changed precompiled header symbol from PCH to PC_HEADER
+ * Revision 3.3  1998/03/19 01:26:39  brianp
+ * fixed API pointer restore problem in glCallList(s)
  *
- * Revision 1.29  1997/07/09 01:26:12  brianp
- * fixed glDrawPixels() GL_COMPILE_AND_EXECUTE infinite loop (Renaud Cazoulat)
+ * Revision 3.2  1998/02/20 04:50:09  brianp
+ * implemented GL_SGIS_multitexture
  *
- * Revision 1.28  1997/06/24 01:13:26  brianp
- * initialize image RefCount to 1 for gl_save_TexSubImage[123]D()
+ * Revision 3.1  1998/02/08 20:23:09  brianp
+ * removed const from gl_save_Bitmap()'s bitmap parameter
  *
- * Revision 1.27  1997/06/20 01:57:57  brianp
- * added gl_save_Color4ubv()
- *
- * Revision 1.26  1997/06/06 02:59:19  brianp
- * renamed destroy_list() to gl_destroy_list()
- *
- * Revision 1.25  1997/05/28 03:24:22  brianp
- * added precompiled header (PCH) support
- *
- * Revision 1.24  1997/05/27 03:13:41  brianp
- * removed some debugging code
- *
- * Revision 1.23  1997/04/30 21:36:22  brianp
- * added casts to gl_TexImage[123]D() calls
- *
- * Revision 1.22  1997/04/28 23:40:47  brianp
- * added #include "rect.h"
- *
- * Revision 1.21  1997/04/24 01:50:53  brianp
- * optimized glColor3f, glColor3fv, glColor4fv
- *
- * Revision 1.20  1997/04/24 00:30:17  brianp
- * optimized glTexCoord2() code
- *
- * Revision 1.19  1997/04/21 01:21:33  brianp
- * added gl_save_Rectf()
- *
- * Revision 1.18  1997/04/20 16:18:15  brianp
- * added glOrtho and glFrustum API pointers
- *
- * Revision 1.17  1997/04/16 23:55:33  brianp
- * added optimized glTexCoord2f code
- *
- * Revision 1.16  1997/04/14 22:18:23  brianp
- * added optimized glVertex3fv code
- *
- * Revision 1.15  1997/04/14 02:00:39  brianp
- * #include "texstate.h" instead of "texture.h"
- *
- * Revision 1.14  1997/04/07 02:58:49  brianp
- * added gl_save_Vertex[23]f() and related code
- *
- * Revision 1.13  1997/04/01 04:26:02  brianp
- * added code for glLoadIdentity(), changed #include's
- *
- * Revision 1.12  1997/02/27 19:58:08  brianp
- * call gl_problem() instead of gl_warning()
- *
- * Revision 1.11  1997/02/09 18:50:18  brianp
- * added GL_EXT_texture3D support
- *
- * Revision 1.10  1997/01/29 18:51:30  brianp
- * small MEMCPY call change for Acorn compiler, per Graham Jones
- *
- * Revision 1.9  1997/01/09 21:25:28  brianp
- * set reference count to one for texture images in display lists
- *
- * Revision 1.8  1996/12/11 20:19:11  brianp
- * abort display list execution if invalid opcode found
- *
- * Revision 1.7  1996/12/09 21:39:23  brianp
- * compare list to MAX_DISPLAYLISTS in gl_IsList()
- *
- * Revision 1.6  1996/12/04 22:14:27  brianp
- * improved the mesa_print_display_list() debug function
- *
- * Revision 1.5  1996/11/07 04:12:45  brianp
- * texture images are now gl_image structs, not gl_texture_image structs
- *
- * Revision 1.4  1996/10/16 00:59:12  brianp
- * fixed bug in gl_save_Lightfv()
- * execution of OPCODE_EDGE_FLAG used enum value instead of boolean
- *
- * Revision 1.3  1996/09/27 01:26:08  brianp
- * removed unused variables
- *
- * Revision 1.2  1996/09/19 00:54:43  brianp
- * fixed bug in gl_save_Rotatef() when in GL_COMPILE_AND_EXECUTE mode
- *
- * Revision 1.1  1996/09/13 01:38:16  brianp
- * Initial revision
+ * Revision 3.0  1998/01/31 20:50:39  brianp
+ * initial rev
  *
  */
 
@@ -292,6 +213,7 @@ typedef enum {
 	OPCODE_MATERIAL,
 	OPCODE_MATRIX_MODE,
 	OPCODE_MULT_MATRIX,
+	OPCODE_MULTI_TEXCOORD4,
         OPCODE_NORMAL,
 	OPCODE_ORTHO,
 	OPCODE_PASSTHROUGH,
@@ -315,6 +237,10 @@ typedef enum {
 	OPCODE_READ_BUFFER,
         OPCODE_SCALE,
 	OPCODE_SCISSOR,
+	OPCODE_SELECT_TEXTURE_SGIS,
+	OPCODE_SELECT_TEXTURE,
+	OPCODE_SELECT_TEXTURE_COORD_SET,
+	OPCODE_SELECT_TEXTURE_TRANSFORM,
 	OPCODE_SHADE_MODEL,
 	OPCODE_STENCIL_FUNC,
 	OPCODE_STENCIL_MASK,
@@ -347,7 +273,7 @@ typedef enum {
  * contiguous nodes in memory.
  * Each node is the union of a variety of datatypes.
  */
-typedef union node {
+union node {
 	OpCode		opcode;
 	GLboolean	b;
 	GLbitfield	bf;
@@ -360,7 +286,7 @@ typedef union node {
 	GLfloat		f;
 	GLvoid		*data;
 	void		*next;	/* If prev node's opcode==OPCODE_CONTINUE */
-} Node;
+};
 
 
 
@@ -368,14 +294,6 @@ typedef union node {
 static GLuint InstSize[ OPCODE_END_OF_LIST+1 ];
 
 
-/* Used while a display list is under construction: */
-static Node *CurrentListPtr;	/* Head of list being compiled */
-static GLuint CurrentListNum;	/* Number of the list being compiled */
-static Node *CurrentBlock;	/* Pointer to current block of nodes */
-static GLuint CurrentPos;	/* Index into current block of nodes */
-
-
-
 
 /**********************************************************************/
 /*****                           Private                          *****/
@@ -393,11 +311,11 @@ static Node *alloc_instruction( GLcontext *ctx, OpCode opcode, GLint argcount )
    Node *n, *newblock;
    GLuint count = InstSize[opcode];
 
-   assert( count == argcount+1 );
+   assert( (GLint) count == argcount+1 );
 
-   if (CurrentPos + count + 2 > BLOCK_SIZE) {
+   if (ctx->CurrentPos + count + 2 > BLOCK_SIZE) {
       /* This block is full.  Allocate a new block and chain to it */
-      n = CurrentBlock + CurrentPos;
+      n = ctx->CurrentBlock + ctx->CurrentPos;
       n[0].opcode = OPCODE_CONTINUE;
       newblock = (Node *) malloc( sizeof(Node) * BLOCK_SIZE );
       if (!newblock) {
@@ -405,12 +323,12 @@ static Node *alloc_instruction( GLcontext *ctx, OpCode opcode, GLint argcount )
          return NULL;
       }
       n[1].next = (Node *) newblock;
-      CurrentBlock = newblock;
-      CurrentPos = 0;
+      ctx->CurrentBlock = newblock;
+      ctx->CurrentPos = 0;
    }
 
-   n = CurrentBlock + CurrentPos;
-   CurrentPos += count;
+   n = ctx->CurrentBlock + ctx->CurrentPos;
+   ctx->CurrentPos += count;
 
    n[0].opcode = opcode;
 
@@ -457,7 +375,7 @@ void gl_destroy_list( GLcontext *ctx, GLuint list )
 	    n += InstSize[n[0].opcode];
 	    break;
 	 case OPCODE_DRAW_PIXELS:
-	    free( n[5].data );
+	    gl_free_image( (struct gl_image *) n[1].data );
 	    n += InstSize[n[0].opcode];
 	    break;
 	 case OPCODE_BITMAP:
@@ -585,9 +503,6 @@ void gl_init_lists( void )
    static int init_flag = 0;
 
    if (init_flag==0) {
-      CurrentListPtr = CurrentBlock = NULL;
-      CurrentListNum = 0;
-
       InstSize[OPCODE_ACCUM] = 3;
       InstSize[OPCODE_ALPHA_FUNC] = 3;
       InstSize[OPCODE_BEGIN] = 2;
@@ -624,7 +539,7 @@ void gl_init_lists( void )
       InstSize[OPCODE_DEPTH_RANGE] = 3;
       InstSize[OPCODE_DISABLE] = 2;
       InstSize[OPCODE_DRAW_BUFFER] = 2;
-      InstSize[OPCODE_DRAW_PIXELS] = 6;
+      InstSize[OPCODE_DRAW_PIXELS] = 2;
       InstSize[OPCODE_ENABLE] = 2;
       InstSize[OPCODE_EDGE_FLAG] = 2;
       InstSize[OPCODE_END] = 1;
@@ -657,6 +572,7 @@ void gl_init_lists( void )
       InstSize[OPCODE_MATERIAL] = 7;
       InstSize[OPCODE_MATRIX_MODE] = 2;
       InstSize[OPCODE_MULT_MATRIX] = 17;
+      InstSize[OPCODE_MULTI_TEXCOORD4] = 6;
       InstSize[OPCODE_NORMAL] = 4;
       InstSize[OPCODE_ORTHO] = 7;
       InstSize[OPCODE_PASSTHROUGH] = 2;
@@ -680,6 +596,10 @@ void gl_init_lists( void )
       InstSize[OPCODE_READ_BUFFER] = 2;
       InstSize[OPCODE_SCALE] = 4;
       InstSize[OPCODE_SCISSOR] = 5;
+      InstSize[OPCODE_SELECT_TEXTURE_SGIS] = 2;
+      InstSize[OPCODE_SELECT_TEXTURE] = 2;
+      InstSize[OPCODE_SELECT_TEXTURE_COORD_SET] = 2;
+      InstSize[OPCODE_SELECT_TEXTURE_TRANSFORM] = 2;
       InstSize[OPCODE_STENCIL_FUNC] = 4;
       InstSize[OPCODE_STENCIL_MASK] = 2;
       InstSize[OPCODE_STENCIL_OP] = 4;
@@ -708,18 +628,6 @@ void gl_init_lists( void )
 }
 
 
-
-/*
- * Return the name of the display list currently being compiled.  This
- * function is only called by glGet().
- */
-GLint gl_list_index( void )
-{
-   return CurrentListNum;
-}
-
-
-
 /*
  * Display List compilation functions
  */
@@ -780,7 +688,7 @@ void gl_save_Bitmap( GLcontext *ctx,
                      GLsizei width, GLsizei height,
 		     GLfloat xorig, GLfloat yorig,
 		     GLfloat xmove, GLfloat ymove,
-		     const struct gl_image *bitmap )
+		     struct gl_image *bitmap )
 {
    Node *n = alloc_instruction( ctx, OPCODE_BITMAP, 7 );
    if (n) {
@@ -792,6 +700,9 @@ void gl_save_Bitmap( GLcontext *ctx,
       n[6].f = ymove;
       n[7].data = (void *) bitmap;
    }
+   if (bitmap) {
+      bitmap->RefCount = 1;
+   }
    if (ctx->ExecuteFlag) {
       (*ctx->Exec.Bitmap)( ctx, width, height,
                     xorig, yorig, xmove, ymove, bitmap );
@@ -855,7 +766,7 @@ void gl_save_CallList( GLcontext *ctx, GLuint list )
 void gl_save_CallLists( GLcontext *ctx,
                         GLsizei n, GLenum type, const GLvoid *lists )
 {
-   GLuint i;
+   GLint i;
 
    for (i=0;i<n;i++) {
       GLuint list = translate_id( i, type, lists );
@@ -1332,22 +1243,18 @@ void gl_save_DrawBuffer( GLcontext *ctx, GLenum mode )
 }
 
 
-void gl_save_DrawPixels( GLcontext *ctx, GLsizei width, GLsizei height,
-                         GLenum format, GLenum type, const GLvoid *pixels )
+void gl_save_DrawPixels( GLcontext *ctx, struct gl_image *image )
 {
-   Node *n = alloc_instruction( ctx, OPCODE_DRAW_PIXELS, 5 );
+   Node *n = alloc_instruction( ctx, OPCODE_DRAW_PIXELS, 1 );
    if (n) {
-      n[1].i = (GLint) width;
-      n[2].i = (GLint) height;
-      n[3].e = format;
-      n[4].e = type;
-      n[5].data = (GLvoid *) pixels;
+      n[1].data = (GLvoid *) image;
+   }
+   if (image) {
+      image->RefCount = 1;
    }
-/* Special case:  gl_DrawPixels takes care of GL_COMPILE_AND_EXECUTE case!
    if (ctx->ExecuteFlag) {
-      (*ctx->Exec.DrawPixels)( ctx, width, height, format, type, pixels );
+      (*ctx->Exec.DrawPixels)( ctx, image );
    }
-*/
 }
 
 
@@ -1707,6 +1614,7 @@ void gl_save_Map1f( GLcontext *ctx,
    if (ctx->ExecuteFlag) {
       (*ctx->Exec.Map1f)( ctx, target, u1, u2, stride, order, points, GL_TRUE );
    }
+   (void) retain;
 }
 
 
@@ -1733,6 +1641,7 @@ void gl_save_Map2f( GLcontext *ctx, GLenum target,
                         u1, u2, ustride, uorder,
                         v1, v2, vstride, vorder, points, GL_TRUE );
    }
+   (void) retain;
 }
 
 
@@ -1818,6 +1727,8 @@ void gl_save_NewList( GLcontext *ctx, GLuint list, GLenum mode )
 {
    /* It's an error to call this function while building a display list */
    gl_error( ctx, GL_INVALID_OPERATION, "glNewList" );
+   (void) list;
+   (void) mode;
 }
 
 
@@ -1951,17 +1862,20 @@ void gl_save_PolygonMode( GLcontext *ctx, GLenum face, GLenum mode )
 }
 
 
-void gl_save_PolygonStipple( GLcontext *ctx, const GLubyte *mask )
+/*
+ * Polygon stipple must have been upacked already!
+ */
+void gl_save_PolygonStipple( GLcontext *ctx, const GLuint *pattern )
 {
    Node *n = alloc_instruction( ctx, OPCODE_POLYGON_STIPPLE, 1 );
    if (n) {
       void *data;
       n[1].data = malloc( 32 * 4 );
       data = n[1].data;   /* This needed for Acorn compiler */
-      MEMCPY( data, mask, 32 * 4 );
+      MEMCPY( data, pattern, 32 * 4 );
    }
    if (ctx->ExecuteFlag) {
-      (*ctx->Exec.PolygonStipple)( ctx, mask );
+      (*ctx->Exec.PolygonStipple)( ctx, pattern );
    }
 }
 
@@ -2558,6 +2472,79 @@ void gl_save_WindowPos4fMESA( GLcontext *ctx,
 }
 
 
+/* GL_SGIS_multitexture */
+void gl_save_MultiTexCoord4f( GLcontext *ctx, GLenum target,
+                              GLfloat s, GLfloat t, GLfloat r, GLfloat q )
+{
+   Node *n = alloc_instruction( ctx, OPCODE_MULTI_TEXCOORD4, 5);
+   if (n) {
+      n[1].e = target;
+      n[2].f = s;
+      n[3].f = t;
+      n[4].f = r;
+      n[5].f = q;
+   }
+   if (ctx->ExecuteFlag) {
+      (*ctx->Exec.MultiTexCoord4f)( ctx, target, s, t, r, q );
+   }
+}
+
+
+/* GL_SGIS_multitexture */
+void gl_save_SelectTextureSGIS( GLcontext *ctx, GLenum target )
+{
+   Node *n = alloc_instruction( ctx, OPCODE_SELECT_TEXTURE_SGIS, 1 );
+   if (n) {
+      n[1].e = target;
+   }
+   if (ctx->ExecuteFlag) {
+      (*ctx->Exec.SelectTextureSGIS)( ctx, target );
+   }
+}
+
+
+/* GL_EXT_multitexture */
+void gl_save_SelectTexture( GLcontext *ctx, GLenum target )
+{
+   Node *n = alloc_instruction( ctx, OPCODE_SELECT_TEXTURE, 1 );
+   if (n) {
+      n[1].e = target;
+   }
+   if (ctx->ExecuteFlag) {
+      (*ctx->Exec.SelectTexture)( ctx, target );
+   }
+}
+
+
+/* GL_SGIS_multitexture */
+void gl_save_SelectTextureCoordSet( GLcontext *ctx, GLenum target )
+{
+   Node *n = alloc_instruction( ctx, OPCODE_SELECT_TEXTURE_COORD_SET, 1 );
+   if (n) {
+      n[1].e = target;
+   }
+   if (ctx->ExecuteFlag) {
+      (*ctx->Exec.SelectTextureCoordSet)( ctx, target );
+   }
+}
+
+
+/* GL_EXT_multitexture */
+void gl_save_SelectTextureTransform( GLcontext *ctx, GLenum target )
+{
+   Node *n = alloc_instruction( ctx, OPCODE_SELECT_TEXTURE_TRANSFORM, 1 );
+   if (n) {
+      n[1].e = target;
+   }
+   if (ctx->ExecuteFlag) {
+      (*ctx->Exec.SelectTextureTransform)( ctx, target );
+   }
+}
+
+
+
+
+
 
 /**********************************************************************/
 /*                     Display list execution                         */
@@ -2764,8 +2751,7 @@ static void execute_list( GLcontext *ctx, GLuint list )
 	    gl_DrawBuffer( ctx, n[1].e );
 	    break;
 	 case OPCODE_DRAW_PIXELS:
-	    gl_DrawPixels( ctx, (GLsizei) n[1].i, (GLsizei) n[2].i,
-			   n[3].e, n[4].e, n[5].data );
+	    gl_DrawPixels( ctx, (struct gl_image *) n[1].data );
 	    break;
 	 case OPCODE_EDGE_FLAG:
             ctx->Current.EdgeFlag = n[1].b;
@@ -2912,6 +2898,10 @@ static void execute_list( GLcontext *ctx, GLuint list )
 	       gl_MultMatrixf( ctx, m );
 	    }
 	    break;
+         case OPCODE_MULTI_TEXCOORD4:
+            (*ctx->Exec.MultiTexCoord4f)( ctx, n[1].e,
+                                          n[2].f, n[3].f, n[4].f, n[5].f );
+            break;
          case OPCODE_ORTHO:
             gl_Ortho( ctx, n[1].f, n[2].f, n[3].f, n[4].f, n[5].f, n[6].f );
             break;
@@ -2943,7 +2933,7 @@ static void execute_list( GLcontext *ctx, GLuint list )
 	    gl_PolygonMode( ctx, n[1].e, n[2].e );
 	    break;
 	 case OPCODE_POLYGON_STIPPLE:
-	    gl_PolygonStipple( ctx, (GLubyte *) n[1].data );
+	    gl_PolygonStipple( ctx, (GLuint *) n[1].data );
 	    break;
 	 case OPCODE_POLYGON_OFFSET:
 	    gl_PolygonOffset( ctx, n[1].f, n[2].f );
@@ -2978,6 +2968,18 @@ static void execute_list( GLcontext *ctx, GLuint list )
          case OPCODE_RECTF:
             gl_Rectf( ctx, n[1].f, n[2].f, n[3].f, n[4].f );
             break;
+         case OPCODE_SELECT_TEXTURE:
+            gl_SelectTexture( ctx, n[1].e );
+            break;
+         case OPCODE_SELECT_TEXTURE_SGIS:
+            gl_SelectTextureSGIS( ctx, n[1].e );
+            break;
+         case OPCODE_SELECT_TEXTURE_COORD_SET:
+            gl_SelectTextureCoordSet( ctx, n[1].e );
+            break;
+         case OPCODE_SELECT_TEXTURE_TRANSFORM:
+            gl_SelectTextureTransform( ctx, n[1].e );
+            break;
          case OPCODE_SCALE:
             gl_Scalef( ctx, n[1].f, n[2].f, n[3].f );
             break;
@@ -3178,7 +3180,7 @@ GLuint gl_GenLists( GLcontext *ctx, GLsizei range )
    base = HashFindFreeKeyBlock(ctx->Shared->DisplayList, range);
    if (base) {
       /* reserve the list IDs by with empty/dummy lists */
-      GLuint i;
+      GLint i;
       for (i=0; i<range; i++) {
          HashInsert(ctx->Shared->DisplayList, base+i, make_empty_list());
       }
@@ -3205,16 +3207,16 @@ void gl_NewList( GLcontext *ctx, GLuint list, GLenum mode )
       gl_error( ctx, GL_INVALID_ENUM, "glNewList" );
       return;
    }
-   if (CurrentListPtr) {
+   if (ctx->CurrentListPtr) {
       /* already compiling a display list */
       gl_error( ctx, GL_INVALID_OPERATION, "glNewList" );
       return;
    }
 
    /* Allocate new display list */
-   CurrentListNum = list;
-   CurrentListPtr = CurrentBlock = (Node *) malloc( sizeof(Node) * BLOCK_SIZE );
-   CurrentPos = 0;
+   ctx->CurrentListNum = list;
+   ctx->CurrentListPtr = ctx->CurrentBlock = (Node *) malloc( sizeof(Node) * BLOCK_SIZE );
+   ctx->CurrentPos = 0;
 
    ctx->CompileFlag = GL_TRUE;
    if (mode==GL_COMPILE) {
@@ -3235,23 +3237,21 @@ void gl_NewList( GLcontext *ctx, GLuint list, GLenum mode )
  */
 void gl_EndList( GLcontext *ctx )
 {
-   Node *n;
-
    /* Check that a list is under construction */
-   if (!CurrentListPtr) {
+   if (!ctx->CurrentListPtr) {
       gl_error( ctx, GL_INVALID_OPERATION, "glEndList" );
       return;
    }
 
-   n = alloc_instruction( ctx, OPCODE_END_OF_LIST, 0 );
+   (void) alloc_instruction( ctx, OPCODE_END_OF_LIST, 0 );
 
    /* Destroy old list, if any */
-   gl_destroy_list(ctx, CurrentListNum);
+   gl_destroy_list(ctx, ctx->CurrentListNum);
    /* Install the list */
-   HashInsert(ctx->Shared->DisplayList, CurrentListNum, CurrentListPtr);
+   HashInsert(ctx->Shared->DisplayList, ctx->CurrentListNum, ctx->CurrentListPtr);
 
-   CurrentListNum = 0;
-   CurrentListPtr = NULL;
+   ctx->CurrentListNum = 0;
+   ctx->CurrentListPtr = NULL;
    ctx->ExecuteFlag = GL_TRUE;
    ctx->CompileFlag = GL_FALSE;
 
@@ -3269,6 +3269,10 @@ void gl_CallList( GLcontext *ctx, GLuint list )
    ctx->CompileFlag = GL_FALSE;
    execute_list( ctx, list );
    ctx->CompileFlag = save_compile_flag;
+
+   /* also restore API function pointers to point to "save" versions */
+   if (save_compile_flag)
+           ctx->API = ctx->Save;
 }
 
 
@@ -3279,7 +3283,8 @@ void gl_CallList( GLcontext *ctx, GLuint list )
 void gl_CallLists( GLcontext *ctx,
                    GLsizei n, GLenum type, const GLvoid *lists )
 {
-   GLuint i, list;
+   GLuint list;
+   GLint i;
    GLboolean save_compile_flag;
 
    /* Save the CompileFlag status, turn it off, execute display list,
@@ -3294,6 +3299,10 @@ void gl_CallLists( GLcontext *ctx,
    }
 
    ctx->CompileFlag = save_compile_flag;
+
+   /* also restore API function pointers to point to "save" versions */
+   if (save_compile_flag)
+           ctx->API = ctx->Save;
 }
 
 
@@ -3382,10 +3391,10 @@ static void print_list( GLcontext *ctx, FILE *f, GLuint list )
 		       n[3].f, n[4].f, n[5].f, n[6].f, (void *) n[7].data );
 	    break;
          case OPCODE_CALL_LIST:
-            fprintf(f,"CallList %d\n", n[1].ui );
+            fprintf(f,"CallList %d\n", (int) n[1].ui );
             break;
          case OPCODE_CALL_LIST_OFFSET:
-            fprintf(f,"CallList %d + offset %d = %d\n", n[1].ui,
+            fprintf(f,"CallList %d + offset %d = %d\n", (int) n[1].ui,
                     ctx->List.ListBase, ctx->List.ListBase + n[1].ui );
             break;
 	 case OPCODE_COLOR_3F:
@@ -3412,7 +3421,7 @@ static void print_list( GLcontext *ctx, FILE *f, GLuint list )
                     n[1].f, n[2].f, n[3].f, n[4].f, n[5].f, n[6].f );
             break;
          case OPCODE_INDEX:
-            fprintf(f,"Index %d\n", n[1].ui );
+            fprintf(f,"Index %d\n", (int) n[1].ui );
             break;
 	 case OPCODE_LINE_STIPPLE:
 	    fprintf(f,"LineStipple %d %x\n", n[1].i, (int) n[2].us );
@@ -3461,7 +3470,7 @@ static void print_list( GLcontext *ctx, FILE *f, GLuint list )
             fprintf(f,"PushMatrix\n");
 	    break;
 	 case OPCODE_PUSH_NAME:
-            fprintf(f,"PushName %d\n", n[1].ui );
+            fprintf(f,"PushName %d\n", (int) n[1].ui );
 	    break;
 	 case OPCODE_RASTER_POS:
             fprintf(f,"RasterPos %g %g %g %g\n", n[1].f, n[2].f,n[3].f,n[4].f);
@@ -3531,7 +3540,7 @@ static void print_list( GLcontext *ctx, FILE *f, GLuint list )
 void mesa_print_display_list( GLuint list )
 {
    GLcontext *ctx;
-#ifdef MULTI_THREADING
+#ifdef THREADS
    ctx = gl_get_thread_context();
 #else
    ctx = CC;
