@@ -1,9 +1,9 @@
-/* $Id: dlist.h,v 1.14 1997/10/29 01:29:09 brianp Exp $ */
+/* $Id: dlist.h,v 3.4 1998/08/21 02:42:21 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.5
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,47 +23,20 @@
 
 /*
  * $Log: dlist.h,v $
- * Revision 1.14  1997/10/29 01:29:09  brianp
- * added GL_EXT_point_parameters extension from Daniel Barrero
+ * Revision 3.4  1998/08/21 02:42:21  brianp
+ * changed gl_save_PolygonStipple() param type to GLuint *
  *
- * Revision 1.13  1997/09/27 00:13:44  brianp
- * added GL_EXT_paletted_texture extension
+ * Revision 3.3  1998/06/07 22:18:52  brianp
+ * implemented GL_EXT_multitexture extension
  *
- * Revision 1.12  1997/06/20 01:57:57  brianp
- * added gl_save_Color4ubv()
+ * Revision 3.2  1998/02/20 04:50:09  brianp
+ * implemented GL_SGIS_multitexture
  *
- * Revision 1.11  1997/06/06 02:59:12  brianp
- * added gl_destroy_list()
+ * Revision 3.1  1998/02/08 20:23:09  brianp
+ * removed const from gl_save_Bitmap()'s bitmap parameter
  *
- * Revision 1.10  1997/04/24 01:50:53  brianp
- * optimized glColor3f, glColor3fv, glColor4fv
- *
- * Revision 1.9  1997/04/21 01:21:33  brianp
- * added gl_save_Rectf()
- *
- * Revision 1.8  1997/04/20 16:18:15  brianp
- * added glOrtho and glFrustum API pointers
- *
- * Revision 1.7  1997/04/16 23:55:33  brianp
- * added optimized glTexCoord2f code
- *
- * Revision 1.6  1997/04/14 22:18:23  brianp
- * added optimized glVertex3fv code
- *
- * Revision 1.5  1997/04/07 02:58:49  brianp
- * added gl_save_Vertex[23]f() and related code
- *
- * Revision 1.4  1997/04/01 04:26:02  brianp
- * added code for glLoadIdentity(), changed #include's
- *
- * Revision 1.3  1997/02/09 18:50:18  brianp
- * added GL_EXT_texture3D support
- *
- * Revision 1.2  1996/11/07 04:12:45  brianp
- * texture images are now gl_image structs, not gl_texture_image structs
- *
- * Revision 1.1  1996/09/13 01:38:16  brianp
- * Initial revision
+ * Revision 3.0  1998/01/31 20:50:39  brianp
+ * initial rev
  *
  */
 
@@ -78,8 +51,6 @@
 
 extern void gl_init_lists( void );
 
-extern GLint gl_list_index( void );
-
 extern void gl_destroy_list( GLcontext *ctx, GLuint list );
 
 
@@ -124,7 +95,7 @@ extern void gl_save_BindTexture( GLcontext *ctx,
 extern void gl_save_Bitmap( GLcontext *ctx, GLsizei width, GLsizei height,
 			    GLfloat xorig, GLfloat yorig,
 			    GLfloat xmove, GLfloat ymove,
-                            const struct gl_image *bitmap );
+                            struct gl_image *bitmap );
 
 extern void gl_save_CallList( GLcontext *ctx, GLuint list );
 
@@ -219,9 +190,7 @@ extern void gl_save_Disable( GLcontext *ctx, GLenum cap );
 
 extern void gl_save_DrawBuffer( GLcontext *ctx, GLenum mode );
 
-extern void gl_save_DrawPixels( GLcontext *ctx,
-                                GLsizei width, GLsizei height, GLenum format,
-			        GLenum type, const GLvoid *pixels );
+extern void gl_save_DrawPixels( GLcontext *ctx, struct gl_image *image );
 
 extern void gl_save_EdgeFlag( GLcontext *ctx, GLboolean flag );
 
@@ -337,7 +306,7 @@ extern void gl_save_PointSize( GLcontext *ctx, GLfloat size );
 
 extern void gl_save_PolygonMode( GLcontext *ctx, GLenum face, GLenum mode );
 
-extern void gl_save_PolygonStipple( GLcontext *ctx, const GLubyte *mask );
+extern void gl_save_PolygonStipple( GLcontext *ctx, const GLuint *pattern );
 
 extern void gl_save_PolygonOffset( GLcontext *ctx,
                                    GLfloat factor, GLfloat units );
@@ -460,4 +429,18 @@ extern void gl_save_WindowPos4fMESA( GLcontext *ctx,
                                      GLfloat z, GLfloat w );
 
 
+/* GL_SGIS_multitexture */
+extern void gl_save_MultiTexCoord4f( GLcontext *ctx, GLenum target,
+                                     GLfloat s, GLfloat t,
+                                     GLfloat r, GLfloat q );
+
+extern void gl_save_SelectTextureSGIS( GLcontext *ctx, GLenum target );
+
+extern void gl_save_SelectTexture( GLcontext *ctx, GLenum target );
+
+extern void gl_save_SelectTextureCoordSet( GLcontext *ctx, GLenum target );
+
+extern void gl_save_SelectTextureTransform( GLcontext *ctx, GLenum target );
+
+
 #endif
