@@ -1,9 +1,9 @@
-/* $Id: enable.c,v 1.21 1997/07/24 01:25:01 brianp Exp $ */
+/* $Id: enable.c,v 3.5 1998/09/01 03:07:37 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,68 +23,23 @@
 
 /*
  * $Log: enable.c,v $
- * Revision 1.21  1997/07/24 01:25:01  brianp
- * changed precompiled header symbol from PCH to PC_HEADER
+ * Revision 3.5  1998/09/01 03:07:37  brianp
+ * references to texture coords now indexed by ctx->TexCoordSet
  *
- * Revision 1.20  1997/06/20 02:20:32  brianp
- * replaced Current.IntColor with Current.ByteColor
+ * Revision 3.4  1998/03/28 03:58:30  brianp
+ * removed unneeded break
  *
- * Revision 1.19  1997/05/31 16:57:14  brianp
- * added MESA_NO_RASTER env var support
+ * Revision 3.3  1998/02/20 04:50:44  brianp
+ * implemented GL_SGIS_multitexture
  *
- * Revision 1.18  1997/05/28 03:24:22  brianp
- * added precompiled header (PCH) support
+ * Revision 3.2  1998/02/08 20:18:20  brianp
+ * removed unneeded headers
  *
- * Revision 1.17  1997/04/24 01:49:30  brianp
- * call gl_set_color_function() instead of directly setting pointers
+ * Revision 3.1  1998/02/01 16:37:19  brianp
+ * added GL_EXT_rescale_normal extension
  *
- * Revision 1.16  1997/04/12 16:54:05  brianp
- * new NEW_POLYGON state flag
- *
- * Revision 1.15  1997/04/12 16:20:27  brianp
- * removed call to Driver.Dither()
- *
- * Revision 1.14  1997/04/02 03:10:36  brianp
- * changed some #include's
- *
- * Revision 1.13  1997/02/27 19:59:08  brianp
- * issue a warning if enable depth or stencil test without such a buffer
- *
- * Revision 1.12  1997/02/09 19:53:43  brianp
- * now use TEXTURE_xD enable constants
- *
- * Revision 1.11  1997/02/09 18:49:37  brianp
- * added GL_EXT_texture3D support
- *
- * Revision 1.10  1997/01/28 22:13:42  brianp
- * now there's separate state for CI and RGBA logic op enabled
- *
- * Revision 1.9  1996/12/18 20:00:57  brianp
- * gl_set_material() now takes a bitmask instead of face and pname
- *
- * Revision 1.8  1996/12/11 20:16:49  brianp
- * more work on the GL_COLOR_MATERIAL bug
- *
- * Revision 1.7  1996/12/09 22:51:51  brianp
- * update API Color4f and Color4ub pointers for GL_COLOR_MATERIAL
- *
- * Revision 1.6  1996/12/07 10:21:07  brianp
- * call gl_set_material() instead of gl_Materialfv()
- *
- * Revision 1.5  1996/11/09 03:11:18  brianp
- * added missing GL_EXT_vertex_array caps to gl_enable()
- *
- * Revision 1.4  1996/10/11 03:44:09  brianp
- * added comments for GL_POLYGON_OFFSET_EXT symbol
- *
- * Revision 1.3  1996/09/27 01:26:40  brianp
- * removed unused variables
- *
- * Revision 1.2  1996/09/15 14:17:30  brianp
- * now use GLframebuffer and GLvisual
- *
- * Revision 1.1  1996/09/13 01:38:16  brianp
- * Initial revision
+ * Revision 3.0  1998/01/31 20:51:23  brianp
+ * initial rev
  *
  */
 
@@ -94,12 +49,9 @@
 #else
 #include <string.h>
 #include "context.h"
-#include "depth.h"
 #include "enable.h"
 #include "light.h"
-#include "dlist.h"
 #include "macros.h"
-#include "stencil.h"
 #include "types.h"
 #include "vbfill.h"
 #endif
@@ -111,6 +63,7 @@
  */
 static void gl_enable( GLcontext* ctx, GLenum cap, GLboolean state )
 {
+   GLuint texSet = ctx->Texture.CurrentSet;
    GLuint p;
 
    if (INSIDE_BEGIN_END(ctx)) {
@@ -167,10 +120,10 @@ static void gl_enable( GLcontext* ctx, GLenum cap, GLboolean state )
             ctx->Light.ColorMaterialEnabled = state;
             if (state) {
                GLfloat color[4];
-               color[0] = ctx->Current.ByteColor[0] * ctx->Visual->InvRedScale;
-               color[1] = ctx->Current.ByteColor[1] * ctx->Visual->InvGreenScale;
-               color[2] = ctx->Current.ByteColor[2] * ctx->Visual->InvBlueScale;
-               color[3] = ctx->Current.ByteColor[3] * ctx->Visual->InvAlphaScale;
+               color[0] = ctx->Current.ByteColor[0] * (1.0F / 255.0F);
+               color[1] = ctx->Current.ByteColor[1] * (1.0F / 255.0F);
+               color[2] = ctx->Current.ByteColor[2] * (1.0F / 255.0F);
+               color[3] = ctx->Current.ByteColor[3] * (1.0F / 255.0F);
                /* update material with current color */
                gl_set_material( ctx, ctx->Light.ColorMaterialBitmask, color );
             }
@@ -345,12 +298,20 @@ static void gl_enable( GLcontext* ctx, GLenum cap, GLboolean state )
             ctx->NewState |= NEW_POLYGON;
          }
          break;
+      case GL_RESCALE_NORMAL_EXT:
+         ctx->Transform.RescaleNormals = state;
+         break;
       case GL_SCISSOR_TEST:
          if (ctx->Scissor.Enabled!=state) {
             ctx->Scissor.Enabled = state;
             ctx->NewState |= NEW_RASTER_OPS;
          }
 	 break;
+      case GL_SHARED_TEXTURE_PALETTE_EXT:
+         ctx->Texture.SharedPalette = state;
+         if (ctx->Driver.UseGlobalTexturePalette)
+            (*ctx->Driver.UseGlobalTexturePalette)( ctx, state );
+         break;
       case GL_STENCIL_TEST:
 	 if (state && ctx->Visual->StencilBits==0) {
             gl_warning(ctx, "glEnable(GL_STENCIL_TEST) but no stencil buffer");
@@ -364,11 +325,12 @@ static void gl_enable( GLcontext* ctx, GLenum cap, GLboolean state )
       case GL_TEXTURE_1D:
          if (ctx->Visual->RGBAflag) {
             /* texturing only works in RGB mode */
+            GLuint bit = TEXTURE0_1D << (ctx->Texture.CurrentSet * 4);
             if (state) {
-               ctx->Texture.Enabled |= TEXTURE_1D;
+               ctx->Texture.Enabled |= bit;
             }
             else {
-               ctx->Texture.Enabled &= (~TEXTURE_1D);
+               ctx->Texture.Enabled &= (~bit);
             }
             ctx->NewState |= (NEW_RASTER_OPS | NEW_TEXTURING);
          }
@@ -376,11 +338,12 @@ static void gl_enable( GLcontext* ctx, GLenum cap, GLboolean state )
       case GL_TEXTURE_2D:
          if (ctx->Visual->RGBAflag) {
             /* texturing only works in RGB mode */
+            GLuint bit = TEXTURE0_2D << (ctx->Texture.CurrentSet * 4);
             if (state) {
-               ctx->Texture.Enabled |= TEXTURE_2D;
+               ctx->Texture.Enabled |= bit;
             }
             else {
-               ctx->Texture.Enabled &= (~TEXTURE_2D);
+               ctx->Texture.Enabled &= (~bit);
             }
             ctx->NewState |= (NEW_RASTER_OPS | NEW_TEXTURING);
          }
@@ -388,48 +351,49 @@ static void gl_enable( GLcontext* ctx, GLenum cap, GLboolean state )
       case GL_TEXTURE_3D_EXT:
          if (ctx->Visual->RGBAflag) {
             /* texturing only works in RGB mode */
+            GLuint bit = TEXTURE0_3D << (ctx->Texture.CurrentSet * 4);
             if (state) {
-               ctx->Texture.Enabled |= TEXTURE_3D;
+               ctx->Texture.Enabled |= bit;
             }
             else {
-               ctx->Texture.Enabled &= (~TEXTURE_3D);
+               ctx->Texture.Enabled &= (~bit);
             }
             ctx->NewState |= (NEW_RASTER_OPS | NEW_TEXTURING);
          }
          break;
       case GL_TEXTURE_GEN_Q:
          if (state) {
-            ctx->Texture.TexGenEnabled |= Q_BIT;
+            ctx->Texture.Set[texSet].TexGenEnabled |= Q_BIT;
          }
          else {
-            ctx->Texture.TexGenEnabled &= ~Q_BIT;
+            ctx->Texture.Set[texSet].TexGenEnabled &= ~Q_BIT;
          }
          ctx->NewState |= NEW_TEXTURING;
 	 break;
       case GL_TEXTURE_GEN_R:
          if (state) {
-            ctx->Texture.TexGenEnabled |= R_BIT;
+            ctx->Texture.Set[texSet].TexGenEnabled |= R_BIT;
          }
          else {
-            ctx->Texture.TexGenEnabled &= ~R_BIT;
+            ctx->Texture.Set[texSet].TexGenEnabled &= ~R_BIT;
          }
          ctx->NewState |= NEW_TEXTURING;
 	 break;
       case GL_TEXTURE_GEN_S:
 	 if (state) {
-            ctx->Texture.TexGenEnabled |= S_BIT;
+            ctx->Texture.Set[texSet].TexGenEnabled |= S_BIT;
          }
          else {
-            ctx->Texture.TexGenEnabled &= ~S_BIT;
+            ctx->Texture.Set[texSet].TexGenEnabled &= ~S_BIT;
          }
          ctx->NewState |= NEW_TEXTURING;
 	 break;
       case GL_TEXTURE_GEN_T:
          if (state) {
-            ctx->Texture.TexGenEnabled |= T_BIT;
+            ctx->Texture.Set[texSet].TexGenEnabled |= T_BIT;
          }
          else {
-            ctx->Texture.TexGenEnabled &= ~T_BIT;
+            ctx->Texture.Set[texSet].TexGenEnabled &= ~T_BIT;
          }
          ctx->NewState |= NEW_TEXTURING;
 	 break;
@@ -450,7 +414,7 @@ static void gl_enable( GLcontext* ctx, GLenum cap, GLboolean state )
          ctx->Array.IndexEnabled = state;
          break;
       case GL_TEXTURE_COORD_ARRAY:
-         ctx->Array.TexCoordEnabled = state;
+         ctx->Array.TexCoordEnabled[ctx->TexCoordSet] = state;
          break;
       case GL_EDGE_FLAG_ARRAY:
          ctx->Array.EdgeFlagEnabled = state;
@@ -580,24 +544,49 @@ GLboolean gl_IsEnabled( GLcontext* ctx, GLenum cap )
       case GL_POLYGON_OFFSET_FILL:
       /*case GL_POLYGON_OFFSET_EXT:*/
 	 return ctx->Polygon.OffsetFill;
+      case GL_RESCALE_NORMAL_EXT:
+         return ctx->Transform.RescaleNormals;
       case GL_SCISSOR_TEST:
 	 return ctx->Scissor.Enabled;
+      case GL_SHARED_TEXTURE_PALETTE_EXT:
+         return ctx->Texture.SharedPalette;
       case GL_STENCIL_TEST:
 	 return ctx->Stencil.Enabled;
       case GL_TEXTURE_1D:
-	 return (ctx->Texture.Enabled & TEXTURE_1D) ? GL_TRUE : GL_FALSE;
+         {
+            GLuint bit = TEXTURE0_1D << (ctx->Texture.CurrentSet * 4);
+            return (ctx->Texture.Enabled & bit) ? GL_TRUE : GL_FALSE;
+         }
       case GL_TEXTURE_2D:
-	 return (ctx->Texture.Enabled & TEXTURE_2D) ? GL_TRUE : GL_FALSE;
+         {
+            GLuint bit = TEXTURE0_1D << (ctx->Texture.CurrentSet * 4);
+            return (ctx->Texture.Enabled & bit) ? GL_TRUE : GL_FALSE;
+         }
       case GL_TEXTURE_3D_EXT:
-         return (ctx->Texture.Enabled & TEXTURE_3D) ? GL_TRUE : GL_FALSE;
+         {
+            GLuint bit = TEXTURE0_1D << (ctx->Texture.CurrentSet * 4);
+            return (ctx->Texture.Enabled & bit) ? GL_TRUE : GL_FALSE;
+         }
       case GL_TEXTURE_GEN_Q:
-	 return (ctx->Texture.TexGenEnabled & Q_BIT) ? GL_TRUE : GL_FALSE;
+         {
+            const struct gl_texture_set *texSet = &ctx->Texture.Set[ctx->Texture.CurrentSet];
+            return (texSet->TexGenEnabled & Q_BIT) ? GL_TRUE : GL_FALSE;
+         }
       case GL_TEXTURE_GEN_R:
-	 return (ctx->Texture.TexGenEnabled & R_BIT) ? GL_TRUE : GL_FALSE;
+         {
+            const struct gl_texture_set *texSet = &ctx->Texture.Set[ctx->Texture.CurrentSet];
+            return (texSet->TexGenEnabled & R_BIT) ? GL_TRUE : GL_FALSE;
+         }
       case GL_TEXTURE_GEN_S:
-	 return (ctx->Texture.TexGenEnabled & S_BIT) ? GL_TRUE : GL_FALSE;
+         {
+            const struct gl_texture_set *texSet = &ctx->Texture.Set[ctx->Texture.CurrentSet];
+            return (texSet->TexGenEnabled & S_BIT) ? GL_TRUE : GL_FALSE;
+         }
       case GL_TEXTURE_GEN_T:
-	 return (ctx->Texture.TexGenEnabled & T_BIT) ? GL_TRUE : GL_FALSE;
+         {
+            const struct gl_texture_set *texSet = &ctx->Texture.Set[ctx->Texture.CurrentSet];
+            return (texSet->TexGenEnabled & T_BIT) ? GL_TRUE : GL_FALSE;
+         }
 
       /*
        * CLIENT STATE!!!
@@ -611,7 +600,7 @@ GLboolean gl_IsEnabled( GLcontext* ctx, GLenum cap )
       case GL_INDEX_ARRAY:
          return ctx->Array.IndexEnabled;
       case GL_TEXTURE_COORD_ARRAY:
-         return ctx->Array.TexCoordEnabled;
+         return ctx->Array.TexCoordEnabled[ctx->TexCoordSet];
       case GL_EDGE_FLAG_ARRAY:
          return ctx->Array.EdgeFlagEnabled;
       default:
@@ -639,7 +628,7 @@ void gl_client_state( GLcontext *ctx, GLenum cap, GLboolean state )
          ctx->Array.IndexEnabled = state;
          break;
       case GL_TEXTURE_COORD_ARRAY:
-         ctx->Array.TexCoordEnabled = state;
+         ctx->Array.TexCoordEnabled[ctx->TexCoordSet] = state;
          break;
       case GL_EDGE_FLAG_ARRAY:
          ctx->Array.EdgeFlagEnabled = state;
