@@ -1,9 +1,9 @@
-/* $Id: eval.c,v 1.8 1997/07/24 01:25:01 brianp Exp $ */
+/* $Id: eval.c,v 3.3 1998/06/23 00:02:54 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,29 +23,17 @@
 
 /*
  * $Log: eval.c,v $
- * Revision 1.8  1997/07/24 01:25:01  brianp
- * changed precompiled header symbol from PCH to PC_HEADER
+ * Revision 3.3  1998/06/23 00:02:54  brianp
+ * fixed evaluator color bug (Eero Pajarre)
  *
- * Revision 1.7  1997/06/20 01:58:47  brianp
- * changed color components from GLfixed to GLubyte
+ * Revision 3.2  1998/03/27 03:37:40  brianp
+ * fixed G++ warnings
  *
- * Revision 1.6  1997/05/28 03:24:22  brianp
- * added precompiled header (PCH) support
+ * Revision 3.1  1998/02/03 00:54:15  brianp
+ * fixed bug in gl_copy_map_points*() functions (Sam Jordan)
  *
- * Revision 1.5  1997/05/14 03:27:04  brianp
- * removed context argument from gl_init_eval()
- *
- * Revision 1.4  1997/05/01 01:38:38  brianp
- * use NORMALIZE_3FV() from mmath.h instead of private NORMALIZE() macro
- *
- * Revision 1.3  1997/04/02 03:10:58  brianp
- * changed some #include's
- *
- * Revision 1.2  1996/09/15 14:17:30  brianp
- * now use GLframebuffer and GLvisual
- *
- * Revision 1.1  1996/09/13 01:38:16  brianp
- * Initial revision
+ * Revision 3.0  1998/01/31 20:51:54  brianp
+ * initial rev
  *
  */
 
@@ -71,7 +59,6 @@
 #include <string.h>
 #include "context.h"
 #include "eval.h"
-#include "dlist.h"
 #include "macros.h"
 #include "mmath.h"
 #include "types.h"
@@ -586,9 +573,9 @@ GLfloat *gl_copy_map_points1f( GLenum target,
                                const GLfloat *points )
 {
    GLfloat *buffer, *p;
-   GLuint i, k, size = components(target);
+   GLint i, k, size = components(target);
 
-   if (!points || components==0) {
+   if (!points || size==0) {
       return NULL;
    }
 
@@ -612,7 +599,11 @@ GLfloat *gl_copy_map_points1d( GLenum target,
 			        const GLdouble *points )
 {
    GLfloat *buffer, *p;
-   GLuint i, k, size = components(target);
+   GLint i, k, size = components(target);
+
+   if (!points || size==0) {
+      return NULL;
+   }
 
    buffer = (GLfloat *) malloc(uorder * size * sizeof(GLfloat));
 
@@ -642,11 +633,15 @@ GLfloat *gl_copy_map_points2f( GLenum target,
 			        const GLfloat *points )
 {
    GLfloat *buffer, *p;
-   GLuint i, j, k, size, dsize, hsize;
+   GLint i, j, k, size, dsize, hsize;
    GLint uinc;
 
    size = components(target);
 
+   if (!points || size==0) {
+      return NULL;
+   }
+
    /* max(uorder, vorder) additional points are used in      */
    /* horner evaluation and uorder*vorder additional */
    /* values are needed for de Casteljau                     */
@@ -681,11 +676,15 @@ GLfloat *gl_copy_map_points2d(GLenum target,
                               const GLdouble *points )
 {
    GLfloat *buffer, *p;
-   GLuint i, j, k, size, hsize, dsize;
+   GLint i, j, k, size, hsize, dsize;
    GLint uinc;
 
    size = components(target);
 
+   if (!points || size==0) {
+      return NULL;
+   }
+
    /* max(uorder, vorder) additional points are used in      */
    /* horner evaluation and uorder*vorder additional */
    /* values are needed for de Casteljau                     */
@@ -822,7 +821,7 @@ void gl_Map1f( GLcontext* ctx, GLenum target,
                GLfloat u1, GLfloat u2, GLint stride,
                GLint order, const GLfloat *points, GLboolean retain )
 {
-   GLuint k;
+   GLint k;
 
    if (!points) {
       gl_error( ctx, GL_OUT_OF_MEMORY, "glMap1f" );
@@ -975,7 +974,7 @@ void gl_Map2f( GLcontext* ctx, GLenum target,
 	      GLfloat v1, GLfloat v2, GLint vstride, GLint vorder,
 	      const GLfloat *points, GLboolean retain )
 {
-   GLuint k;
+   GLint k;
 
    if (INSIDE_BEGIN_END(ctx)) {
       gl_error( ctx, GL_INVALID_OPERATION, "glMap2" );
@@ -1154,7 +1153,7 @@ void gl_Map2f( GLcontext* ctx, GLenum target,
 
 void gl_GetMapdv( GLcontext* ctx, GLenum target, GLenum query, GLdouble *v )
 {
-   GLuint i, n;
+   GLint i, n;
    GLfloat *data;
 
    switch (query) {
@@ -1423,7 +1422,7 @@ void gl_GetMapdv( GLcontext* ctx, GLenum target, GLenum query, GLdouble *v )
 
 void gl_GetMapfv( GLcontext* ctx, GLenum target, GLenum query, GLfloat *v )
 {
-   GLuint i, n;
+   GLint i, n;
    GLfloat *data;
 
    switch (query) {
@@ -1966,7 +1965,7 @@ void gl_EvalCoord1f(GLcontext* ctx, GLfloat u)
   GLfloat normal[3];
   GLfloat fcolor[4];
   GLubyte icolor[4];
-  GLubyte *colorptr;
+  const GLubyte *colorPtr;
   GLfloat texcoord[4];
   GLuint index;
   register GLfloat uu;
@@ -2004,16 +2003,14 @@ void gl_EvalCoord1f(GLcontext* ctx, GLfloat u)
      struct gl_1d_map *map = &ctx->EvalMap.Map1Color4;
      uu = (u - map->u1) / (map->u2 - map->u1);
      horner_bezier_curve(map->Points, fcolor, uu, 4, map->Order);
-     icolor[0] = (GLint) (fcolor[0] * ctx->Visual->RedScale);
-     icolor[1] = (GLint) (fcolor[1] * ctx->Visual->GreenScale);
-     icolor[2] = (GLint) (fcolor[2] * ctx->Visual->BlueScale);
-     icolor[3] = (GLint) (fcolor[3] * ctx->Visual->AlphaScale);
-     colorptr = icolor;
+     icolor[0] = (GLint) (fcolor[0] * 255.0F);
+     icolor[1] = (GLint) (fcolor[1] * 255.0F);
+     icolor[2] = (GLint) (fcolor[2] * 255.0F);
+     icolor[3] = (GLint) (fcolor[3] * 255.0F);
+     colorPtr = icolor;
   }
   else {
-     GLubyte col[4];
-     COPY_4V(col, ctx->Current.ByteColor );
-     colorptr = col;
+     colorPtr = ctx->Current.ByteColor;
   }
 
   /** Normal Vector **/
@@ -2062,7 +2059,7 @@ void gl_EvalCoord1f(GLcontext* ctx, GLfloat u)
      texcoord[3] = ctx->Current.TexCoord[3];
   }
   
-  gl_eval_vertex( ctx, vertex, normal, colorptr, index, texcoord );
+  gl_eval_vertex( ctx, vertex, normal, colorPtr, index, texcoord );
 }
 
 
@@ -2072,7 +2069,7 @@ void gl_EvalCoord2f( GLcontext* ctx, GLfloat u, GLfloat v )
    GLfloat normal[3];
    GLfloat fcolor[4];
    GLubyte icolor[4];
-   GLubyte *colorptr;
+   const GLubyte *colorPtr;
    GLfloat texcoord[4];
    GLuint index;
    register GLfloat uu, vv;
@@ -2142,16 +2139,14 @@ void gl_EvalCoord2f( GLcontext* ctx, GLfloat u, GLfloat v )
       vv = (v - map->v1) / (map->v2 - map->v1);
       horner_bezier_surf(map->Points, fcolor, uu, vv, 4,
                          map->Uorder, map->Vorder);
-      icolor[0] = (GLint) (fcolor[0] * ctx->Visual->RedScale);
-      icolor[1] = (GLint) (fcolor[1] * ctx->Visual->GreenScale);
-      icolor[2] = (GLint) (fcolor[2] * ctx->Visual->BlueScale);
-      icolor[3] = (GLint) (fcolor[3] * ctx->Visual->AlphaScale);
-      colorptr = icolor;
+      icolor[0] = (GLint) (fcolor[0] * 255.0F);
+      icolor[1] = (GLint) (fcolor[1] * 255.0F);
+      icolor[2] = (GLint) (fcolor[2] * 255.0F);
+      icolor[3] = (GLint) (fcolor[3] * 255.0F);
+      colorPtr = icolor;
    }
    else {
-     GLubyte col[4];
-     COPY_4V(col, ctx->Current.ByteColor );
-     colorptr = col;
+      colorPtr = ctx->Current.ByteColor;
    }
 
    /** Normal **/
@@ -2214,7 +2209,7 @@ void gl_EvalCoord2f( GLcontext* ctx, GLfloat u, GLfloat v )
      texcoord[3] = ctx->Current.TexCoord[3];
    }
 
-   gl_eval_vertex( ctx, vertex, normal, colorptr, index, texcoord );
+   gl_eval_vertex( ctx, vertex, normal, colorPtr, index, texcoord );
 }
 
 
@@ -2260,54 +2255,54 @@ void gl_MapGrid2f( GLcontext* ctx, GLint un, GLfloat u1, GLfloat u2,
 
 void gl_EvalPoint1( GLcontext* ctx, GLint i )
 {
-	GLfloat u, du;
+   GLfloat u, du;
 
-	if (i==0) {
-		u = ctx->Eval.MapGrid1u1;
-	}
-	else if (i==ctx->Eval.MapGrid1un) {
-		u = ctx->Eval.MapGrid1u2;
-	}
-	else {
-		du = (ctx->Eval.MapGrid1u2 - ctx->Eval.MapGrid1u1)
-			/ (GLfloat) ctx->Eval.MapGrid1un;
-		u = i * du + ctx->Eval.MapGrid1u1;
-	}
-	gl_EvalCoord1f( ctx, u );
+   if (i==0) {
+      u = ctx->Eval.MapGrid1u1;
+   }
+   else if (i==ctx->Eval.MapGrid1un) {
+      u = ctx->Eval.MapGrid1u2;
+   }
+   else {
+      du = (ctx->Eval.MapGrid1u2 - ctx->Eval.MapGrid1u1)
+         / (GLfloat) ctx->Eval.MapGrid1un;
+      u = i * du + ctx->Eval.MapGrid1u1;
+   }
+   gl_EvalCoord1f( ctx, u );
 }
 
 
 
 void gl_EvalPoint2( GLcontext* ctx, GLint i, GLint j )
 {
-	GLfloat u, du;
-	GLfloat v, dv;
+   GLfloat u, du;
+   GLfloat v, dv;
 
-	if (i==0) {
-		u = ctx->Eval.MapGrid2u1;
-	}
-	else if (i==ctx->Eval.MapGrid2un) {
-		u = ctx->Eval.MapGrid2u2;
-	}
-	else {
-		du = (ctx->Eval.MapGrid2u2 - ctx->Eval.MapGrid2u1)
-			/ (GLfloat) ctx->Eval.MapGrid2un;
-		u = i * du + ctx->Eval.MapGrid2u1;
-	}
+   if (i==0) {
+      u = ctx->Eval.MapGrid2u1;
+   }
+   else if (i==ctx->Eval.MapGrid2un) {
+      u = ctx->Eval.MapGrid2u2;
+   }
+   else {
+      du = (ctx->Eval.MapGrid2u2 - ctx->Eval.MapGrid2u1)
+         / (GLfloat) ctx->Eval.MapGrid2un;
+      u = i * du + ctx->Eval.MapGrid2u1;
+   }
 
-	if (j==0) {
-		v = ctx->Eval.MapGrid2v1;
-	}
-	else if (j==ctx->Eval.MapGrid2vn) {
-		v = ctx->Eval.MapGrid2v2;
-	}
-	else {
-		dv = (ctx->Eval.MapGrid2v2 - ctx->Eval.MapGrid2v1)
-			/ (GLfloat) ctx->Eval.MapGrid2vn;
-		v = j * dv + ctx->Eval.MapGrid2v1;
-	}
+   if (j==0) {
+      v = ctx->Eval.MapGrid2v1;
+   }
+   else if (j==ctx->Eval.MapGrid2vn) {
+      v = ctx->Eval.MapGrid2v2;
+   }
+   else {
+      dv = (ctx->Eval.MapGrid2v2 - ctx->Eval.MapGrid2v1)
+         / (GLfloat) ctx->Eval.MapGrid2vn;
+      v = j * dv + ctx->Eval.MapGrid2v1;
+   }
 
-	gl_EvalCoord2f( ctx, u, v );
+   gl_EvalCoord2f( ctx, u, v );
 }
 
 
@@ -2318,40 +2313,40 @@ void gl_EvalMesh1( GLcontext* ctx, GLenum mode, GLint i1, GLint i2 )
    GLfloat u, du;
    GLenum prim;
 
-	if (INSIDE_BEGIN_END(ctx)) {
-		gl_error( ctx, GL_INVALID_OPERATION, "glEvalMesh1" );
-		return;
-	}
+   if (INSIDE_BEGIN_END(ctx)) {
+      gl_error( ctx, GL_INVALID_OPERATION, "glEvalMesh1" );
+      return;
+   }
 
-	switch (mode) {
-	case GL_POINT:
-		prim = GL_POINTS;
-		break;
-	case GL_LINE:
-		prim = GL_LINE_STRIP;
-		break;
-	default:
-		gl_error( ctx, GL_INVALID_ENUM, "glEvalMesh1(mode)" );
-		return;
-	}
+   switch (mode) {
+      case GL_POINT:
+         prim = GL_POINTS;
+         break;
+      case GL_LINE:
+         prim = GL_LINE_STRIP;
+         break;
+      default:
+         gl_error( ctx, GL_INVALID_ENUM, "glEvalMesh1(mode)" );
+         return;
+   }
 
-	du = (ctx->Eval.MapGrid1u2 - ctx->Eval.MapGrid1u1)
-		/ (GLfloat) ctx->Eval.MapGrid1un;
+   du = (ctx->Eval.MapGrid1u2 - ctx->Eval.MapGrid1u1)
+      / (GLfloat) ctx->Eval.MapGrid1un;
 
-	gl_Begin( ctx, prim );
-	for (i=i1;i<=i2;i++) {
-		if (i==0) {
-			u = ctx->Eval.MapGrid1u1;
-		}
-		else if (i==ctx->Eval.MapGrid1un) {
-			u = ctx->Eval.MapGrid1u2;
-		}
-		else {
-			u = i * du + ctx->Eval.MapGrid1u1;
-		}
-		gl_EvalCoord1f( ctx, u );
-	}
-	gl_End(ctx);
+   gl_Begin( ctx, prim );
+   for (i=i1;i<=i2;i++) {
+      if (i==0) {
+         u = ctx->Eval.MapGrid1u1;
+      }
+      else if (i==ctx->Eval.MapGrid1un) {
+         u = ctx->Eval.MapGrid1u2;
+      }
+      else {
+         u = i * du + ctx->Eval.MapGrid1u1;
+      }
+      gl_EvalCoord1f( ctx, u );
+   }
+   gl_End(ctx);
 }
 
 
