@@ -1,9 +1,9 @@
-/* $Id: hash.h,v 1.1 1997/08/22 01:15:10 brianp Exp $ */
+/* $Id: hash.h,v 3.0 1998/01/31 20:53:58 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,8 +23,8 @@
 
 /*
  * $Log: hash.h,v $
- * Revision 1.1  1997/08/22 01:15:10  brianp
- * Initial revision
+ * Revision 3.0  1998/01/31 20:53:58  brianp
+ * initial rev
  *
  */
 
@@ -48,6 +48,10 @@ extern void *HashLookup(const struct HashTable *table, GLuint key);
 
 extern void HashInsert(struct HashTable *table, GLuint key, void *data);
 
+extern void HashRemove(struct HashTable *table, GLuint key);
+
+extern GLuint HashFirstEntry(const struct HashTable *table);
+
 extern void HashPrint(const struct HashTable *table);
 
 extern GLuint HashFindFreeKeyBlock(const struct HashTable *table, GLuint numKeys);
