@@ -1,9 +1,9 @@
-/* $Id: light.c,v 1.14 1997/07/24 01:24:11 brianp Exp $ */
+/* $Id: light.c,v 3.4 1998/03/27 03:37:40 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,48 +23,20 @@
 
 /*
  * $Log: light.c,v $
- * Revision 1.14  1997/07/24 01:24:11  brianp
- * changed precompiled header symbol from PCH to PC_HEADER
+ * Revision 3.4  1998/03/27 03:37:40  brianp
+ * fixed G++ warnings
  *
- * Revision 1.13  1997/06/20 04:15:43  brianp
- * optimized changing of SHININESS (Henk Kok)
+ * Revision 3.3  1998/02/27 00:44:52  brianp
+ * fixed an incorrect error message string
  *
- * Revision 1.12  1997/05/28 03:25:26  brianp
- * added precompiled header (PCH) support
+ * Revision 3.2  1998/02/08 20:18:20  brianp
+ * removed unneeded headers
  *
- * Revision 1.11  1997/05/01 01:38:57  brianp
- * now use NORMALIZE_3FV() macro from mmath.h
+ * Revision 3.1  1998/02/02 03:09:34  brianp
+ * added GL_LIGHT_MODEL_COLOR_CONTROL (separate specular color interpolation)
  *
- * Revision 1.10  1997/04/20 20:28:49  brianp
- * replaced abort() with gl_problem()
- *
- * Revision 1.9  1997/04/07 02:59:17  brianp
- * small optimization to setting of shininess and spot exponent
- *
- * Revision 1.8  1997/04/01 04:09:31  brianp
- * misc code clean-ups.  moved shading code to shade.c
- *
- * Revision 1.7  1997/03/11 00:37:39  brianp
- * spotlight factor now effects ambient lighting
- *
- * Revision 1.6  1996/12/18 20:02:07  brianp
- * glColorMaterial() and glMaterial() should finally work right!
- *
- * Revision 1.5  1996/12/07 10:22:41  brianp
- * gl_Materialfv() now calls gl_set_material() if GL_COLOR_MATERIAL disabled
- * implemented gl_GetLightiv()
- *
- * Revision 1.4  1996/11/08 04:39:23  brianp
- * new gl_compute_spot_exp_table() contributed by Randy Frank
- *
- * Revision 1.3  1996/09/27 01:27:55  brianp
- * removed unused variables
- *
- * Revision 1.2  1996/09/15 14:18:10  brianp
- * now use GLframebuffer and GLvisual
- *
- * Revision 1.1  1996/09/13 01:38:16  brianp
- * Initial revision
+ * Revision 3.0  1998/01/31 20:54:56  brianp
+ * initial rev
  *
  */
 
@@ -78,7 +50,6 @@
 #include <stdlib.h>
 #include "context.h"
 #include "light.h"
-#include "dlist.h"
 #include "macros.h"
 #include "matrix.h"
 #include "mmath.h"
@@ -117,8 +88,10 @@ void gl_Lightfv( GLcontext *ctx,
 {
    GLint l;
 
+   (void) nparams;
+
    if (INSIDE_BEGIN_END(ctx)) {
-      gl_error( ctx, GL_INVALID_OPERATION, "glShadeModel" );
+      gl_error( ctx, GL_INVALID_OPERATION, "glLight" );
       return;
    }
 
@@ -286,30 +259,30 @@ void gl_GetLightiv( GLcontext *ctx, GLenum light, GLenum pname, GLint *params )
          params[3] = FLOAT_TO_INT(ctx->Light.Light[l].Specular[3]);
          break;
       case GL_POSITION:
-         params[0] = ctx->Light.Light[l].Position[0];
-         params[1] = ctx->Light.Light[l].Position[1];
-         params[2] = ctx->Light.Light[l].Position[2];
-         params[3] = ctx->Light.Light[l].Position[3];
+         params[0] = (GLint) ctx->Light.Light[l].Position[0];
+         params[1] = (GLint) ctx->Light.Light[l].Position[1];
+         params[2] = (GLint) ctx->Light.Light[l].Position[2];
+         params[3] = (GLint) ctx->Light.Light[l].Position[3];
          break;
       case GL_SPOT_DIRECTION:
-         params[0] = ctx->Light.Light[l].Direction[0];
-         params[1] = ctx->Light.Light[l].Direction[1];
-         params[2] = ctx->Light.Light[l].Direction[2];
+         params[0] = (GLint) ctx->Light.Light[l].Direction[0];
+         params[1] = (GLint) ctx->Light.Light[l].Direction[1];
+         params[2] = (GLint) ctx->Light.Light[l].Direction[2];
          break;
       case GL_SPOT_EXPONENT:
-         params[0] = ctx->Light.Light[l].SpotExponent;
+         params[0] = (GLint) ctx->Light.Light[l].SpotExponent;
          break;
       case GL_SPOT_CUTOFF:
-         params[0] = ctx->Light.Light[l].SpotCutoff;
+         params[0] = (GLint) ctx->Light.Light[l].SpotCutoff;
          break;
       case GL_CONSTANT_ATTENUATION:
-         params[0] = ctx->Light.Light[l].ConstantAttenuation;
+         params[0] = (GLint) ctx->Light.Light[l].ConstantAttenuation;
          break;
       case GL_LINEAR_ATTENUATION:
-         params[0] = ctx->Light.Light[l].LinearAttenuation;
+         params[0] = (GLint) ctx->Light.Light[l].LinearAttenuation;
          break;
       case GL_QUADRATIC_ATTENUATION:
-         params[0] = ctx->Light.Light[l].QuadraticAttenuation;
+         params[0] = (GLint) ctx->Light.Light[l].QuadraticAttenuation;
          break;
       default:
          gl_error( ctx, GL_INVALID_ENUM, "glGetLightiv" );
@@ -342,6 +315,14 @@ void gl_LightModelfv( GLcontext *ctx, GLenum pname, const GLfloat *params )
          else
             ctx->Light.Model.TwoSide = GL_TRUE;
          break;
+      case GL_LIGHT_MODEL_COLOR_CONTROL:
+         if (params[0] == (GLfloat) GL_SINGLE_COLOR)
+            ctx->Light.Model.ColorControl = GL_SINGLE_COLOR;
+         else if (params[0] == (GLfloat) GL_SEPARATE_SPECULAR_COLOR)
+            ctx->Light.Model.ColorControl = GL_SEPARATE_SPECULAR_COLOR;
+         else
+            gl_error( ctx, GL_INVALID_ENUM, "glLightModel(param)" );
+         break;
       default:
          gl_error( ctx, GL_INVALID_ENUM, "glLightModel" );
          break;
@@ -741,22 +722,10 @@ void gl_compute_spot_exp_table( struct gl_light *l )
 void gl_compute_material_shine_table( struct gl_material *m )
 {
    int i;
-   double exponent = m->Shininess;
-
    m->ShineTable[0] = 0.0F;
    for (i=1;i<SHINE_TABLE_SIZE;i++) {
-#if 0
-      double x = pow( i/(double)(SHINE_TABLE_SIZE-1), exponent );
-      if (x<1.0e-10) {
-         m->ShineTable[i] = 0.0F;
-      }
-      else {
-         m->ShineTable[i] = x;
-      }
-#else
       /* just invalidate the table */
       m->ShineTable[i] = -1.0;
-#endif
    }
 }
 
