@@ -1,9 +1,9 @@
-/* $Id: pointers.c,v 1.18 1997/07/24 01:23:44 brianp Exp $ */
+/* $Id: pointers.c,v 3.3 1998/06/07 22:18:52 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,59 +23,17 @@
 
 /*
  * $Log: pointers.c,v $
- * Revision 1.18  1997/07/24 01:23:44  brianp
- * changed precompiled header symbol from PCH to PC_HEADER
+ * Revision 3.3  1998/06/07 22:18:52  brianp
+ * implemented GL_EXT_multitexture extension
  *
- * Revision 1.17  1997/06/20 02:50:55  brianp
- * added Color4ubv API pointer
+ * Revision 3.2  1998/02/20 04:50:44  brianp
+ * implemented GL_SGIS_multitexture
  *
- * Revision 1.16  1997/05/28 03:26:02  brianp
- * added precompiled header (PCH) support
+ * Revision 3.1  1998/02/01 20:05:10  brianp
+ * added glDrawRangeElements()
  *
- * Revision 1.15  1997/05/24 12:08:25  brianp
- * broke gl_init_api_function_pointers() into sub-functions
- *
- * Revision 1.14  1997/04/24 01:50:53  brianp
- * optimized glColor3f, glColor3fv, glColor4fv
- *
- * Revision 1.13  1997/04/21 01:22:27  brianp
- * added gl_save_Rectf()
- *
- * Revision 1.12  1997/04/20 20:28:49  brianp
- * replaced abort() with gl_problem()
- *
- * Revision 1.11  1997/04/20 16:18:15  brianp
- * added glOrtho and glFrustum API pointers
- *
- * Revision 1.10  1997/04/16 23:55:33  brianp
- * added optimized glTexCoord2f code
- *
- * Revision 1.9  1997/04/14 22:18:23  brianp
- * added optimized glVertex3fv code
- *
- * Revision 1.8  1997/04/14 02:00:39  brianp
- * #include "texstate.h" instead of "texture.h"
- *
- * Revision 1.7  1997/04/12 12:33:12  brianp
- * added #include "rect.h"
- *
- * Revision 1.6  1997/04/07 02:57:13  brianp
- * added API.Vertex[23] functions
- *
- * Revision 1.5  1997/04/01 04:25:18  brianp
- * added pointer for LoadIdentity, changed #include's
- *
- * Revision 1.4  1997/02/10 19:49:29  brianp
- * added glResizeBuffersMESA() code
- *
- * Revision 1.3  1997/02/09 18:50:42  brianp
- * added GL_EXT_texture3D support
- *
- * Revision 1.2  1996/10/16 00:52:22  brianp
- * gl_initialize_api_function_pointers() now gl_init_api_function_pointers()
- *
- * Revision 1.1  1996/09/13 01:38:16  brianp
- * Initial revision
+ * Revision 3.0  1998/01/31 21:00:28  brianp
+ * initial rev
  *
  */
 
@@ -93,6 +51,7 @@
 #include "blend.h"
 #include "clip.h"
 #include "context.h"
+#include "colortab.h"
 #include "copypix.h"
 #include "depth.h"
 #include "drawpix.h"
@@ -153,7 +112,7 @@ static void check_pointers( struct gl_api_table *table )
  * Assign all the pointers in 'table' to point to Mesa's immediate-mode
  * execution functions.
  */
-static void init_exec_pointers( struct gl_api_table *table )
+static void init_execution_pointers( struct gl_api_table *table )
 {
    table->Accum = gl_Accum;
    table->AlphaFunc = gl_AlphaFunc;
@@ -183,6 +142,8 @@ static void init_exec_pointers( struct gl_api_table *table )
    table->ColorMask = gl_ColorMask;
    table->ColorMaterial = gl_ColorMaterial;
    table->ColorPointer = gl_ColorPointer;
+   table->ColorTable = gl_ColorTable;
+   table->ColorSubTable = gl_ColorSubTable;
    table->CopyPixels = gl_CopyPixels;
    table->CopyTexImage1D = gl_CopyTexImage1D;
    table->CopyTexImage2D = gl_CopyTexImage2D;
@@ -201,6 +162,7 @@ static void init_exec_pointers( struct gl_api_table *table )
    table->DrawBuffer = gl_DrawBuffer;
    table->DrawElements = gl_DrawElements;
    table->DrawPixels = gl_DrawPixels;
+   table->DrawRangeElements = gl_DrawRangeElements;
    table->EdgeFlag = gl_EdgeFlag;
    table->EdgeFlagPointer = gl_EdgeFlagPointer;
    table->Enable = gl_Enable;
@@ -223,6 +185,8 @@ static void init_exec_pointers( struct gl_api_table *table )
    table->GenTextures = gl_GenTextures;
    table->GetBooleanv = gl_GetBooleanv;
    table->GetClipPlane = gl_GetClipPlane;
+   table->GetColorTable = gl_GetColorTable;
+   table->GetColorTableParameteriv = gl_GetColorTableParameteriv;
    table->GetDoublev = gl_GetDoublev;
    table->GetError = gl_GetError;
    table->GetFloatv = gl_GetFloatv;
@@ -323,6 +287,7 @@ static void init_exec_pointers( struct gl_api_table *table )
    table->TexSubImage1D = gl_TexSubImage1D;
    table->TexSubImage2D = gl_TexSubImage2D;
    table->TexSubImage3DEXT = gl_TexSubImage3DEXT;
+   table->PointParameterfvEXT = gl_PointParameterfvEXT;
    table->TexParameterfv = gl_TexParameterfv;
    table->Translatef = gl_Translatef;
    table->Vertex2f = gl_vertex2f_nop;
@@ -337,6 +302,15 @@ static void init_exec_pointers( struct gl_api_table *table )
 
    /* GL_MESA_resize_buffers extension */
    table->ResizeBuffersMESA = gl_ResizeBuffersMESA;
+
+   /* GL_SGIS_multitexture / GL_EXT_multitexture */
+   table->MultiTexCoord4f = gl_MultiTexCoord4f;
+   table->MultiTexCoordPointer = gl_MultiTexCoordPointer;
+   table->InterleavedTextureCoordSets = gl_InterleavedTextureCoordSets;
+   table->SelectTextureSGIS = gl_SelectTextureSGIS;
+   table->SelectTexture = gl_SelectTexture;
+   table->SelectTextureCoordSet = gl_SelectTextureCoordSet;
+   table->SelectTextureTransform = gl_SelectTextureTransform;
 }
 
 
@@ -375,6 +349,8 @@ static void init_dlist_pointers( struct gl_api_table *table )
    table->ColorMask = gl_save_ColorMask;
    table->ColorMaterial = gl_save_ColorMaterial;
    table->ColorPointer = gl_ColorPointer;
+   table->ColorTable = gl_save_ColorTable;
+   table->ColorSubTable = gl_save_ColorSubTable;
    table->CopyPixels = gl_save_CopyPixels;
    table->CopyTexImage1D = gl_save_CopyTexImage1D;
    table->CopyTexImage2D = gl_save_CopyTexImage2D;
@@ -392,7 +368,8 @@ static void init_dlist_pointers( struct gl_api_table *table )
    table->DrawArrays = gl_save_DrawArrays;
    table->DrawBuffer = gl_save_DrawBuffer;
    table->DrawElements = gl_save_DrawElements;
-   table->DrawPixels = gl_DrawPixels;   /* SPECIAL CASE */
+   table->DrawPixels = gl_save_DrawPixels;
+   table->DrawRangeElements = gl_save_DrawRangeElements;
    table->EdgeFlag = gl_save_EdgeFlag;
    table->EdgeFlagPointer = gl_EdgeFlagPointer;
    table->Enable = gl_save_Enable;
@@ -417,6 +394,8 @@ static void init_dlist_pointers( struct gl_api_table *table )
    /* NONE OF THESE COMMANDS ARE COMPILED INTO DISPLAY LISTS */
    table->GetBooleanv = gl_GetBooleanv;
    table->GetClipPlane = gl_GetClipPlane;
+   table->GetColorTable = gl_GetColorTable;
+   table->GetColorTableParameteriv = gl_GetColorTableParameteriv;
    table->GetDoublev = gl_GetDoublev;
    table->GetError = gl_GetError;
    table->GetFloatv = gl_GetFloatv;
@@ -476,6 +455,7 @@ static void init_dlist_pointers( struct gl_api_table *table )
    table->Normal3fv = gl_save_Normal3fv;
    table->NormalPointer = gl_NormalPointer;  /* NOT SAVED */
    table->Ortho = gl_save_Ortho;
+   table->PointParameterfvEXT = gl_save_PointParameterfvEXT;
    table->PassThrough = gl_save_PassThrough;
    table->PixelMapfv = gl_save_PixelMapfv;
    table->PixelStorei = gl_PixelStorei;   /* NOT SAVED */
@@ -527,18 +507,27 @@ static void init_dlist_pointers( struct gl_api_table *table )
    table->VertexPointer = gl_VertexPointer;  /* NOT SAVED */
    table->Viewport = gl_save_Viewport;
 
-   /* GL_SAVE_MESA_window_pos extension */
+   /* GL_MESA_window_pos extension */
    table->WindowPos4fMESA = gl_save_WindowPos4fMESA;
 
    /* GL_MESA_resize_buffers extension */
    table->ResizeBuffersMESA = gl_ResizeBuffersMESA;
+
+   /* GL_SGIS_multitexture */
+   table->MultiTexCoord4f = gl_save_MultiTexCoord4f;
+   table->MultiTexCoordPointer = gl_MultiTexCoordPointer;   /* NOT SAVED */
+   table->InterleavedTextureCoordSets = gl_InterleavedTextureCoordSets;  /* NOT SAVED */
+   table->SelectTextureSGIS = gl_save_SelectTextureSGIS;
+   table->SelectTexture = gl_save_SelectTexture;
+   table->SelectTextureCoordSet = gl_save_SelectTextureCoordSet;
+   table->SelectTextureTransform = gl_save_SelectTextureTransform;
 }
 
 
 
 void gl_init_api_function_pointers( GLcontext *ctx )
 {
-   init_exec_pointers( &ctx->Exec );
+   init_execution_pointers( &ctx->Exec );
 
    init_dlist_pointers( &ctx->Save );
 
