@@ -1,9 +1,9 @@
-/* $Id: points.c,v 1.14 1997/10/29 01:29:09 brianp Exp $ */
+/* $Id: points.c,v 3.5 1998/07/29 04:08:09 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.5
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,47 +23,23 @@
 
 /*
  * $Log: points.c,v $
- * Revision 1.14  1997/10/29 01:29:09  brianp
- * added GL_EXT_point_parameters extension from Daniel Barrero
+ * Revision 3.5  1998/07/29 04:08:09  brianp
+ * feedback returned wrong point color
  *
- * Revision 1.13  1997/07/24 01:23:44  brianp
- * changed precompiled header symbol from PCH to PC_HEADER
+ * Revision 3.4  1998/06/07 22:18:52  brianp
+ * implemented GL_EXT_multitexture extension
  *
- * Revision 1.12  1997/06/20 02:50:39  brianp
- * changed color components from GLfixed to GLubyte
+ * Revision 3.3  1998/03/27 04:17:31  brianp
+ * fixed G++ warnings
  *
- * Revision 1.11  1997/05/28 03:26:02  brianp
- * added precompiled header (PCH) support
+ * Revision 3.2  1998/02/08 20:19:12  brianp
+ * removed unneeded headers
  *
- * Revision 1.10  1997/05/03 00:51:02  brianp
- * removed calls to gl_texturing_enabled()
+ * Revision 3.1  1998/02/04 00:44:29  brianp
+ * fixed casts and conditional expression problems for Amiga StormC compiler
  *
- * Revision 1.9  1997/04/14 02:00:39  brianp
- * #include "texstate.h" instead of "texture.h"
- *
- * Revision 1.8  1997/04/12 12:24:43  brianp
- * replaced ctx->PointsFunc with ctx->Driver.PointsFunc
- *
- * Revision 1.7  1997/04/02 03:11:38  brianp
- * replaced VB->Unclipped with VB->ClipMask
- *
- * Revision 1.6  1997/03/08 02:04:27  brianp
- * better implementation of feedback function
- *
- * Revision 1.5  1997/02/09 18:43:52  brianp
- * added GL_EXT_texture3D support
- *
- * Revision 1.4  1997/01/09 19:48:00  brianp
- * now call gl_texturing_enabled()
- *
- * Revision 1.3  1996/11/08 02:21:21  brianp
- * added null drawing function for GL_NO_RASTER
- *
- * Revision 1.2  1996/09/15 14:18:37  brianp
- * now use GLframebuffer and GLvisual
- *
- * Revision 1.1  1996/09/13 01:38:16  brianp
- * Initial revision
+ * Revision 3.0  1998/01/31 21:01:27  brianp
+ * initial rev
  *
  */
 
@@ -73,7 +49,6 @@
 #else
 #include "context.h"
 #include "feedback.h"
-#include "dlist.h"
 #include "macros.h"
 #include "pb.h"
 #include "span.h"
@@ -158,44 +133,31 @@ void gl_PointParameterfvEXT( GLcontext *ctx, GLenum pname,
 static void feedback_points( GLcontext *ctx, GLuint first, GLuint last )
 {
    struct vertex_buffer *VB = ctx->VB;
+   GLuint texSet = ctx->Texture.CurrentTransformSet;
    GLuint i;
-   GLfloat invRedScale   = ctx->Visual->InvRedScale;
-   GLfloat invGreenScale = ctx->Visual->InvGreenScale;
-   GLfloat invBlueScale  = ctx->Visual->InvBlueScale;
-   GLfloat invAlphaScale = ctx->Visual->InvAlphaScale;
 
    for (i=first;i<=last;i++) {
       if (VB->ClipMask[i]==0) {
          GLfloat x, y, z, w, invq;
          GLfloat color[4], texcoord[4];
-
          x = VB->Win[i][0];
          y = VB->Win[i][1];
          z = VB->Win[i][2] / DEPTH_SCALE;
          w = VB->Clip[i][3];
 
          /* convert color from integer back to a float in [0,1] */
-         if (ctx->Light.ShadeModel==GL_SMOOTH) {
-            /* smooth shading - colors are in fixed point */
-            color[0] = FixedToFloat(VB->Color[i][0]) * invRedScale;
-            color[1] = FixedToFloat(VB->Color[i][1]) * invGreenScale;
-            color[2] = FixedToFloat(VB->Color[i][2]) * invBlueScale;
-            color[3] = FixedToFloat(VB->Color[i][3]) * invAlphaScale;
-         }
-         else {
-            /* flat shading - colors are integers */
-            color[0] = VB->Color[i][0] * invRedScale;
-            color[1] = VB->Color[i][1] * invGreenScale;
-            color[2] = VB->Color[i][2] * invBlueScale;
-            color[3] = VB->Color[i][3] * invAlphaScale;
-         }
-         invq = 1.0F / VB->TexCoord[i][3];
-         texcoord[0] = VB->TexCoord[i][0] * invq;
-         texcoord[1] = VB->TexCoord[i][1] * invq;
-         texcoord[2] = VB->TexCoord[i][2] * invq;
-         texcoord[3] = VB->TexCoord[i][3];
+         color[0] = VB->Color[i][0] * (1.0F / 255.0F);
+         color[1] = VB->Color[i][1] * (1.0F / 255.0F);
+         color[2] = VB->Color[i][2] * (1.0F / 255.0F);
+         color[3] = VB->Color[i][3] * (1.0F / 255.0F);
 
-         FEEDBACK_TOKEN( ctx, (GLfloat) GL_POINT_TOKEN );
+         invq = 1.0F / VB->MultiTexCoord[texSet][i][3];
+         texcoord[0] = VB->MultiTexCoord[texSet][i][0] * invq;
+         texcoord[1] = VB->MultiTexCoord[texSet][i][1] * invq;
+         texcoord[2] = VB->MultiTexCoord[texSet][i][2] * invq;
+         texcoord[3] = VB->MultiTexCoord[texSet][i][3];
+
+         FEEDBACK_TOKEN( ctx, (GLfloat) (GLint) GL_POINT_TOKEN );
          gl_feedback_vertex( ctx, x, y, z, w, color,
                              (GLfloat) VB->Index[i], texcoord );
       }
@@ -561,8 +523,13 @@ static void antialiased_rgba_points( GLcontext *ctx,
  */
 static void null_points( GLcontext *ctx, GLuint first, GLuint last )
 {
+   (void) ctx;
+   (void) first;
+   (void) last;
 }
 
+
+
 /* Definition of the functions for GL_EXT_point_parameters */
 
 /*
@@ -658,15 +625,16 @@ static void dist_atten_general_rgba_points( GLcontext *ctx, GLuint first,
          y = (GLint)  VB->Win[i][1];
          z = (GLint) (VB->Win[i][2] + ctx->PointZoffset);
          dsize=psize*dist_attenuation(ctx,VB->Eye[i]);
-         if(dsize>=ctx->Point.Threshold) {
-            isize=(GLint) (MIN2(dsize,ctx->Point.MaxSize)+0.5F);
-            alpha=VB->Color[i][3];
-         } else {
-            isize=(GLint) (MAX2(ctx->Point.Threshold,ctx->Point.MinSize)+0.5F);
-            dsize/=ctx->Point.Threshold;
-            alpha=VB->Color[i][3]* (dsize*dsize);
+         if (dsize >= ctx->Point.Threshold) {
+            isize = (GLint) (MIN2(dsize,ctx->Point.MaxSize)+0.5F);
+            alpha = VB->Color[i][3];
          }
-         if (isize&1) {
+         else {
+            isize = (GLint) (MAX2(ctx->Point.Threshold,ctx->Point.MinSize)+0.5F);
+            dsize /= ctx->Point.Threshold;
+            alpha = (GLint) (VB->Color[i][3]* (dsize*dsize));
+         }
+         if (isize & 1) {
             /* odd size */
             x0 = x - isize/2;
             x1 = x + isize/2;
@@ -728,7 +696,7 @@ static void dist_atten_textured_rgba_points( GLcontext *ctx, GLuint first,
          } else {
             isize=(GLint) (MAX2(ctx->Point.Threshold,ctx->Point.MinSize)+0.5F);
             dsize/=ctx->Point.Threshold;
-            alpha=VB->Color[i][3]* (dsize*dsize);
+            alpha = (GLint) (VB->Color[i][3]* (dsize*dsize));
          }
 
          if (isize<1) {
@@ -834,7 +802,7 @@ static void dist_atten_antialiased_rgba_points( GLcontext *ctx,
                         /* coverage is in [0,256] */
                         alpha = (alpha * coverage) >> 8;
                      }
-                     alpha*=alphaf;
+                     alpha = (GLint) (alpha * alphaf);
                      PB_WRITE_TEX_PIXEL( PB, x,y,z, red, green, blue, alpha, s,
 t, u );
                   }
@@ -889,7 +857,7 @@ t, u );
                         /* coverage is in [0,256] */
                         alpha = (alpha * coverage) >> 8;
                      }
-                     alpha*=alphaf;
+                     alpha = (GLint) (alpha * alphaf);
                      PB_WRITE_RGBA_PIXEL( PB, x, y, z, red, green, blue, alpha )
 ;
                   }
@@ -929,13 +897,17 @@ void gl_set_point_function( GLcontext *ctx )
          }
          else if (ctx->Point.Size==1.0) {
             /* size=1, any raster ops */
-            ctx->Driver.PointsFunc = rgbmode ? size1_rgba_points
-                                             : size1_ci_points;
+            if (rgbmode)
+               ctx->Driver.PointsFunc = size1_rgba_points;
+            else
+               ctx->Driver.PointsFunc = size1_ci_points;
          }
          else {
 	    /* every other kind of point rendering */
-	    ctx->Driver.PointsFunc = rgbmode ? general_rgba_points
-                                             : general_ci_points;
+            if (rgbmode)
+               ctx->Driver.PointsFunc = general_rgba_points;
+            else
+               ctx->Driver.PointsFunc = general_ci_points;
          }
       } 
       else if(ctx->Point.SmoothFlag && rgbmode) {
@@ -946,8 +918,10 @@ void gl_set_point_function( GLcontext *ctx )
       } 
       else {
          /* every other kind of point rendering */
-         ctx->Driver.PointsFunc = rgbmode ? dist_atten_general_rgba_points
-                                          : dist_atten_general_ci_points;
+         if (rgbmode)
+            ctx->Driver.PointsFunc = dist_atten_general_rgba_points;
+         else
+            ctx->Driver.PointsFunc = dist_atten_general_ci_points;
      }
    }
    else if (ctx->RenderMode==GL_FEEDBACK) {
