@@ -1,9 +1,9 @@
-/* $Id: scissor.c,v 1.5 1997/07/24 01:21:56 brianp Exp $ */
+/* $Id: scissor.c,v 3.1 1998/02/08 20:17:42 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,20 +23,11 @@
 
 /*
  * $Log: scissor.c,v $
- * Revision 1.5  1997/07/24 01:21:56  brianp
- * changed precompiled header symbol from PCH to PC_HEADER
+ * Revision 3.1  1998/02/08 20:17:42  brianp
+ * removed unneeded headers
  *
- * Revision 1.4  1997/05/28 03:26:29  brianp
- * added precompiled header (PCH) support
- *
- * Revision 1.3  1997/05/17 03:17:50  brianp
- * faster gl_scissor_span() from Mats Lofkvist
- *
- * Revision 1.2  1996/09/15 14:18:37  brianp
- * now use GLframebuffer and GLvisual
- *
- * Revision 1.1  1996/09/13 01:38:16  brianp
- * Initial revision
+ * Revision 3.0  1998/01/31 21:03:42  brianp
+ * initial rev
  *
  */
 
@@ -46,7 +37,6 @@
 #else
 #include "context.h"
 #include "macros.h"
-#include "dlist.h"
 #include "scissor.h"
 #include "types.h"
 #endif
@@ -64,12 +54,14 @@ void gl_Scissor( GLcontext *ctx,
       return;
    }
 
-   ctx->Scissor.X = x;
-   ctx->Scissor.Y = y;
-   ctx->Scissor.Width = width;
-   ctx->Scissor.Height = height;
-
-   ctx->NewState |= NEW_ALL;  /* TODO: this is overkill */
+   if (x!=ctx->Scissor.X || y!=ctx->Scissor.Y || 
+       width!=ctx->Scissor.Width || height!=ctx->Scissor.Height) {
+      ctx->Scissor.X = x;
+      ctx->Scissor.Y = y;
+      ctx->Scissor.Width = width;
+      ctx->Scissor.Height = height;
+      ctx->NewState |= NEW_ALL;  /* TODO: this is overkill */
+   }
 }
 
 
