@@ -1,9 +1,9 @@
-/* $Id: stencil.c,v 1.7 1997/07/24 01:21:56 brianp Exp $ */
+/* $Id: stencil.c,v 3.2 1998/03/27 04:17:31 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,26 +23,14 @@
 
 /*
  * $Log: stencil.c,v $
- * Revision 1.7  1997/07/24 01:21:56  brianp
- * changed precompiled header symbol from PCH to PC_HEADER
+ * Revision 3.2  1998/03/27 04:17:31  brianp
+ * fixed G++ warnings
  *
- * Revision 1.6  1997/05/28 03:26:29  brianp
- * added precompiled header (PCH) support
+ * Revision 3.1  1998/02/08 20:17:42  brianp
+ * removed unneeded headers
  *
- * Revision 1.5  1997/04/29 01:26:37  brianp
- * fixed a few return statements which were missing values
- *
- * Revision 1.4  1997/04/20 20:29:11  brianp
- * replaced abort() with gl_problem()
- *
- * Revision 1.3  1997/02/27 19:58:35  brianp
- * don't try to clear stencil buffer if there isn't one
- *
- * Revision 1.2  1996/09/15 14:18:55  brianp
- * now use GLframebuffer and GLvisual
- *
- * Revision 1.1  1996/09/13 01:38:16  brianp
- * Initial revision
+ * Revision 3.0  1998/01/31 21:03:42  brianp
+ * initial rev
  *
  */
 
@@ -53,7 +41,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include "context.h"
-#include "dlist.h"
 #include "macros.h"
 #include "pb.h"
 #include "stencil.h"
@@ -205,7 +192,7 @@ static void apply_stencil_op_to_span( GLcontext *ctx,
                                       GLuint n, GLint x, GLint y,
 				      GLenum oper, GLubyte mask[] )
 {
-   GLint i;
+   GLuint i;
    GLstencil s, ref;
    GLstencil wrtmask, invmask;
    GLstencil *stencil;
@@ -532,7 +519,8 @@ void gl_depth_stencil_span( GLcontext *ctx,
       }
 
       /* apply the depth test */
-      (*ctx->Driver.DepthTestSpan)( ctx, n, x, y, z, mask );
+      if (ctx->Driver.DepthTestSpan)
+         (*ctx->Driver.DepthTestSpan)( ctx, n, x, y, z, mask );
 
       /* set the stencil pass/fail flags according to result of depth test */
       for (i=0;i<n;i++) {
@@ -568,7 +556,7 @@ static void apply_stencil_op_to_pixels( GLcontext *ctx,
 				        const GLint y[],
 				        GLenum oper, GLubyte mask[] )
 {
-   GLint i;
+   GLuint i;
    GLstencil ref;
    GLstencil wrtmask, invmask;
 
@@ -899,7 +887,8 @@ void gl_depth_stencil_pixels( GLcontext *ctx,
       }
 
       /* apply the depth test */
-      (*ctx->Driver.DepthTestPixels)( ctx, n, x, y, z, mask );
+      if (ctx->Driver.DepthTestPixels)
+         (*ctx->Driver.DepthTestPixels)( ctx, n, x, y, z, mask );
 
       /* set the stencil pass/fail flags according to result of depth test */
       for (i=0;i<n;i++) {
