@@ -1,9 +1,9 @@
-/* $Id: vbfill.c,v 1.16 1997/08/13 01:31:11 brianp Exp $ */
+/* $Id: vbfill.c,v 3.6 1998/06/07 22:18:52 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.4
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,53 +23,26 @@
 
 /*
  * $Log: vbfill.c,v $
- * Revision 1.16  1997/08/13 01:31:11  brianp
- * LightTwoSide is now a GLboolean
+ * Revision 3.6  1998/06/07 22:18:52  brianp
+ * implemented GL_EXT_multitexture extension
  *
- * Revision 1.15  1997/07/24 01:25:27  brianp
- * changed precompiled header symbol from PCH to PC_HEADER
+ * Revision 3.5  1998/04/18 05:00:42  brianp
+ * moved FLOAT_COLOR_TO_UBYTE_COLOR macro to mmath.h
  *
- * Revision 1.14  1997/06/20 02:47:41  brianp
- * added Color4ubv API pointer
+ * Revision 3.4  1998/03/27 04:26:44  brianp
+ * fixed G++ warnings
  *
- * Revision 1.13  1997/06/20 02:46:49  brianp
- * changed color components from GLfixed to GLubyte
+ * Revision 3.3  1998/02/20 04:53:07  brianp
+ * implemented GL_SGIS_multitexture
  *
- * Revision 1.12  1997/05/28 03:26:49  brianp
- * added precompiled header (PCH) support
+ * Revision 3.2  1998/02/08 20:17:42  brianp
+ * removed unneeded headers
  *
- * Revision 1.11  1997/05/27 03:13:41  brianp
- * removed some debugging code
+ * Revision 3.1  1998/02/02 03:09:34  brianp
+ * added GL_LIGHT_MODEL_COLOR_CONTROL (separate specular color interpolation)
  *
- * Revision 1.10  1997/04/28 02:05:44  brianp
- * renamed some vertex functions, also save color with texcoords
- *
- * Revision 1.9  1997/04/24 01:50:53  brianp
- * optimized glColor3f, glColor3fv, glColor4fv
- *
- * Revision 1.8  1997/04/24 00:30:17  brianp
- * optimized glTexCoord2() code
- *
- * Revision 1.7  1997/04/20 15:59:30  brianp
- * removed VERTEX2_BIT stuff
- *
- * Revision 1.6  1997/04/16 23:55:33  brianp
- * added optimized glTexCoord2f code
- *
- * Revision 1.5  1997/04/14 22:18:23  brianp
- * added optimized glVertex3fv code
- *
- * Revision 1.4  1997/04/12 16:21:54  brianp
- * added ctx->Exec.Vertex2f = vertex2_feedback; statement
- *
- * Revision 1.3  1997/04/12 12:23:26  brianp
- * fixed 3 bugs in gl_eval_vertex
- *
- * Revision 1.2  1997/04/07 03:01:11  brianp
- * optimized vertex[234] code
- *
- * Revision 1.1  1997/04/02 03:13:56  brianp
- * Initial revision
+ * Revision 3.0  1998/01/31 21:06:45  brianp
+ * initial rev
  *
  */
 
@@ -86,11 +59,9 @@
 #include <assert.h>
 #include "context.h"
 #include "light.h"
-#include "clip.h"
-#include "dlist.h"
-#include "feedback.h"
 #include "macros.h"
 #include "matrix.h"
+#include "mmath.h"
 #include "pb.h"
 #include "types.h"
 #include "vb.h"
@@ -159,15 +130,16 @@ void gl_Indexi( GLcontext *ctx, GLint c )
 /******                     glColor functions                     *****/
 /**********************************************************************/
 
+
 /*
  * Caller:  context->API.Color3f pointer.
  */
 void gl_Color3f( GLcontext *ctx, GLfloat red, GLfloat green, GLfloat blue )
 {
-   ctx->Current.ByteColor[0] = red   * ctx->Visual->RedScale;
-   ctx->Current.ByteColor[1] = green * ctx->Visual->GreenScale;
-   ctx->Current.ByteColor[2] = blue  * ctx->Visual->BlueScale;
-   ctx->Current.ByteColor[3] = ctx->Visual->AlphaScale;
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[0], red);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[1], green);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[2], blue);
+   ctx->Current.ByteColor[3] = 255;
    ASSERT( !ctx->Light.ColorMaterialEnabled );
    ctx->VB->MonoColor = GL_FALSE;
 }
@@ -178,26 +150,26 @@ void gl_Color3f( GLcontext *ctx, GLfloat red, GLfloat green, GLfloat blue )
  */
 void gl_Color3fv( GLcontext *ctx, const GLfloat *c )
 {
-   ctx->Current.ByteColor[0] = c[0] * ctx->Visual->RedScale;
-   ctx->Current.ByteColor[1] = c[1] * ctx->Visual->GreenScale;
-   ctx->Current.ByteColor[2] = c[2] * ctx->Visual->BlueScale;
-   ctx->Current.ByteColor[3] = ctx->Visual->AlphaScale;
-
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[0], c[0]);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[1], c[1]);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[2], c[2]);
+   ctx->Current.ByteColor[3] = 255;
    ASSERT( !ctx->Light.ColorMaterialEnabled );
    ctx->VB->MonoColor = GL_FALSE;
 }
 
 
+
 /*
  * Caller:  context->API.Color4f pointer.
  */
 void gl_Color4f( GLcontext *ctx,
                  GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha )
 {
-   ctx->Current.ByteColor[0] = red   * ctx->Visual->RedScale;
-   ctx->Current.ByteColor[1] = green * ctx->Visual->GreenScale;
-   ctx->Current.ByteColor[2] = blue  * ctx->Visual->BlueScale;
-   ctx->Current.ByteColor[3] = alpha * ctx->Visual->AlphaScale;
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[0], red);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[1], green);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[2], blue);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[3], alpha);
    ASSERT( !ctx->Light.ColorMaterialEnabled );
    ctx->VB->MonoColor = GL_FALSE;
 }
@@ -208,37 +180,20 @@ void gl_Color4f( GLcontext *ctx,
  */
 void gl_Color4fv( GLcontext *ctx, const GLfloat *c )
 {
-   ctx->Current.ByteColor[0] = c[0] * ctx->Visual->RedScale;
-   ctx->Current.ByteColor[1] = c[1] * ctx->Visual->GreenScale;
-   ctx->Current.ByteColor[2] = c[2] * ctx->Visual->BlueScale;
-   ctx->Current.ByteColor[3] = c[3] * ctx->Visual->AlphaScale;
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[0], c[0]);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[1], c[1]);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[2], c[2]);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[3], c[3]);
    ASSERT( !ctx->Light.ColorMaterialEnabled );
    ctx->VB->MonoColor = GL_FALSE;
 }
 
 
 /*
- * Used when colors are not scaled to [0,255]
  * Caller:  context->API.Color4ub pointer.
  */
 void gl_Color4ub( GLcontext *ctx,
                   GLubyte red, GLubyte green, GLubyte blue, GLubyte alpha )
-{
-   ctx->Current.ByteColor[0] = red   * ctx->Visual->RedScale   * (1.0F/255.0F);
-   ctx->Current.ByteColor[1] = green * ctx->Visual->GreenScale * (1.0F/255.0F);
-   ctx->Current.ByteColor[2] = blue  * ctx->Visual->BlueScale  * (1.0F/255.0F);
-   ctx->Current.ByteColor[3] = alpha * ctx->Visual->AlphaScale * (1.0F/255.0F);
-   ASSERT( !ctx->Light.ColorMaterialEnabled );
-   ctx->VB->MonoColor = GL_FALSE;
-}
-
-
-/*
- * Used when colors are scaled to [0,255].
- * Caller:  context->API.Color4ub pointer.
- */
-void gl_Color4ub8bit( GLcontext *ctx,
-                      GLubyte red, GLubyte green, GLubyte blue, GLubyte alpha )
 {
    ASSIGN_4V( ctx->Current.ByteColor, red, green, blue, alpha );
    ASSERT( !ctx->Light.ColorMaterialEnabled );
@@ -247,25 +202,9 @@ void gl_Color4ub8bit( GLcontext *ctx,
 
 
 /*
- * Used when colors are not scaled to [0,255]
  * Caller:  context->API.Color4ub pointer.
  */
 void gl_Color4ubv( GLcontext *ctx, const GLubyte *c )
-{
-   ctx->Current.ByteColor[0] = c[0] * ctx->Visual->RedScale   * (1.0F/255.0F);
-   ctx->Current.ByteColor[1] = c[1] * ctx->Visual->GreenScale * (1.0F/255.0F);
-   ctx->Current.ByteColor[2] = c[2] * ctx->Visual->BlueScale  * (1.0F/255.0F);
-   ctx->Current.ByteColor[3] = c[3] * ctx->Visual->AlphaScale * (1.0F/255.0F);
-   ASSERT( !ctx->Light.ColorMaterialEnabled );
-   ctx->VB->MonoColor = GL_FALSE;
-}
-
-
-/*
- * Used when colors are scaled to [0,255].
- * Caller:  context->API.Color4ub pointer.
- */
-void gl_Color4ubv8bit( GLcontext *ctx, const GLubyte *c )
 {
    COPY_4UBV( ctx->Current.ByteColor, c );
    ASSERT( !ctx->Light.ColorMaterialEnabled );
@@ -280,10 +219,10 @@ void gl_Color4ubv8bit( GLcontext *ctx, const GLubyte *c )
 void gl_ColorMat3f( GLcontext *ctx, GLfloat red, GLfloat green, GLfloat blue )
 {
    GLfloat color[4];
-   ctx->Current.ByteColor[0] = red   * ctx->Visual->RedScale;
-   ctx->Current.ByteColor[1] = green * ctx->Visual->GreenScale;
-   ctx->Current.ByteColor[2] = blue  * ctx->Visual->BlueScale;
-   ctx->Current.ByteColor[3] = ctx->Visual->AlphaScale;
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[0], red);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[1], green);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[2], blue);
+   ctx->Current.ByteColor[3] = 255;
    /* update material */
    ASSERT( ctx->Light.ColorMaterialEnabled );
    ASSIGN_4V( color, red, green, blue, 1.0F );
@@ -299,10 +238,10 @@ void gl_ColorMat3f( GLcontext *ctx, GLfloat red, GLfloat green, GLfloat blue )
 void gl_ColorMat3fv( GLcontext *ctx, const GLfloat *c )
 {
    GLfloat color[4];
-   ctx->Current.ByteColor[0] = c[0] * ctx->Visual->RedScale;
-   ctx->Current.ByteColor[1] = c[1] * ctx->Visual->GreenScale;
-   ctx->Current.ByteColor[2] = c[2] * ctx->Visual->BlueScale;
-   ctx->Current.ByteColor[3] = ctx->Visual->AlphaScale;
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[0], c[0]);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[1], c[1]);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[2], c[2]);
+   ctx->Current.ByteColor[3] = 255;
    /* update material */
    ASSERT( ctx->Light.ColorMaterialEnabled );
    ASSIGN_4V( color, c[0], c[1], c[2], 1.0F );
@@ -319,10 +258,10 @@ void gl_ColorMat4f( GLcontext *ctx,
                     GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha )
 {
    GLfloat color[4];
-   ctx->Current.ByteColor[0] = red   * ctx->Visual->RedScale;
-   ctx->Current.ByteColor[1] = green * ctx->Visual->GreenScale;
-   ctx->Current.ByteColor[2] = blue  * ctx->Visual->BlueScale;
-   ctx->Current.ByteColor[3] = alpha * ctx->Visual->AlphaScale;
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[0], red);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[1], green);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[2], blue);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[3], alpha);
    /* update material */
    ASSERT( ctx->Light.ColorMaterialEnabled );
    ASSIGN_4V( color, red, green, blue, alpha );
@@ -338,10 +277,10 @@ void gl_ColorMat4f( GLcontext *ctx,
 void gl_ColorMat4fv( GLcontext *ctx, const GLfloat *c )
 {
    GLfloat color[4];
-   ctx->Current.ByteColor[0] = c[0] * ctx->Visual->RedScale;
-   ctx->Current.ByteColor[1] = c[1] * ctx->Visual->GreenScale;
-   ctx->Current.ByteColor[2] = c[2] * ctx->Visual->BlueScale;
-   ctx->Current.ByteColor[3] = c[3] * ctx->Visual->AlphaScale;
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[0], c[0]);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[1], c[1]);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[2], c[2]);
+   FLOAT_COLOR_TO_UBYTE_COLOR(ctx->Current.ByteColor[3], c[3]);
    /* update material */
    ASSERT( ctx->Light.ColorMaterialEnabled );
    ASSIGN_4V( color, c[0], c[1], c[2], c[3] );
@@ -358,15 +297,7 @@ void gl_ColorMat4ub( GLcontext *ctx,
                      GLubyte red, GLubyte green, GLubyte blue, GLubyte alpha )
 {
    GLfloat color[4];
-   if (ctx->Visual->EightBitColor) {
-      ASSIGN_4V( ctx->Current.ByteColor, red, green, blue, alpha );
-   }
-   else {
-      ctx->Current.ByteColor[0] = red   * ctx->Visual->RedScale   * (1.0F/255.0F);
-      ctx->Current.ByteColor[1] = green * ctx->Visual->GreenScale * (1.0F/255.0F);
-      ctx->Current.ByteColor[2] = blue  * ctx->Visual->BlueScale  * (1.0F/255.0F);
-      ctx->Current.ByteColor[3] = alpha * ctx->Visual->AlphaScale * (1.0F/255.0F);
-   }
+   ASSIGN_4V( ctx->Current.ByteColor, red, green, blue, alpha );
    /* update material */
    ASSERT( ctx->Light.ColorMaterialEnabled );
    color[0] = red   * (1.0F/255.0F);
@@ -525,7 +456,9 @@ static void vertex3f_normal_color_tex4( GLcontext *ctx,
    ASSIGN_3V( VB->Obj[count], x, y, z );
    COPY_4UBV( VB->Fcolor[count], ctx->Current.ByteColor );
    COPY_3V( VB->Normal[count], ctx->Current.Normal );
-   COPY_4V( VB->TexCoord[count], ctx->Current.TexCoord );
+   /* GL_SGIS_multitexture */
+   COPY_4V( VB->MultiTexCoord[0][count], ctx->Current.MultiTexCoord[0] );
+   COPY_4V( VB->MultiTexCoord[1][count], ctx->Current.MultiTexCoord[1] );
    VB->Edgeflag[count] = ctx->Current.EdgeFlag;
 
    count++;
@@ -681,7 +614,9 @@ static void vertex2f_normal_color_tex4( GLcontext *ctx, GLfloat x, GLfloat y )
    ASSIGN_3V( VB->Obj[count], x, y, 0.0F );
    COPY_4UBV( VB->Fcolor[count], ctx->Current.ByteColor );
    COPY_3V( VB->Normal[count], ctx->Current.Normal );
-   COPY_4V( VB->TexCoord[count], ctx->Current.TexCoord );
+   /* GL_SGIS_multitexture */
+   COPY_4V( VB->MultiTexCoord[0][count], ctx->Current.MultiTexCoord[0] );
+   COPY_4V( VB->MultiTexCoord[1][count], ctx->Current.MultiTexCoord[1] );
    VB->Edgeflag[count] = ctx->Current.EdgeFlag;
 
    count++;
@@ -836,7 +771,9 @@ static void vertex3fv_normal_color_tex4( GLcontext *ctx, const GLfloat v[3] )
    COPY_3V( VB->Obj[count], v );
    COPY_4UBV( VB->Fcolor[count], ctx->Current.ByteColor );
    COPY_3V( VB->Normal[count], ctx->Current.Normal );
-   COPY_4V( VB->TexCoord[count], ctx->Current.TexCoord );
+   /* GL_SGIS_multitexture */
+   COPY_4V( VB->MultiTexCoord[0][count], ctx->Current.MultiTexCoord[0] );
+   COPY_4V( VB->MultiTexCoord[1][count], ctx->Current.MultiTexCoord[1] );
    VB->Edgeflag[count] = ctx->Current.EdgeFlag;
 
    count++;
@@ -962,21 +899,31 @@ static void vertex3fv_index( GLcontext *ctx, const GLfloat v[3] )
 void gl_vertex4f_nop( GLcontext *ctx,
                       GLfloat x, GLfloat y, GLfloat z, GLfloat w )
 {
+   (void) x;
+   (void) y;
+   (void) z;
+   (void) w;
    gl_error( ctx, GL_INVALID_OPERATION, "glVertex4" );
 }
 
 void gl_vertex3f_nop( GLcontext *ctx, GLfloat x, GLfloat y, GLfloat z )
 {
+   (void) x;
+   (void) y;
+   (void) z;
    gl_error( ctx, GL_INVALID_OPERATION, "glVertex3" );
 }
 
 void gl_vertex2f_nop( GLcontext *ctx, GLfloat x, GLfloat y )
 {
+   (void) x;
+   (void) y;
    gl_error( ctx, GL_INVALID_OPERATION, "glVertex2" );
 }
 
 void gl_vertex3fv_nop( GLcontext *ctx, const GLfloat v[3] )
 {
+   (void) v;
    gl_error( ctx, GL_INVALID_OPERATION, "glVertex3v" );
 }
 
@@ -1028,6 +975,28 @@ void gl_TexCoord4f( GLcontext *ctx, GLfloat s, GLfloat t, GLfloat r, GLfloat q)
 }
 
 
+/* GL_SGIS_multitexture / GL_EXT_multitexture */
+void gl_MultiTexCoord4f( GLcontext *ctx, GLenum target,
+                         GLfloat s, GLfloat t, GLfloat r, GLfloat q )
+{
+   GLint texSet;
+   if (target >= GL_TEXTURE0_SGIS && target <= GL_TEXTURE1_SGIS) {
+      texSet = target - GL_TEXTURE0_SGIS;
+   }
+   else if (target >= GL_TEXTURE0_EXT && target <= GL_TEXTURE1_EXT) {
+      texSet = target - GL_TEXTURE0_EXT;
+   }
+   else {
+      gl_error(ctx, GL_INVALID_ENUM, "glMultiTexCoord(target)");
+      return;
+   }
+
+   ctx->Current.MultiTexCoord[texSet][0] = s;
+   ctx->Current.MultiTexCoord[texSet][1] = t;
+   ctx->Current.MultiTexCoord[texSet][2] = r;
+   ctx->Current.MultiTexCoord[texSet][3] = q;
+}
+
 
 
 /*
@@ -1046,7 +1015,13 @@ void gl_set_vertex_function( GLcontext *ctx )
    else {
       ctx->Exec.Vertex4f = vertex4;
       if (ctx->Visual->RGBAflag) {
-         if (ctx->NeedNormals) {
+         if (ctx->Texture.Enabled >= TEXTURE1_1D) {
+            /* Multi texture coords */
+            ctx->Exec.Vertex2f = vertex2f_normal_color_tex4;
+            ctx->Exec.Vertex3f = vertex3f_normal_color_tex4;
+            ctx->Exec.Vertex3fv = vertex3fv_normal_color_tex4;
+         }
+         else if (ctx->NeedNormals) {
             /* lighting enabled, need normal vectors */
             if (ctx->Texture.Enabled) {
                if (ctx->VB->TexCoordSize==2) {
@@ -1134,10 +1109,8 @@ void gl_set_color_function( GLcontext *ctx )
       ctx->Exec.Color3fv = gl_Color3fv;
       ctx->Exec.Color4f = gl_Color4f;
       ctx->Exec.Color4fv = gl_Color4fv;
-      ctx->Exec.Color4ub
-         = ctx->Visual->EightBitColor ? gl_Color4ub8bit : gl_Color4ub;
-      ctx->Exec.Color4ubv
-         = ctx->Visual->EightBitColor ? gl_Color4ubv8bit : gl_Color4ubv;
+      ctx->Exec.Color4ub = gl_Color4ub;
+      ctx->Exec.Color4ubv = gl_Color4ubv;
    }
    if (!ctx->CompileFlag) {
       ctx->API.Color3f = ctx->Exec.Color3f;
@@ -1173,6 +1146,7 @@ void gl_eval_vertex( GLcontext *ctx,
 {
    struct vertex_buffer *VB = ctx->VB;
    GLuint count = VB->Count;  /* copy to local var to encourage optimization */
+   GLuint texSet = ctx->Texture.CurrentTransformSet;
 
    VB->VertexSizeMask = VERTEX4_BIT;
    VB->MonoNormal = GL_FALSE;
@@ -1184,15 +1158,15 @@ void gl_eval_vertex( GLcontext *ctx,
    if (ctx->Light.ColorMaterialEnabled
        && (ctx->Eval.Map1Color4 || ctx->Eval.Map2Color4)) {
       GLfloat fcolor[4];
-      fcolor[0] = color[0] * ctx->Visual->InvRedScale;
-      fcolor[1] = color[1] * ctx->Visual->InvGreenScale;
-      fcolor[2] = color[2] * ctx->Visual->InvBlueScale;
-      fcolor[3] = color[3] * ctx->Visual->InvAlphaScale;
+      fcolor[0] = color[0] * (1.0F / 255.0F);
+      fcolor[1] = color[1] * (1.0F / 255.0F);
+      fcolor[2] = color[2] * (1.0F / 255.0F);
+      fcolor[3] = color[3] * (1.0F / 255.0F);
       gl_set_material( ctx, ctx->Light.ColorMaterialBitmask, fcolor );
    }
 #endif
    VB->Findex[count] = index;
-   COPY_4V( VB->TexCoord[count], texcoord );
+   COPY_4V( VB->MultiTexCoord[texSet][count], texcoord );
    VB->Edgeflag[count] = ctx->Current.EdgeFlag;
 
    count++;
@@ -1269,6 +1243,7 @@ void gl_Begin( GLcontext *ctx, GLenum p )
    /* By default use front color/index.  Two-sided lighting may override. */
    VB->Color = VB->Fcolor;
    VB->Index = VB->Findex;
+   VB->Specular = VB->Fspec;
 
    switch (ctx->Primitive) {
       case GL_POINTS:
