@@ -1,9 +1,9 @@
-/* $Id: vbxform.c,v 1.20 1997/11/20 00:09:38 brianp Exp $ */
+/* $Id: vbxform.c,v 3.10 1998/08/23 22:19:30 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.5
- * Copyright (C) 1995-1997  Brian Paul
+ * Version:  3.0
+ * Copyright (C) 1995-1998  Brian Paul
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,75 +23,47 @@
 
 /*
  * $Log: vbxform.c,v $
- * Revision 1.20  1997/11/20 00:09:38  brianp
- * transform_points4() wasn't calling asm routines
+ * Revision 3.10  1998/08/23 22:19:30  brianp
+ * added DoViewportMapping to context struct
  *
- * Revision 1.19  1997/10/30 06:00:06  brianp
- * added Intel X86 assembly optimzations (Josh Vanderhoof)
+ * Revision 3.9  1998/07/09 03:15:41  brianp
+ * include asm_386.h instead of asm-386.h
  *
- * Revision 1.18  1997/10/15 00:36:36  brianp
- * renamed the FAST/REGULAR_MATH macros
+ * Revision 3.8  1998/06/22 03:16:28  brianp
+ * added MITS code
  *
- * Revision 1.17  1997/10/04 00:30:52  brianp
- * vertices specified with glVertex4 weren't always projected correctly
+ * Revision 3.7  1998/06/07 22:18:52  brianp
+ * implemented GL_EXT_multitexture extension
  *
- * Revision 1.16  1997/09/29 22:24:22  brianp
- * added REGULAR/FAST_MATH macros
+ * Revision 3.6  1998/04/18 05:01:18  brianp
+ * renamed USE_ASM to USE_X86_ASM
  *
- * Revision 1.15  1997/09/18 01:32:47  brianp
- * fixed divide by zero problem for "weird" projection matrices
+ * Revision 3.5  1998/03/28 04:01:53  brianp
+ * added CONST macro to fix IRIX compilation problems
  *
- * Revision 1.14  1997/09/10 00:28:11  brianp
- * fixed an optimization bug in viewport_map_vertices()
+ * Revision 3.4  1998/03/27 04:26:44  brianp
+ * fixed G++ warnings
  *
- * Revision 1.13  1997/07/24 01:25:27  brianp
- * changed precompiled header symbol from PCH to PC_HEADER
+ * Revision 3.3  1998/02/20 04:53:07  brianp
+ * implemented GL_SGIS_multitexture
  *
- * Revision 1.12  1997/06/20 02:57:59  brianp
- * changed color components from GLfixed to GLubyte
+ * Revision 3.2  1998/02/02 03:09:34  brianp
+ * added GL_LIGHT_MODEL_COLOR_CONTROL (separate specular color interpolation)
  *
- * Revision 1.11  1997/05/28 03:26:49  brianp
- * added precompiled header (PCH) support
+ * Revision 3.1  1998/02/01 16:37:19  brianp
+ * added GL_EXT_rescale_normal extension
  *
- * Revision 1.10  1997/05/23 03:01:45  brianp
- * commented out a few const keywords because IRIX cc chokes on them
- *
- * Revision 1.9  1997/04/29 01:31:07  brianp
- * added RasterSetup() function to device driver
- *
- * Revision 1.8  1997/04/21 01:21:52  brianp
- * added MATRIX_2D_NO_ROT
- *
- * Revision 1.7  1997/04/20 19:47:27  brianp
- * added RenderVB to device driver
- *
- * Revision 1.6  1997/04/20 15:59:30  brianp
- * removed VERTEX2_BIT stuff
- *
- * Revision 1.5  1997/04/14 02:12:53  brianp
- * small optimization in transform_texcoords()
- *
- * Revision 1.4  1997/04/12 16:22:22  brianp
- * removed gl_init_vb()
- *
- * Revision 1.3  1997/04/12 12:28:39  brianp
- * fixed <= material_update bug, removed some unused vars
- *
- * Revision 1.2  1997/04/07 03:01:11  brianp
- * optimized vertex[234] code
- *
- * Revision 1.1  1997/04/02 03:14:29  brianp
- * Initial revision
+ * Revision 3.0  1998/01/31 21:06:45  brianp
+ * initial rev
  *
  */
 
 
 /*
- * This file implements transformation, clip testing, and lighting of
+ * This file implements transformation, clip testing and projection of
  * vertices in the vertex buffer.
  *
  * The entry points to this file are the functions:
- *    gl_init_vb() - initialize the vertex buffer
  *    gl_transform_vb_part1() - first stage of vertex transformation
  *    gl_transform_vb_part2() - second stage of vertex transformation
  */
@@ -101,6 +73,7 @@
 #include "all.h"
 #else
 #include <stdlib.h>
+#include "asm_386.h"
 #include "context.h"
 #include "fog.h"
 #include "light.h"
@@ -117,7 +90,7 @@
 #endif
 
 
-#if 0
+#if 0  /* NOT USED AT THIS TIME */
 /*
  * Use the current modelview matrix to transform XY vertices from object
  * to eye coordinates.
@@ -220,9 +193,9 @@ static void transform_points2( GLcontext *ctx, GLuint n,
  * In/Out;  vEye - array [n][4] of eye coordinates
  */
 static void transform_points3( GLcontext *ctx, GLuint n,
-                               /*const*/ GLfloat vObj[][4], GLfloat vEye[][4] )
+                               CONST GLfloat vObj[][4], GLfloat vEye[][4] )
 {
-#ifndef USE_ASM
+#ifndef USE_X86_ASM
    START_FAST_MATH;
    switch (ctx->ModelViewMatrixType) {
       case MATRIX_GENERAL:
@@ -340,9 +313,9 @@ static void transform_points3( GLcontext *ctx, GLuint n,
  * In/Out;  vEye - array [n][4] of eye coordinates
  */
 static void transform_points4( GLcontext *ctx, GLuint n,
-                               /*const*/ GLfloat vObj[][4], GLfloat vEye[][4] )
+                               CONST GLfloat vObj[][4], GLfloat vEye[][4] )
 {
-#ifndef USE_ASM
+#ifndef USE_X86_ASM
    START_FAST_MATH;
    switch (ctx->ModelViewMatrixType) {
       case MATRIX_GENERAL:
@@ -459,16 +432,18 @@ static void transform_points4( GLcontext *ctx, GLuint n,
  * Transform an array of texture coordinates by the current texture matrix.
  * Input:  ctx - the context
  *         n - number of texture coordinates in array
+ *         texSet - which texture matrix to use
  * In/Out:  t - array [n][4] of texture coordinates to transform
  */
-static void transform_texcoords( GLcontext *ctx, GLuint n, GLfloat t[][4] )
+static void transform_texcoords( GLcontext *ctx, GLuint n, GLfloat t[][4],
+                                 GLuint texSet )
 {
-#ifndef USE_ASM
+#ifndef USE_X86_ASM
    START_FAST_MATH;
-   switch (ctx->TextureMatrixType) {
+   switch (ctx->TextureMatrixType[texSet]) {
       case MATRIX_GENERAL:
          {
-            const GLfloat *m = ctx->TextureMatrix;
+            const GLfloat *m = ctx->TextureMatrix[texSet];
             GLfloat m0 = m[0],  m4 = m[4],  m8 = m[8],  m12 = m[12];
             GLfloat m1 = m[1],  m5 = m[5],  m9 = m[9],  m13 = m[13];
             GLfloat m2 = m[2],  m6 = m[6],  m10 = m[10],  m14 = m[14];
@@ -488,7 +463,7 @@ static void transform_texcoords( GLcontext *ctx, GLuint n, GLfloat t[][4] )
          break;
       case MATRIX_2D:
          {
-            const GLfloat *m = ctx->TextureMatrix;
+            const GLfloat *m = ctx->TextureMatrix[texSet];
             GLfloat m0 = m[0], m1 = m[1], m4 = m[4], m5 = m[5];
             GLfloat m12 = m[12], m13 = m[13];
             GLuint i;
@@ -503,7 +478,7 @@ static void transform_texcoords( GLcontext *ctx, GLuint n, GLfloat t[][4] )
          break;
       case MATRIX_3D:
          {
-            const GLfloat *m = ctx->TextureMatrix;
+            const GLfloat *m = ctx->TextureMatrix[texSet];
             GLfloat m0 = m[0], m1 = m[1], m2 = m[2], m4 = m[4], m5 = m[5];
             GLfloat m6 = m[6], m8 = m[8], m9 = m[9], m10 = m[10];
             GLfloat m12 = m[12], m13 = m[13], m14 = m[14];
@@ -523,18 +498,18 @@ static void transform_texcoords( GLcontext *ctx, GLuint n, GLfloat t[][4] )
    }
    END_FAST_MATH;
 #else
-   switch (ctx->TextureMatrixType) {
+   switch (ctx->TextureMatrixType[texSet]) {
       case MATRIX_GENERAL:
-         asm_transform_points4_general( n, t, ctx->TextureMatrix, t );
+         asm_transform_points4_general( n, t, ctx->TextureMatrix[texSet], t );
          break;
       case MATRIX_IDENTITY:
          /* Do nothing */
          break;
       case MATRIX_2D:
-         asm_transform_points4_2d( n, t, ctx->TextureMatrix, t );
+         asm_transform_points4_2d( n, t, ctx->TextureMatrix[texSet], t );
          break;
       case MATRIX_3D:
-         asm_transform_points4_3d( n, t, ctx->TextureMatrix, t );
+         asm_transform_points4_3d( n, t, ctx->TextureMatrix[texSet], t );
          break;
       default:
          /* should never get here */
@@ -550,6 +525,13 @@ static void transform_texcoords( GLcontext *ctx, GLuint n, GLfloat t[][4] )
  * Apply the projection matrix to an array of vertices in Eye coordinates
  * resulting in Clip coordinates.  Also, compute the ClipMask bitfield for
  * each vertex.
+ *
+ * NOTE: the volatile keyword is used in this function to ensure that the
+ * FP computations are computed to low-precision.  If high precision is
+ * used (ala 80-bit X86 arithmetic) then the clipMask results may be
+ * inconsistant with the computations in clip.c.  Later, clipped polygons
+ * may be rendered incorrectly.
+ *
  * Input:  ctx - the context
  *         n - number of vertices
  *         vEye - array [n][4] of Eye coordinates
@@ -557,16 +539,17 @@ static void transform_texcoords( GLcontext *ctx, GLuint n, GLfloat t[][4] )
  *          clipMask - array [n] of clip masks
  */
 static void project_and_cliptest( GLcontext *ctx,
-                                  GLuint n, /*const*/ GLfloat vEye[][4],
+                                  GLuint n, CONST GLfloat vEye[][4],
                                   GLfloat vClip[][4], GLubyte clipMask[],
                                   GLubyte *orMask, GLubyte *andMask )
 
 {
-#ifndef USE_ASM
+#ifndef USE_X86_ASM
    GLubyte tmpOrMask = *orMask;
    GLubyte tmpAndMask = *andMask;
 
    START_FAST_MATH;
+
    switch (ctx->ProjectionMatrixType) {
       case MATRIX_GENERAL:
          {
@@ -634,10 +617,10 @@ static void project_and_cliptest( GLcontext *ctx,
             for (i=0;i<n;i++) {
                GLfloat ex = vEye[i][0], ey = vEye[i][1];
                GLfloat ez = vEye[i][2], ew = vEye[i][3];
-               GLfloat cx = m0 * ex                      + m12 * ew;
-               GLfloat cy =           m5 * ey            + m13 * ew;
-               GLfloat cz =                     m10 * ez + m14 * ew;
-               GLfloat cw =                                      ew;
+               volatile GLfloat cx = m0 * ex                      + m12 * ew;
+               volatile GLfloat cy =           m5 * ey            + m13 * ew;
+               volatile GLfloat cz =                     m10 * ez + m14 * ew;
+               volatile GLfloat cw =                                      ew;
                GLubyte mask = 0;
                vClip[i][0] = cx;
                vClip[i][1] = cy;
@@ -666,10 +649,10 @@ static void project_and_cliptest( GLcontext *ctx,
             for (i=0;i<n;i++) {
                GLfloat ex = vEye[i][0], ey = vEye[i][1];
                GLfloat ez = vEye[i][2], ew = vEye[i][3];
-               GLfloat cx = m0 * ex           + m8  * ez           ;
-               GLfloat cy =           m5 * ey + m9  * ez           ;
-               GLfloat cz =                     m10 * ez + m14 * ew;
-               GLfloat cw =                          -ez           ;
+               volatile GLfloat cx = m0 * ex           + m8  * ez           ;
+               volatile GLfloat cy =           m5 * ey + m9  * ez           ;
+               volatile GLfloat cz =                     m10 * ez + m14 * ew;
+               volatile GLfloat cw =                          -ez           ;
                GLubyte mask = 0;
                vClip[i][0] = cx;
                vClip[i][1] = cy;
@@ -723,6 +706,8 @@ static void project_and_cliptest( GLcontext *ctx,
 }
 
 
+/* This value matches the one in clip.c, used to cope with numeric error. */
+#define MAGIC_NUMBER -0.8e-03F
 
 /*
  * Test an array of vertices against the user-defined clipping planes.
@@ -735,7 +720,7 @@ static void project_and_cliptest( GLcontext *ctx,
  *          CLIP_SOME - if some vertices were clipped
  */
 static GLuint userclip_vertices( GLcontext *ctx, GLuint n,
-                                 /*const*/ GLfloat vEye[][4],
+                                 CONST GLfloat vEye[][4],
                                  GLubyte clipMask[] )
 {
    GLboolean anyClipped = GL_FALSE;
@@ -756,7 +741,7 @@ static GLuint userclip_vertices( GLcontext *ctx, GLuint n,
          for (i=0;i<n;i++) {
             GLfloat dot = vEye[i][0] * a + vEye[i][1] * b
                         + vEye[i][2] * c + vEye[i][3] * d;
-            if (dot < 0.0F) {
+            if (dot < MAGIC_NUMBER) {
                /* this vertex is clipped */
                clipMask[i] = CLIP_USER_BIT;
                anyClipped = GL_TRUE;
@@ -778,7 +763,6 @@ static GLuint userclip_vertices( GLcontext *ctx, GLuint n,
 }
 
 
-
 /*
  * Transform an array of vertices from clip coordinate space to window
  * coordinates.
@@ -789,9 +773,10 @@ static GLuint userclip_vertices( GLcontext *ctx, GLuint n,
  * Output:  vWin - array [n] of vertices in window coordinate system
  */
 static void viewport_map_vertices( GLcontext *ctx,
-                                   GLuint n, /*const*/ GLfloat vClip[][4],
+                                   GLuint n, CONST GLfloat vClip[][4],
                                    const GLubyte clipMask[], GLfloat vWin[][3])
 {
+
    GLfloat sx = ctx->Viewport.Sx;
    GLfloat tx = ctx->Viewport.Tx;
    GLfloat sy = ctx->Viewport.Sy;
@@ -799,6 +784,7 @@ static void viewport_map_vertices( GLcontext *ctx,
    GLfloat sz = ctx->Viewport.Sz;
    GLfloat tz = ctx->Viewport.Tz;
 
+
    START_FAST_MATH;
 
    if ((ctx->ProjectionMatrixType==MATRIX_ORTHO || 
@@ -936,27 +922,65 @@ static void update_material( GLcontext *ctx, GLuint i )
 /*
  * Compute the shading (lighting) for the vertices in the vertex buffer.
  */
-static void shade_vertices( GLcontext *ctx )
+static void shade_vertices( GLcontext *ctx, GLuint vbStart, GLuint vbCount )
 {
    struct vertex_buffer *VB = ctx->VB;
 
-   if (ctx->Visual->RGBAflag) {
+   if (ctx->Visual->RGBAflag && ctx->Texture.Enabled
+       && ctx->Light.Model.ColorControl==GL_SEPARATE_SPECULAR_COLOR) {
+      /* Separate specular color */
       if (!VB->MonoMaterial) {
          /* Material may change with each vertex */
          GLuint i;
-         for (i=VB->Start; i<VB->Count; i++) {
+         for (i=vbStart; i<vbCount; i++) {
             update_material( ctx, i );
-            gl_color_shade_vertices( ctx, 0, 1, &VB->Eye[i],
-                                     &VB->Normal[i], &VB->Fcolor[i]);
+            gl_shade_rgba_spec( ctx, 0, 1, &VB->Eye[i],
+                                &VB->Normal[i], &VB->Fcolor[i], &VB->Fspec[i]);
             if (ctx->Light.Model.TwoSide) {
-               gl_color_shade_vertices( ctx, 1, 1, &VB->Eye[i],
-                                        &VB->Normal[i], &VB->Bcolor[i]);
+               gl_shade_rgba_spec( ctx, 1, 1, &VB->Eye[i],
+                                 &VB->Normal[i], &VB->Bcolor[i], &VB->Bspec[i]);
             }
          }
          /* Need this in case a glColor/glMaterial is called after the
           * last vertex between glBegin/glEnd.
           */
-         update_material( ctx, VB->Count );
+         update_material( ctx, vbCount );
+      }
+      else {
+         /* call slower, full-featured shader */
+         gl_shade_rgba_spec( ctx, 0,
+                             vbCount - vbStart,
+                             VB->Eye + vbStart,
+                             VB->Normal + vbStart,
+                             VB->Fcolor + vbStart,
+                             VB->Fspec + vbStart );
+         if (ctx->Light.Model.TwoSide) {
+            gl_shade_rgba_spec( ctx, 1,
+                                vbCount - vbStart,
+                                VB->Eye + vbStart,
+                                VB->Normal + vbStart,
+                                VB->Bcolor + vbStart,
+                                VB->Bspec + vbStart );
+         }
+      }
+   }
+   else if (ctx->Visual->RGBAflag) {
+      if (!VB->MonoMaterial) {
+         /* Material may change with each vertex */
+         GLuint i;
+         for (i=vbStart; i<vbCount; i++) {
+            update_material( ctx, i );
+            gl_shade_rgba( ctx, 0, 1, &VB->Eye[i],
+                           &VB->Normal[i], &VB->Fcolor[i]);
+            if (ctx->Light.Model.TwoSide) {
+               gl_shade_rgba( ctx, 1, 1, &VB->Eye[i],
+                              &VB->Normal[i], &VB->Bcolor[i]);
+            }
+         }
+         /* Need this in case a glColor/glMaterial is called after the
+          * last vertex between glBegin/glEnd.
+          */
+         update_material( ctx, vbCount );
       }
       else {
          if (ctx->Light.Fast) {
@@ -964,19 +988,15 @@ static void shade_vertices( GLcontext *ctx )
                /* call optimized shader */
                GLubyte color[1][4];
                GLuint i;
-               gl_color_shade_vertices_fast( ctx, 0,  /* front side */
-                                             1,
-                                             VB->Normal + VB->Start,
-                                             color );
-               for (i=VB->Start; i<VB->Count; i++) {
+               gl_shade_rgba_fast( ctx, 0,  /* front side */
+                                   1, VB->Normal + vbStart, color );
+               for (i=vbStart; i<vbCount; i++) {
                   COPY_4V( VB->Fcolor[i], color[0] );
                }
                if (ctx->Light.Model.TwoSide) {
-                  gl_color_shade_vertices_fast( ctx, 1,  /* back side */
-                                                1,
-                                                VB->Normal + VB->Start,
-                                                color );
-                  for (i=VB->Start; i<VB->Count; i++) {
+                  gl_shade_rgba_fast( ctx, 1,  /* back side */
+                                      1, VB->Normal + vbStart, color );
+                  for (i=vbStart; i<vbCount; i++) {
                      COPY_4V( VB->Bcolor[i], color[0] );
                   }
                }
@@ -984,31 +1004,31 @@ static void shade_vertices( GLcontext *ctx )
             }
             else {
                /* call optimized shader */
-               gl_color_shade_vertices_fast( ctx, 0,  /* front side */
-                                             VB->Count - VB->Start,
-                                             VB->Normal + VB->Start,
-                                             VB->Fcolor + VB->Start );
+               gl_shade_rgba_fast( ctx, 0,  /* front side */
+                                   vbCount - vbStart,
+                                   VB->Normal + vbStart,
+                                   VB->Fcolor + vbStart );
                if (ctx->Light.Model.TwoSide) {
-                  gl_color_shade_vertices_fast( ctx, 1,  /* back side */
-                                                VB->Count - VB->Start,
-                                                VB->Normal + VB->Start,
-                                                VB->Bcolor + VB->Start );
+                  gl_shade_rgba_fast( ctx, 1,  /* back side */
+                                      vbCount - vbStart,
+                                      VB->Normal + vbStart,
+                                      VB->Bcolor + vbStart );
                }
             }
          }
          else {
             /* call slower, full-featured shader */
-            gl_color_shade_vertices( ctx, 0,
-                                     VB->Count - VB->Start,
-                                     VB->Eye + VB->Start,
-                                     VB->Normal + VB->Start,
-                                     VB->Fcolor + VB->Start );
+            gl_shade_rgba( ctx, 0,
+                           vbCount - vbStart,
+                           VB->Eye + vbStart,
+                           VB->Normal + vbStart,
+                           VB->Fcolor + vbStart );
             if (ctx->Light.Model.TwoSide) {
-               gl_color_shade_vertices( ctx, 1,
-                                        VB->Count - VB->Start,
-                                        VB->Eye + VB->Start,
-                                        VB->Normal + VB->Start,
-                                        VB->Bcolor + VB->Start );
+               gl_shade_rgba( ctx, 1,
+                              vbCount - vbStart,
+                              VB->Eye + vbStart,
+                              VB->Normal + vbStart,
+                              VB->Bcolor + vbStart );
             }
          }
       }
@@ -1021,32 +1041,32 @@ static void shade_vertices( GLcontext *ctx )
          /* NOTE the <= here.  This is needed in case glColor/glMaterial
           * is called after the last glVertex inside a glBegin/glEnd pair.
           */
-         for (i=VB->Start; i<VB->Count; i++) {
+         for (i=vbStart; i<vbCount; i++) {
             update_material( ctx, i );
-            gl_index_shade_vertices( ctx, 0, 1, &VB->Eye[i],
-                                     &VB->Normal[i], &VB->Findex[i] );
+            gl_shade_ci( ctx, 0, 1, &VB->Eye[i],
+                         &VB->Normal[i], &VB->Findex[i] );
             if (ctx->Light.Model.TwoSide) {
-               gl_index_shade_vertices( ctx, 1, 1, &VB->Eye[i],
-                                        &VB->Normal[i], &VB->Bindex[i] );
+               gl_shade_ci( ctx, 1, 1, &VB->Eye[i],
+                            &VB->Normal[i], &VB->Bindex[i] );
             }
          }
          /* Need this in case a glColor/glMaterial is called after the
           * last vertex between glBegin/glEnd.
           */
-         update_material( ctx, VB->Count );
+         update_material( ctx, vbCount );
       }
       else {
-         gl_index_shade_vertices( ctx, 0,
-                                  VB->Count - VB->Start,
-                                  VB->Eye + VB->Start,
-                                  VB->Normal + VB->Start,
-                                  VB->Findex + VB->Start );
+         gl_shade_ci( ctx, 0,
+                      vbCount - vbStart,
+                      VB->Eye + vbStart,
+                      VB->Normal + vbStart,
+                      VB->Findex + vbStart );
          if (ctx->Light.Model.TwoSide) {
-            gl_index_shade_vertices( ctx, 1,
-                                     VB->Count - VB->Start,
-                                     VB->Eye + VB->Start,
-                                     VB->Normal + VB->Start,
-                                     VB->Bindex + VB->Start );
+            gl_shade_ci( ctx, 1,
+                         vbCount - vbStart,
+                         VB->Eye + vbStart,
+                         VB->Normal + vbStart,
+                         VB->Bindex + vbStart );
          }
       }
    }
@@ -1057,35 +1077,353 @@ static void shade_vertices( GLcontext *ctx )
 /*
  * Compute fog for the vertices in the vertex buffer.
  */
-static void fog_vertices( GLcontext *ctx )
+static void fog_vertices( GLcontext *ctx, GLuint vbStart, GLuint vbCount )
 {
    struct vertex_buffer *VB = ctx->VB;
 
    if (ctx->Visual->RGBAflag) {
       /* Fog RGB colors */
-      gl_fog_color_vertices( ctx, VB->Count - VB->Start,
-                             VB->Eye + VB->Start,
-                             VB->Fcolor + VB->Start );
+      gl_fog_rgba_vertices( ctx, vbCount - vbStart,
+                            VB->Eye + vbStart,
+                            VB->Fcolor + vbStart );
       if (ctx->LightTwoSide) {
-         gl_fog_color_vertices( ctx, VB->Count - VB->Start,
-                                VB->Eye + VB->Start,
-                                VB->Bcolor + VB->Start );
+         gl_fog_rgba_vertices( ctx, vbCount - vbStart,
+                               VB->Eye + vbStart,
+                               VB->Bcolor + vbStart );
       }
    }
    else {
       /* Fog color indexes */
-      gl_fog_index_vertices( ctx, VB->Count - VB->Start,
-                             VB->Eye + VB->Start,
-                             VB->Findex + VB->Start );
+      gl_fog_ci_vertices( ctx, vbCount - vbStart,
+                          VB->Eye + vbStart,
+                          VB->Findex + vbStart );
       if (ctx->LightTwoSide) {
-         gl_fog_index_vertices( ctx, VB->Count - VB->Start,
-                                VB->Eye + VB->Start,
-                                VB->Bindex + VB->Start );
+         gl_fog_ci_vertices( ctx, vbCount - vbStart,
+                             VB->Eye + vbStart,
+                             VB->Bindex + vbStart );
       }
    }
 }
 
 
+/*
+ * Transform vertices, compute lighting, clipflags,
+ * fog, texture coords, etc.
+ *
+ * Return the clipOrMask and clipAndMask values for the given VB range
+ */
+static void gl_transform_vb_range( GLcontext *ctx, GLuint vbStart, GLuint vbCount,
+				   GLubyte *clipOrMask, GLubyte *clipAndMask,
+				   GLboolean firstPartToDo)
+{
+  struct vertex_buffer *VB = ctx->VB;
+
+  if (firstPartToDo) {
+    /* Apply the modelview matrix to transform vertexes from Object
+     * to Eye coords.
+     */
+    if (VB->VertexSizeMask==VERTEX4_BIT) {
+      transform_points4( ctx, vbCount - vbStart,
+			 VB->Obj + vbStart, VB->Eye + vbStart );
+    }
+    else {
+      transform_points3( ctx, vbCount - vbStart,
+			 VB->Obj + vbStart, VB->Eye + vbStart );
+    }
+
+    /* Now transform the normal vectors */
+    if (ctx->NeedNormals) {
+      gl_xform_normals_3fv( vbCount - vbStart,
+			    VB->Normal + vbStart, ctx->ModelViewInv,
+			    VB->Normal + vbStart, ctx->Transform.Normalize,
+			    ctx->Transform.RescaleNormals );
+    }
+  }
+
+  /* Test vertices in eye coordinate space against user clipping planes */
+  if (ctx->Transform.AnyClip) {
+    GLuint result = userclip_vertices( ctx, vbCount - vbStart,
+				       VB->Eye + vbStart,
+				       VB->ClipMask + vbStart );
+    if (result==CLIP_ALL) {
+      /* All vertices were outside one of the clip planes! */
+      *clipOrMask = CLIP_ALL_BITS; /* force reset of clipping flags */
+      *clipAndMask = CLIP_ALL_BITS;
+      return;
+    }
+    else if (result==CLIP_SOME) {
+      *clipOrMask = CLIP_USER_BIT;
+    }
+    else {
+      *clipAndMask = 0;
+    }
+  }
+
+  /* Apply the projection matrix to the Eye coordinates, resulting in
+   * Clip coordinates.  Also, compute the ClipMask for each vertex.
+   */
+  project_and_cliptest( ctx, vbCount - vbStart, VB->Eye + vbStart,
+			VB->Clip + vbStart, VB->ClipMask + vbStart,
+			clipOrMask, clipAndMask );
+
+  if (*clipAndMask) {
+    /* All vertices clipped by one plane, all done! */
+    *clipOrMask = CLIP_ALL_BITS; /* force reset of clipping flags */
+    return;
+  }
+
+  /* Lighting */
+  if (ctx->Light.Enabled) {
+    shade_vertices(ctx, vbStart, vbCount);
+  }
+
+  /* Per-vertex fog */
+  if (ctx->Fog.Enabled && ctx->Hint.Fog!=GL_NICEST) {
+    fog_vertices(ctx, vbStart, vbCount);
+  }
+
+  /* Generate/transform texture coords */
+  if (ctx->Texture.Enabled || ctx->RenderMode==GL_FEEDBACK) {
+    GLuint texSet;
+    for (texSet=0; texSet<MAX_TEX_COORD_SETS; texSet++) {
+      if (ctx->Texture.Set[texSet].TexGenEnabled) {
+	gl_texgen( ctx, vbCount - vbStart,
+		   VB->Obj + vbStart,
+		   VB->Eye + vbStart,
+		   VB->Normal + vbStart,
+		   VB->MultiTexCoord[texSet] + vbStart,
+		   texSet );
+      }
+    }
+    for (texSet=0; texSet<MAX_TEX_COORD_SETS; texSet++) {
+      if (ctx->TextureMatrixType[texSet] != MATRIX_IDENTITY) {
+	transform_texcoords( ctx, vbCount - vbStart,
+			     VB->MultiTexCoord[texSet] + vbStart,
+			     texSet );
+      }
+    }
+  }
+  
+  /* Use the viewport parameters to transform vertices from Clip
+   * coordinates to Window coordinates.
+   */
+  if (ctx->DoViewportMapping) {
+     viewport_map_vertices( ctx, vbCount - vbStart, VB->Clip + vbStart,
+                     (*clipOrMask) ? VB->ClipMask + vbStart : (GLubyte*) NULL,
+		     VB->Win + vbStart );
+  }
+
+  /* Device driver rasterization setup.  3Dfx driver, for example. */
+  if (ctx->Driver.RasterSetup) {
+    (*ctx->Driver.RasterSetup)( ctx, vbStart, vbCount );
+  }
+}
+
+
+#ifdef MITS
+
+#include <pthread.h>
+#include <semaphore.h>
+#include <sys/time.h>
+#include <sys/resource.h>
+#include <sched.h>
+#include <unistd.h>
+
+
+/* Flag to indicate first time through */
+static int firsttime=1;
+
+/* Semaphores for the vertex buffer processing threads */
+static sem_t tDone1, tDone2;
+
+pthread_attr_t attr1, attr2;
+pthread_t thread1, thread2;
+
+
+typedef struct {
+
+  GLuint start;
+  GLuint count;
+  GLcontext *ctx;
+  GLboolean firstPartToDo;
+  sem_t *thread_sem;
+  volatile int tsync;
+
+} GLtsched;
+
+
+static  GLtsched set1, set2;
+
+
+/* These are the processing threads for the vertex buffer */
+void *gl_transform_vb_range_scheduler(void *s) 
+{
+  struct vertex_buffer *VB;
+  GLtsched *set=(GLtsched *)s;
+
+  int sval;
+
+  while(1) {
+
+    /* Sleep until main thread wakes us up again */
+    sem_wait( set->thread_sem );
+
+    VB = set->ctx->VB;
+
+    gl_transform_vb_range( set->ctx, set->start, set->count,
+                           &VB->ClipOrMask, &VB->ClipAndMask,
+                           set->firstPartToDo );
+    set->tsync = 1;
+
+  } /* end while */
+
+  return NULL;
+
+} /* end gl_transform_vb_range_scheduler */
+#endif
+
+
+void gl_transform_vb( GLcontext *ctx, GLboolean firstPartToDo, GLboolean allDone )
+{
+  struct vertex_buffer *VB = ctx->VB;
+#ifdef MITS
+  struct sched_param sparam1, sparam2, mparams;
+#endif
+
+#ifdef PROFILE
+  GLdouble t0 = gl_time();
+#endif
+
+  ASSERT( VB->Count>0 );
+
+#ifdef PROFILE
+  ctx->VertexTime += gl_time() - t0;
+  ctx->VertexCount += VB->Count - VB->Start;
+#endif
+  
+  if ((ctx->Texture.Enabled || ctx->RenderMode==GL_FEEDBACK) &&
+      ctx->NewTextureMatrix)
+    gl_analyze_texture_matrix(ctx);
+
+  if (ctx->Driver.RasterSetup && VB->Start) {
+      (*ctx->Driver.RasterSetup)( ctx, 0, VB->Start );
+  }
+
+#ifdef MITS
+
+  if (VB->Count > 72) {
+
+    if( firsttime ) {
+
+      int policy;
+     
+
+      if( !getuid() ) {
+	policy = SCHED_FIFO;
+      }
+      else {
+	policy = SCHED_OTHER;
+      }
+
+
+      mparams.sched_priority = sched_get_priority_max(policy) - 1;
+      sched_setscheduler(0, policy, &mparams);
+
+      sparam1.sched_priority = sched_get_priority_max(policy);
+      sparam2.sched_priority = sched_get_priority_max(policy);
+
+
+      sem_init( &tDone1, 0, 0);
+      sem_init( &tDone2, 0, 0);	    
+
+
+      /* Set up threads with OTHER policy and maximum priority */
+      pthread_attr_init( &attr1 );
+      pthread_attr_setscope( &attr1, PTHREAD_SCOPE_SYSTEM);
+      pthread_attr_setschedpolicy( &attr1, policy );
+      pthread_attr_setdetachstate( &attr1, PTHREAD_CREATE_DETACHED);
+      pthread_attr_setschedparam( &attr1, &sparam1 );
+
+      pthread_attr_init( &attr2 );
+      pthread_attr_setscope( &attr2, PTHREAD_SCOPE_SYSTEM);
+      pthread_attr_setschedpolicy( &attr2, policy );
+      pthread_attr_setdetachstate( &attr2, PTHREAD_CREATE_DETACHED);
+      pthread_attr_setschedparam( &attr2, &sparam2 );
+
+      set1.thread_sem = &tDone1;
+      set2.thread_sem = &tDone2;
+
+      pthread_create( &thread1, &attr1, gl_transform_vb_range_scheduler, &set1);
+      pthread_create( &thread2, &attr2, gl_transform_vb_range_scheduler, &set2); 
+      firsttime = 0;
+
+    } /* end if */
+
+
+    set1.ctx = ctx;
+    set1.start = VB->Start;
+    set1.count = VB->Start + (VB->Count-VB->Start)/2;
+    set1.firstPartToDo = firstPartToDo;
+
+
+    set2.ctx = ctx;
+    set2.start = VB->Start + (VB->Count - VB->Start)/2;
+    set2.count = VB->Count;
+    set2.firstPartToDo = firstPartToDo;
+    
+    set1.tsync = set2.tsync = 0;
+
+
+    /* Wake the vertex buffer processing threads */
+    sem_post( &tDone1 );
+    sem_post( &tDone2 );
+    sched_yield();
+
+
+    /* Spin until both are done */
+    while( !set1.tsync && !set2.tsync);
+
+
+  }
+  else {
+
+
+    gl_transform_vb_range( ctx, VB->Start, VB->Count,
+			 &VB->ClipOrMask, &VB->ClipAndMask,
+			 firstPartToDo );
+
+  } /* end if */
+
+#else
+
+    gl_transform_vb_range( ctx, VB->Start, VB->Count,
+			 &VB->ClipOrMask, &VB->ClipAndMask,
+			 firstPartToDo );
+
+#endif
+
+  if (VB->ClipAndMask) {
+    gl_reset_vb( ctx, allDone );
+    return;
+  }
+
+
+#ifdef PROFILE
+  ctx->VertexTime += gl_time() - t0;
+  ctx->VertexCount += VB->Count - VB->Start;
+#endif
+
+  /*
+   * Now we're ready to rasterize the Vertex Buffer!!!
+   *
+   * If the device driver can't rasterize the vertex buffer then we'll
+   * do it ourselves.
+   */
+  if (!ctx->Driver.RenderVB || !(*ctx->Driver.RenderVB)(ctx,allDone)) {
+    gl_render_vb( ctx, allDone );
+  }
+}
+
 
 /*
  * When the Vertex Buffer is full, this function applies the modelview
@@ -1095,42 +1433,10 @@ static void fog_vertices( GLcontext *ctx )
  */
 void gl_transform_vb_part1( GLcontext *ctx, GLboolean allDone )
 {
-   struct vertex_buffer *VB = ctx->VB;
-#ifdef PROFILE
-   GLdouble t0 = gl_time();
-#endif
-
-   ASSERT( VB->Count>0 );
-
-   /* Apply the modelview matrix to transform vertexes from Object
-    * to Eye coords.
-    */
-   if (VB->VertexSizeMask==VERTEX4_BIT) {
-      transform_points4( ctx, VB->Count - VB->Start,
-                         VB->Obj + VB->Start, VB->Eye + VB->Start );
-   }
-   else {
-      transform_points3( ctx, VB->Count - VB->Start,
-                         VB->Obj + VB->Start, VB->Eye + VB->Start );
-   }
-
-   /* Now transform the normal vectors */
-   if (ctx->NeedNormals) {
-      gl_xform_normals_3fv( VB->Count - VB->Start,
-                            VB->Normal + VB->Start, ctx->ModelViewInv,
-                            VB->Normal + VB->Start, ctx->Transform.Normalize );
-   }
-
-#ifdef PROFILE
-   ctx->VertexTime += gl_time() - t0;
-#endif
-
-   /* lighting, project, etc */
-   gl_transform_vb_part2( ctx, allDone );
+  gl_transform_vb( ctx, GL_TRUE, allDone );
 }
 
 
-
 /*
  * Part 2 of Vertex Buffer transformation:  compute lighting, clipflags,
  * fog, texture coords, etc.
@@ -1140,97 +1446,6 @@ void gl_transform_vb_part1( GLcontext *ctx, GLboolean allDone )
  */
 void gl_transform_vb_part2( GLcontext *ctx, GLboolean allDone )
 {
-   struct vertex_buffer *VB = ctx->VB;
-#ifdef PROFILE
-   GLdouble t0 = gl_time();
-#endif
-
-   ASSERT( VB->Count>0 );
-
-   /* Test vertices in eye coordinate space against user clipping planes */
-   if (ctx->Transform.AnyClip) {
-      GLuint result = userclip_vertices( ctx, VB->Count - VB->Start,
-                                         VB->Eye + VB->Start,
-                                         VB->ClipMask + VB->Start );
-      if (result==CLIP_ALL) {
-         /* All vertices were outside one of the clip planes! */
-         gl_reset_vb( ctx, allDone );
-         return;
-      }
-      else if (result==CLIP_SOME) {
-         VB->ClipOrMask = CLIP_USER_BIT;
-      }
-      else {
-         VB->ClipAndMask = 0;
-      }
-   }
-
-   /* Apply the projection matrix to the Eye coordinates, resulting in
-    * Clip coordinates.  Also, compute the ClipMask for each vertex.
-    */
-   project_and_cliptest( ctx, VB->Count - VB->Start, VB->Eye + VB->Start,
-                         VB->Clip + VB->Start, VB->ClipMask + VB->Start,
-                         &VB->ClipOrMask, &VB->ClipAndMask );
-
-   if (VB->ClipAndMask) {
-      /* All vertices clipped by one plane, all done! */
-      gl_reset_vb( ctx, allDone );
-      return;
-   }
-
-   /* Lighting */
-   if (ctx->Light.Enabled) {
-      shade_vertices(ctx);
-   }
-
-   /* Per-vertex fog */
-   if (ctx->Fog.Enabled && ctx->Hint.Fog!=GL_NICEST) {
-      fog_vertices(ctx);
-   }
-
-   /* Generate/transform texture coords */
-   if (ctx->Texture.Enabled || ctx->RenderMode==GL_FEEDBACK) {
-      if (ctx->Texture.TexGenEnabled) {
-         gl_texgen( ctx, VB->Count - VB->Start,
-                    VB->Obj + VB->Start,
-                    VB->Eye + VB->Start,
-                    VB->Normal + VB->Start,
-                    VB->TexCoord + VB->Start );
-      }
-      if (ctx->NewTextureMatrix) {
-         gl_analyze_texture_matrix(ctx);
-      }
-      if (ctx->TextureMatrixType!=MATRIX_IDENTITY) {
-         transform_texcoords( ctx, VB->Count - VB->Start,
-                              VB->TexCoord + VB->Start );
-      }
-   }
-
-   /* Use the viewport parameters to transform vertices from Clip
-    * coordinates to Window coordinates.
-    */
-   viewport_map_vertices( ctx, VB->Count - VB->Start, VB->Clip + VB->Start,
-                          VB->ClipOrMask ? VB->ClipMask + VB->Start : NULL,
-                          VB->Win + VB->Start );
-
-   /* Device driver rasterization setup.  3Dfx driver, for example. */
-   if (ctx->Driver.RasterSetup) {
-      (*ctx->Driver.RasterSetup)( ctx, 0, VB->Count );
-   }
-
-
-#ifdef PROFILE
-   ctx->VertexTime += gl_time() - t0;
-   ctx->VertexCount += VB->Count - VB->Start;
-#endif
-
-   /*
-    * Now we're ready to rasterize the Vertex Buffer!!!
-    *
-    * If the device driver can't rasterize the vertex buffer then we'll
-    * do it ourselves.
-    */
-   if (!ctx->Driver.RenderVB || !(*ctx->Driver.RenderVB)(ctx,allDone)) {
-      gl_render_vb( ctx, allDone );
-   }
+  gl_transform_vb( ctx, GL_FALSE, allDone );
 }
+
